@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -346,6 +346,15 @@ namespace ipe {
     inline static Attribute ARROW_ARC() { return Attribute(ESymbolic + 8); }
     //! Create symbolic attribute "arrow/farc(spx)"
     inline static Attribute ARROW_FARC() { return Attribute(ESymbolic + 9); }
+    //! Create symbolic attribute "arrow/ptarc(spx)"
+    inline static Attribute ARROW_PTARC() { return Attribute(ESymbolic + 10); }
+    //! Create symbolic attribute "arrow/fptarc(spx)"
+    inline static Attribute ARROW_FPTARC() { return Attribute(ESymbolic + 11); }
+
+    //! Is it one of the symbolic attributes "arrow/*arc(spc)"?
+    inline bool isArcArrow() const {
+      return ESymbolic + 8 <= iName && iName <= ESymbolic + 11;
+    }
 
     static Attribute makeColor(String str, Attribute deflt);
     static Attribute makeScalar(String str, Attribute deflt);
