@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -43,12 +43,18 @@
 
 typedef unsigned char uchar;
 
-#if defined(__MINGW32__) || defined(__APPLE__)
+#if defined(WIN32) || defined(__APPLE__)
 typedef unsigned short ushort;
 typedef unsigned int uint;
 typedef unsigned long int ulong;
 #endif
 
+#ifdef WIN32
+#define IPESEP ('\\')
+#else
+#define IPESEP ('/')
+#endif
+
 #undef assert
 #define assert(e) ((e) ? (void)0 : ipeAssertionFailed(__FILE__, __LINE__, #e))
 extern void ipeAssertionFailed(const char *, int, const char *);
@@ -74,14 +80,14 @@ namespace ipe {
 
   //! Ipelib version.
   /*! \ingroup base */
-  const int IPELIB_VERSION = 70104;
+  const int IPELIB_VERSION = 70110;
 
   //! Oldest readable file format version.
   /*! \ingroup base */
   const int OLDEST_FILE_FORMAT = 70000;
   //! Current file format version.
   /*! \ingroup base */
-  const int FILE_FORMAT = 70005;
+  const int FILE_FORMAT = 70107;
 
   // --------------------------------------------------------------------
 
@@ -336,19 +342,31 @@ namespace ipe {
   public:
     typedef void (*DebugHandler)(const char *);
 
+#ifdef IPEBUNDLE
+    static String ipeDir(const char *suffix, const char *fname = 0);
+#endif
 #ifdef WIN32
-    static String ipeDir(const char *suffix);
+    static String winConv(const String &utf8, const String &encoding);
+    static String wideToUtf8(const wchar_t *wbuf);
+    static void utf8ToWide(const char *utf8, std::vector<wchar_t> &wbuf);
+    static FILE *fopen(const char *fname, const char *mode);
+    static int mkdir(const char *dname);
+#else
+    static FILE *fopen(const char *fname, const char *mode) {
+      return ::fopen(fname, mode);
+    }
 #endif
     static int libVersion();
     static void initLib(int version);
     static void setDebug(bool debug);
-    static char pathSeparator();
     static String currentDirectory();
     static String latexDirectory();
     static String fontmapFile();
     static bool fileExists(String fname);
+    static bool listDirectory(String path, std::vector<String> &files);
     static String readFile(String fname);
     static int runPdfLatex(String dir);
+    static double toDouble(String s);
   };
 
   // --------------------------------------------------------------------
