@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -33,6 +33,7 @@
 #define IPEBITMAP_H
 
 #include "ipebase.h"
+#include "ipegeo.h"
 #include "ipexml.h"
 
 // --------------------------------------------------------------------
@@ -90,6 +91,16 @@ namespace ipe {
     inline bool operator!=(const Bitmap &rhs) const;
     inline bool operator<(const Bitmap &rhs) const;
 
+    static const char *readJpegInfo(FILE *file, int &width, int &height,
+				    Vector &dotsPerInch,
+				    TColorSpace &colorSpace,
+				    int &bitsPerComponent);
+
+    static Bitmap readJpeg(const char *fname, Vector &dotsPerInch,
+			   const char * &errmsg);
+    static Bitmap readPNG(const char *fname, bool deflate,
+			  Vector &dotsPerInch, const char * &errmsg);
+
   private:
     int init(const XmlAttributes &attr);
     void computeChecksum();
