@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -78,7 +78,6 @@ namespace ipe {
       EExport = 1,     //!< Don't include Ipe markup
       ENoZip = 2,      //!< Do not compress streams
       EMarkedView = 4, //!< Create marked views only
-      ENoColor = 8,    //!< No color commands in EPS output
     };
 
     //! Errors that can happen while loading documents
@@ -86,6 +85,7 @@ namespace ipe {
       EVersionTooOld = -1,    //!< The version of the file is too old.
       EVersionTooRecent = -2, //!< The file version is newer than this Ipelib.
       EFileOpenError = -3,    //!< Error opening the file
+      ENotAnIpeFile = -4,     //!< The file was not created by Ipe.
     };
 
     Document();
