@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -82,8 +82,7 @@ namespace ipe {
     //! Return transformation matrix.
     inline const Matrix &matrix() const { return iMatrix; }
 
-    virtual bool setAttribute(Property prop, Attribute value,
-			      Attribute stroke, Attribute fill);
+    virtual bool setAttribute(Property prop, Attribute value);
     virtual Attribute getAttribute(Property prop);
 
     //! Save the object in XML format.
@@ -116,17 +115,15 @@ namespace ipe {
     virtual void addToBBox(Rect &box, const Matrix &m, bool cp)
       const = 0;
 
-    //! Compute possible vertex snapping position for transformed object.
-    /*! Looks only for positions closer than \a bound.
-      If successful, modify \a pos and \a bound. */
     virtual void snapVtx(const Vector &mouse, const Matrix &m,
-			 Vector &pos, double &bound) const = 0;
-
-    virtual void checkStyle(const Cascade *sheet, AttributeSeq &seq) const;
-
+			 Vector &pos, double &bound) const;
+    virtual void snapCtl(const Vector &mouse, const Matrix &m,
+			 Vector &pos, double &bound) const;
     virtual void snapBnd(const Vector &mouse, const Matrix &m,
 			 Vector &pos, double &bound) const;
 
+    virtual void checkStyle(const Cascade *sheet, AttributeSeq &seq) const;
+
 
   protected:
     explicit Object();
