@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -102,8 +102,6 @@ namespace ipe {
 
     void setVisible(int view, String layer, bool vis);
 
-    Rect textBox(const Cascade *sheet) const;
-
     //! Return title of this page.
     String title() const;
     void setTitle(String title);
@@ -147,6 +145,7 @@ namespace ipe {
     void transform(int i, const Matrix &m);
     double distance(int i, const Vector &v, double bound) const;
     void snapVtx(int i, const Vector &mouse, Vector &pos, double &bound) const;
+    void snapCtl(int i, const Vector &mouse, Vector &pos, double &bound) const;
     void snapBnd(int i, const Vector &mouse, Vector &pos, double &bound) const;
     void invalidateBBox(int i) const;
 
@@ -154,8 +153,7 @@ namespace ipe {
     void append(TSelect sel, int layer, Object *obj);
     void remove(int i);
     void replace(int i, Object *obj);
-    bool setAttribute(int i, Property prop, Attribute value,
-		      Attribute stroke, Attribute fill);
+    bool setAttribute(int i, Property prop, Attribute value);
 
     int primarySelection() const;
     bool hasSelection() const;
