@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -90,14 +90,15 @@ namespace ipe {
 			    double bound) const;
     virtual void snapVtx(const Vector &mouse, const Matrix &m,
 			 Vector &pos, double &bound) const;
+    virtual void snapCtl(const Vector &mouse, const Matrix &m,
+			 Vector &pos, double &bound) const;
     virtual void snapBnd(const Vector &mouse, const Matrix &m,
 			 Vector &pos, double &bound) const;
 
     virtual void checkStyle(const Cascade *sheet,
 			    AttributeSeq &seq) const;
 
-    virtual bool setAttribute(Property prop, Attribute value,
-			      Attribute stroke, Attribute fill);
+    virtual bool setAttribute(Property prop, Attribute value);
     virtual Attribute getAttribute(Property prop);
 
     inline bool arrow() const;
