@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -90,8 +90,7 @@ namespace ipe {
     //! Return position of symbol on page.
     inline Vector position() const { return iPos; }
 
-    virtual bool setAttribute(Property prop, Attribute value,
-			      Attribute stroke, Attribute fill);
+    virtual bool setAttribute(Property prop, Attribute value);
     virtual Attribute getAttribute(Property prop);
 
     inline uint flags() const { return iFlags; }
