@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -46,7 +46,7 @@ namespace ipe {
 
   class CurveSegment {
   public:
-    enum Type { EArc, ESegment, EQuad, EBezier, ESpline };
+    enum Type { EArc, ESegment, ESpline, EOldSpline };
 
     //! Type of segment.
     inline Type type() const { return iType; }
@@ -59,7 +59,6 @@ namespace ipe {
     //! Matrix (if Type() == EArc).
     inline Matrix matrix() const { return *iM; }
 
-    Bezier bezier() const;
     Arc arc() const;
     void beziers(std::vector<Bezier> &bez) const;
 
@@ -67,12 +66,12 @@ namespace ipe {
     void addToBBox(Rect &box, const Matrix &m, bool cp) const;
     double distance(const Vector &v, const Matrix &m, double bound) const;
     void snapVtx(const Vector &mouse, const Matrix &m,
-		 Vector &pos, double &bound) const;
+		 Vector &pos, double &bound, bool cp) const;
     void snapBnd(const Vector &mouse, const Matrix &m,
 		 Vector &pos, double &bound) const;
   private:
     CurveSegment(Type type, int num, const Vector *cp,
-		const Matrix *m = 0);
+		 const Matrix *m = 0);
   private:
     Type iType;
     const Vector *iCP;
@@ -109,7 +108,7 @@ namespace ipe {
 			    double bound) const = 0;
     //! Snap to vertex.
     virtual void snapVtx(const Vector &mouse, const Matrix &m,
-			 Vector &pos, double &bound) const = 0;
+			 Vector &pos, double &bound, bool cp) const = 0;
     //! Snap to boundary of subpath.
     virtual void snapBnd(const Vector &mouse, const Matrix &m,
 			 Vector &pos, double &bound) const = 0;
@@ -129,7 +128,7 @@ namespace ipe {
     virtual double distance(const Vector &v, const Matrix &m,
 			    double bound) const;
     virtual void snapVtx(const Vector &mouse, const Matrix &m,
-			 Vector &pos, double &bound) const;
+			 Vector &pos, double &bound, bool cp) const;
     virtual void snapBnd(const Vector &mouse, const Matrix &m,
 			 Vector &pos, double &bound) const;
   private:
@@ -148,7 +147,7 @@ namespace ipe {
     virtual double distance(const Vector &v, const Matrix &m,
 			    double bound) const;
     virtual void snapVtx(const Vector &mouse, const Matrix &m,
-			 Vector &pos, double &bound) const;
+			 Vector &pos, double &bound, bool cp) const;
     virtual void snapBnd(const Vector &mouse, const Matrix &m,
 			 Vector &pos, double &bound) const;
   public:
@@ -167,7 +166,7 @@ namespace ipe {
     virtual double distance(const Vector &v, const Matrix &m,
 			    double bound) const;
     virtual void snapVtx(const Vector &mouse, const Matrix &m,
-			 Vector &pos, double &bound) const;
+			 Vector &pos, double &bound, bool cp) const;
     virtual void snapBnd(const Vector &mouse, const Matrix &m,
 			 Vector &pos, double &bound) const;
 
@@ -179,12 +178,15 @@ namespace ipe {
 
     void appendSegment(const Vector &v0, const Vector &v1);
     void appendArc(const Matrix &m, const Vector &v0, const Vector &v1);
-    void appendQuad(const Vector &v0, const Vector &v1, const Vector &v2);
-    void appendBezier(const Vector &v0, const Vector &v1,
-		      const Vector &v2, const Vector &v3);
-    void appendSpline(const std::vector<Vector> &v);
+    void appendSpline(const std::vector<Vector> &v) {
+      appendSpline(v, CurveSegment::ESpline); }
+    void appendOldSpline(const std::vector<Vector> &v) {
+      appendSpline(v, CurveSegment::EOldSpline); }
     void setClosed(bool closed);
 
+  private:
+    void appendSpline(const std::vector<Vector> &v, CurveSegment::Type type);
+
   private:
     struct Seg {
       CurveSegment::Type iType;
@@ -216,7 +218,7 @@ namespace ipe {
     void addToBBox(Rect &box, const Matrix &m, bool cp) const;
     double distance(const Vector &v, const Matrix &m, double bound) const;
     void snapVtx(const Vector &mouse, const Matrix &m,
-		 Vector &pos, double &bound) const;
+		 Vector &pos, double &bound, bool cp) const;
     void snapBnd(const Vector &mouse, const Matrix &m,
 		 Vector &pos, double &bound) const;
 
