@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -33,6 +33,7 @@
 #define IPESNAP_H
 
 #include "ipegeo.h"
+#include "ipetoolbase.h"
 
 // --------------------------------------------------------------------
 
@@ -44,9 +45,10 @@ namespace ipe {
   public:
     //! The different snap modes as bitmasks.
     enum TSnapModes { ESnapNone = 0,
-		      ESnapVtx = 1, ESnapBd = 2, ESnapInt = 4,
-		      ESnapGrid = 8, ESnapAngle = 0x10,
-		      ESnapAuto = 0x20 };
+		      ESnapVtx = 1, ESnapCtl = 2,
+		      ESnapBd = 4, ESnapInt = 8,
+		      ESnapGrid = 0x10, ESnapAngle = 0x20,
+		      ESnapAuto = 0x40 };
 
     int iSnap;           //!< Activated snapping modes (TSnapModes)
     bool iGridVisible;   //!< Is the grid visible?
@@ -57,15 +59,15 @@ namespace ipe {
     Vector iOrigin;      //!< Origin of coordinate system
     Angle iDir;          //!< Direction of x-axis
 
-    bool intersectionSnap(Vector &pos, const Page *page,
-			  double snapDist) const;
+    void intersectionSnap(const Vector &pos,  Vector &fifi,
+			  const Page *page, double &snapDist) const;
     bool snapAngularIntersection(Vector &pos, const Line &l,
 				 const Page *page,
 				 double snapDist) const;
-    bool simpleSnap(Vector &pos, const Page *page,
-		    double snapDist) const;
-    bool snap(Vector &pos, const Page *page, double snapDist,
-	      Vector *autoOrg = 0) const;
+    TSnapModes simpleSnap(Vector &pos, const Page *page,
+			  double snapDist, Tool *tool = 0) const;
+    TSnapModes snap(Vector &pos, const Page *page, double snapDist,
+		    Tool *tool = 0, Vector *autoOrg = 0) const;
     Line getLine(const Vector &mouse, const Vector &base) const;
     bool setEdge(const Vector &pos, const Page *page);
   };
