@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -104,11 +104,15 @@ void AppUiBase::buildMenus()
   addItem(EFileMenu, "Save", "save");
   addItem(EFileMenu, "Save as", "save_as");
   addItem(EFileMenu);
+  addItem(EFileMenu, "Export as PNG", "export_png");
+  addItem(EFileMenu, "Export as EPS", "export_eps");
+  addItem(EFileMenu);
+  addItem(EFileMenu, "Insert image", "insert_image");
+  addItem(EFileMenu);
   addItem(EFileMenu, "Automatically run Latex", "*auto_latex");
   addItem(EFileMenu, "Run Latex", "run_latex");
   addItem(EFileMenu);
   addItem(EFileMenu, "Close", "close");
-  addItem(EFileMenu, "Exit", "quit");
 
   addItem(EEditMenu, "Undo", "undo");
   addItem(EEditMenu, "Redo", "redo");
@@ -186,7 +190,6 @@ void AppUiBase::buildMenus()
   addItem(EModeMenu, "Pan the canvas", "mode_pan");
   addItem(EModeMenu, "Shred objects", "mode_shredder");
   addItem(EModeMenu);
-  // TODO iObjectTools->addSeparator();
   addItem(EModeMenu, "Text labels", "mode_label");
   addItem(EModeMenu, "Mathematical symbols", "mode_math");
   addItem(EModeMenu, "Paragraphs", "mode_paragraph");
@@ -206,7 +209,14 @@ void AppUiBase::buildMenus()
   addItem(EModeMenu, "Circles (by 3 points)", "mode_circle3");
   addItem(EModeMenu, "Ink", "mode_ink");
 
+  // @ means the action can be used while drawing
+  // * means the action is checkable (on/off)
+  // Checkable actions work differently in Qt and Win32:
+  //   Qt already toggles the state
+  //   In Win32 the action needs to toggle the state.
+
   addItem(ESnapMenu, "Snap to vertex", "@snapvtx");
+  addItem(ESnapMenu, "Snap to control point", "@snapctl");
   addItem(ESnapMenu, "Snap to boundary", "@snapbd");
   addItem(ESnapMenu, "Snap to intersection", "@snapint");
   addItem(ESnapMenu, "Snap to grid", "@snapgrid");
@@ -220,10 +230,9 @@ void AppUiBase::buildMenus()
   addItem(ESnapMenu, "Set line", "@set_line");
   addItem(ESnapMenu, "Set line && snap", "@set_line_snap");
 
-#ifndef IPEUI_WIN32
   addItem(EZoomMenu, "Fullscreen", "@*fullscreen");
-#endif
   addItem(EZoomMenu, "Grid visible", "@*grid_visible");
+  addItem(EZoomMenu, "Pretty display", "@*pretty_display");
 
   startSubMenu(EZoomMenu, "Coordinates");
   addSubItem("points", "@coordinates|points");
@@ -298,14 +307,26 @@ void AppUiBase::buildMenus()
   addItem(EPageMenu, "Page sorter", "page_sorter");
   addItem(EPageMenu);
 #ifdef IPEUI_WIN32
-  addItem(EPageMenu, "Notes", "toggle_notes");
-  addItem(EPageMenu, "Bookmarks", "toggle_bookmarks");
+  // In Qt these are created using "toggleViewAction()"
+  addItem(EPageMenu, "Notes", "@*toggle_notes");
+  addItem(EPageMenu, "Bookmarks", "@*toggle_bookmarks");
 #endif
 
   addItem(EHelpMenu, "Ipe &manual", "manual");
   addItem(EHelpMenu, "Onscreen keyboard", "@keyboard");
   addItem(EHelpMenu, "Show &configuration", "show_configuration");
-  addItem(EHelpMenu, "About the &ipelets", "about_ipelets");
+  addItem(EHelpMenu, "&Ipelet information", "about_ipelets");
+
+  lua_getglobal(L, "prefs");
+  lua_getfield(L, -1, "developer");
+  if (lua_toboolean(L, -1)) {
+    startSubMenu(EHelpMenu, "Developer");
+    addSubItem("Reload ipelets", "developer_reload_ipelets");
+    addSubItem("List shortcuts", "developer_list_shortcuts");
+    endSubMenu();
+  }
+  lua_pop(L, 2); // developer, prefs
+
   addItem(EHelpMenu, "&About Ipe", "about");
 
   // build Ipelet menu
@@ -314,6 +335,10 @@ void AppUiBase::buildMenus()
   for (int i = 1; i <= n; ++i) {
     lua_rawgeti(L, -1, i);
     lua_getfield(L, -1, "label");
+    if (!lua_isstring(L, -1)) {
+      lua_pop(L, 2); // label, ipelet
+      continue;
+    }
     String label(lua_tostring(L, -1));
     lua_pop(L, 1);
     lua_getfield(L, -1, "name");
@@ -350,8 +375,11 @@ void AppUiBase::canvasObserverWheelMoved(int degrees)
 {
   if (degrees > 0)
     action("wheel_zoom_in");
-  else
+  else if (degrees < 0)
     action("wheel_zoom_out");
+  else
+    // result of a zoom gesture
+    setZoom(iCanvas->zoom());
 }
 
 void AppUiBase::canvasObserverToolChanged(bool hasTool)
@@ -392,7 +420,7 @@ void AppUiBase::canvasObserverPositionChanged()
   adjust(v.x, iMouseIn, iMouseFactor);
   adjust(v.y, iMouseIn, iMouseFactor);
   const char *units = mouse_units[iMouseIn];
-  char s[32];
+  char s[256];
   sprintf(s, iCoordinatesFormat.z(), v.x, units, v.y, units);
   setMouseIndicator(s);
 }
@@ -566,17 +594,11 @@ void AppUiBase::setupSymbolicNames(const Cascade *sheet)
   resetCombos();
   for (int i = 0; i < EUiView; ++i)
     iComboContents[i].clear();
-  AttributeSeq seq;
+  AttributeSeq seq, absColor;
   sheet->allNames(EColor, seq);
-  for (uint i = 0; i < seq.size(); ++i) {
-    Attribute abs = sheet->find(EColor, seq[i]);
-    Color color = abs.color();
-    String s = seq[i].string();
-    addComboColor(EUiStroke, s, color);
-    addComboColor(EUiFill, s, color);
-    iComboContents[EUiStroke].push_back(s);
-    iComboContents[EUiFill].push_back(s);
-  }
+  for (uint i = 0; i < seq.size(); ++i)
+    absColor.push_back(sheet->find(EColor, seq[i]));
+  addComboColors(seq, absColor);
   showInCombo(sheet, EPen, EUiPen);
   showInCombo(sheet, ETextSize, EUiTextSize);
   showInCombo(sheet, ESymbolSize, EUiSymbolSize);
@@ -589,6 +611,8 @@ void AppUiBase::setGridAngleSize(Attribute abs_grid, Attribute abs_angle)
 {
   AttributeSeq seq;
   iCascade->allNames(EGridSize, seq);
+  if (!seq.size())
+    setComboCurrent(EUiGridSize, 0);
   for (uint i = 0; i < seq.size(); ++i) {
     if (iCascade->find(EGridSize, seq[i]) == abs_grid) {
       setComboCurrent(EUiGridSize, i);
@@ -597,6 +621,8 @@ void AppUiBase::setGridAngleSize(Attribute abs_grid, Attribute abs_angle)
   }
   seq.clear();
   iCascade->allNames(EAngleSize, seq);
+  if (!seq.size())
+    setComboCurrent(EUiAngleSize, 0);
   for (uint i = 0; i < seq.size(); ++i) {
     if (iCascade->find(EAngleSize, seq[i]) == abs_angle) {
       setComboCurrent(EUiAngleSize, i);
@@ -631,7 +657,6 @@ void AppUiBase::setAttributes(const AllAttributes &all, Cascade *sheet)
   Color fill = iCascade->find(EColor, iAll.iFill).color();
   setButtonColor(EUiStroke, stroke);
   setButtonColor(EUiFill, fill);
-  // iButton[]->setIcon(prefsColorIcon(stroke));
   setAttribute(EUiPen, iAll.iPen);
   setAttribute(EUiTextSize, iAll.iTextSize);
   setAttribute(EUiSymbolSize, iAll.iSymbolSize);
