@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -45,8 +45,6 @@ typedef GtkWidget *WINID;
 typedef GtkMenu *MENUHANDLE;
 #endif
 #ifdef IPEUI_WIN32
-#define _WIN32_WINNT 0x0501
-#define _WIN32_IE 0x0501
 #include <windows.h>
 #include <commctrl.h>
 typedef HWND WINID;
@@ -90,6 +88,7 @@ public:
 public:  // What platforms must implement:
   virtual WINID windowId() = 0;
   virtual int actionId(const char *name) const = 0;
+  virtual int actionAlwaysOn(const char *name) const = 0;
   virtual void closeWindow() = 0;
   virtual bool actionState(const char *name) = 0;
   virtual void setActionState(const char *name, bool value) = 0;
@@ -105,6 +104,10 @@ public:  // What platforms must implement:
   virtual void setMouseIndicator(const char *s) = 0;
   virtual void setBookmarks(int no, const String *s) = 0;
   virtual void setToolVisible(int m, bool vis) = 0;
+  virtual int pageSorter(lua_State *L, Document *doc,
+			 int width, int height, int thumbWidth) = 0;
+  virtual int clipboard(lua_State *L) = 0;
+  virtual int setClipboard(lua_State *L) = 0;
 
 protected:   // What platforms must implement:
   virtual void addRootMenu(int id, const char *name) = 0;
@@ -116,7 +119,7 @@ protected:   // What platforms must implement:
   virtual MENUHANDLE endSubMenu() = 0;
   virtual void addCombo(int sel, String s) = 0;
   virtual void resetCombos() = 0;
-  virtual void addComboColor(int sel, String s, Color color) = 0;
+  virtual void addComboColors(AttributeSeq &sym, AttributeSeq &abs) = 0;
   virtual void setComboCurrent(int sel, int idx) = 0;
   virtual void setPathView(const AllAttributes &all, Cascade *sheet) = 0;
   virtual void setCheckMark(String name, Attribute a) = 0;
