@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -50,6 +50,8 @@
 #include <QMessageBox>
 #include <QSignalMapper>
 #include <QToolTip>
+#include <QDialogButtonBox>
+#include <QClipboard>
 
 using namespace ipe;
 using namespace ipelua;
@@ -122,6 +124,12 @@ int AppUi::actionId(const char *name) const
   return 0;
 }
 
+// Not used
+int AppUi::actionAlwaysOn(const char *name) const
+{
+  return 0;
+}
+
 void AppUi::addItem(QMenu *m, const QString &title, const char *name)
 {
   // bool canUseWhileDrawing = false;
@@ -187,7 +195,6 @@ void AppUi::addEdit(const char *name)
 {
   QAction *a = findAction(name);
   assert(a);
-  // a->setCheckable(true);
   iEditTools->addAction(a);
 }
 
@@ -225,15 +232,43 @@ MENUHANDLE AppUi::endSubMenu()
   return submenu;
 }
 
+Qt::DockWidgetArea getDockSide(lua_State *L, const char *name,
+			       Qt::DockWidgetArea deflt)
+{
+  Qt::DockWidgetArea r = deflt;
+  lua_getglobal(L, "prefs");
+  lua_getfield(L, -1, "docks");
+  if (lua_istable(L, -1)) {
+    lua_getfield(L, -1, name);
+    const char *s = lua_tolstring(L, -1, 0);
+    if (!strcmp(s, "left"))
+      r = Qt::LeftDockWidgetArea;
+    else if (!strcmp(s, "right"))
+      r = Qt::RightDockWidgetArea;
+    lua_pop(L, 1); // left or right
+  }
+  lua_pop(L, 2); // prefs, docks
+  return r;
+}
+
 // --------------------------------------------------------------------
 
+extern void qt_set_sequence_auto_mnemonic(bool b);
+
+#if QT_VERSION < 0x050000
 AppUi::AppUi(lua_State *L0, int model, Qt::WFlags f)
+#else
+AppUi::AppUi(lua_State *L0, int model, Qt::WindowFlags f)
+#endif
   : QMainWindow(0, f), AppUiBase(L0, model)
 {
   setWindowIcon(prefsIcon("ipe"));
   QMainWindow::setAttribute(Qt::WA_DeleteOnClose);
   setDockOptions(AnimatedDocks); // do not allow stacking properties and layers
 
+  // enable automatic shortcuts on Mac OS X
+  qt_set_sequence_auto_mnemonic(true);
+
   Canvas *canvas = new Canvas(this);
   iCanvas = canvas;
   setCentralWidget(canvas);
@@ -288,6 +323,7 @@ AppUi::AppUi(lua_State *L0, int model, Qt::WFlags f)
   comboMap->setMapping(iSelector[EUiAngleSize], EUiAngleSize);
 
   addSnap("snapvtx");
+  addSnap("snapctl");
   addSnap("snapbd");
   addSnap("snapint");
   addSnap("snapgrid");
@@ -313,29 +349,36 @@ AppUi::AppUi(lua_State *L0, int model, Qt::WFlags f)
   iShiftKey->setIcon(prefsIcon("shift_key"));
   iEditTools->addAction(iShiftKey);
   iEditTools->addAction(findAction("grid_visible"));
+  iAbortButton = new QAction("stop", this);
+  iAbortButton->setIcon(prefsIcon("stop"));
+  iEditTools->addAction(iAbortButton);
   connect(iShiftKey, SIGNAL(triggered()), SLOT(toolbarModifiersChanged()));
+  connect(iAbortButton, SIGNAL(triggered()), SLOT(abortDrawing()));
 
   iPropertiesTools = new QDockWidget("Properties", this);
-  addDockWidget(Qt::LeftDockWidgetArea, iPropertiesTools);
+  addDockWidget(getDockSide(L, "properties", Qt::LeftDockWidgetArea),
+		iPropertiesTools);
   iPropertiesTools->setAllowedAreas(Qt::LeftDockWidgetArea|
 				    Qt::RightDockWidgetArea);
 
-  iLayerTools = new QDockWidget("Layers", this);
-  addDockWidget(Qt::LeftDockWidgetArea, iLayerTools);
-  iLayerTools->setAllowedAreas(Qt::LeftDockWidgetArea|Qt::RightDockWidgetArea);
+  iBookmarkTools = new QDockWidget("Bookmarks", this);
+  addDockWidget(getDockSide(L, "bookmarks", Qt::RightDockWidgetArea),
+		iBookmarkTools);
+  iBookmarkTools->setAllowedAreas(Qt::LeftDockWidgetArea|
+				  Qt::RightDockWidgetArea);
+  iMenu[EPageMenu]->addAction(iBookmarkTools->toggleViewAction());
 
   iNotesTools = new QDockWidget("Notes", this);
-  addDockWidget(Qt::RightDockWidgetArea, iNotesTools);
+  addDockWidget(getDockSide(L, "notes", Qt::RightDockWidgetArea),
+		iNotesTools);
   iNotesTools->setAllowedAreas(Qt::LeftDockWidgetArea|
 			       Qt::RightDockWidgetArea);
   iMenu[EPageMenu]->addAction(iNotesTools->toggleViewAction());
 
-
-  iBookmarkTools = new QDockWidget("Bookmarks", this);
-  addDockWidget(Qt::RightDockWidgetArea, iBookmarkTools);
-  iBookmarkTools->setAllowedAreas(Qt::LeftDockWidgetArea|
-				  Qt::RightDockWidgetArea);
-  iMenu[EPageMenu]->addAction(iBookmarkTools->toggleViewAction());
+  iLayerTools = new QDockWidget("Layers", this);
+  addDockWidget(getDockSide(L, "layers", Qt::LeftDockWidgetArea),
+		iLayerTools);
+  iLayerTools->setAllowedAreas(Qt::LeftDockWidgetArea|Qt::RightDockWidgetArea);
 
   // object names are used for saving toolbar state
   iSnapTools->setObjectName(QLatin1String("SnapTools"));
@@ -468,6 +511,13 @@ AppUi::AppUi(lua_State *L0, int model, Qt::WFlags f)
   iResolution = new QLabel(statusBar());
   statusBar()->addPermanentWidget(iResolution, 0);
 
+  lua_getglobal(L, "prefs");
+  lua_getfield(L, -1, "retina");
+  int retina = lua_toboolean(L, -1) ? 2 : 1;
+  ipeDebug("setRetina(%d)", retina);
+  iCanvas->setRetina(retina);
+  lua_pop(L, 2); // retina, prefs
+
   iCanvas->setObserver(this);
 }
 
@@ -536,6 +586,11 @@ void AppUi::toolbarModifiersChanged()
   }
 }
 
+void AppUi::abortDrawing()
+{
+  action("stop");
+}
+
 // --------------------------------------------------------------------
 
 void AppUi::resetCombos()
@@ -544,10 +599,17 @@ void AppUi::resetCombos()
     iSelector[i]->clear();
 }
 
-void AppUi::addComboColor(int sel, String s, Color color)
+void AppUi::addComboColors(AttributeSeq &sym, AttributeSeq &abs)
 {
-  QIcon icon = prefsColorIcon(color);
-  iSelector[sel]->addItem(icon, QIpe(s));
+  for (uint i = 0; i < sym.size(); ++i) {
+    Color color = abs[i].color();
+    String s = sym[i].string();
+    QIcon icon = prefsColorIcon(color);
+    iSelector[EUiStroke]->addItem(icon, QIpe(s));
+    iSelector[EUiFill]->addItem(icon, QIpe(s));
+    iComboContents[EUiStroke].push_back(s);
+    iComboContents[EUiFill].push_back(s);
+  }
 }
 
 void AppUi::addCombo(int sel, String s)
@@ -745,7 +807,7 @@ void AppUi::comboSelector(int id)
 
 static const char * const aboutText =
 "<qt><h2>Ipe %d.%d.%d</h2>"
-"<p>Copyright (c) 1993-2013 Otfried Cheong</p>"
+"<p>Copyright (c) 1993-2015 Otfried Cheong</p>"
 "<p>The extensible drawing editor Ipe creates figures "
 "in Postscript and PDF format, "
 "using LaTeX to format the text in the figures.</p>"
@@ -760,7 +822,7 @@ static const char * const aboutText =
 "<li> Some code from Xpdf"
 "</ul>"
 "<p>Ipe is released under the GNU Public License.</p>"
-"<p>See <a href=\"http://ipe7.sourceforge.net\">ipe7.sourceforge.net</a>"
+"<p>See <a href=\"http://ipe.otfried.org\">ipe.otfried.org</a>"
 " for details.</p>"
 "</qt>";
 
@@ -804,6 +866,41 @@ void AppUi::action(String name)
 
 // --------------------------------------------------------------------
 
+int AppUi::pageSorter(lua_State *L, Document *doc,
+		      int width, int height, int thumbWidth)
+{
+  QDialog *d = new QDialog();
+  d->setWindowTitle("Ipe Page Sorter");
+
+  QLayout *lo = new QVBoxLayout;
+  PageSorter *p = new PageSorter(doc, thumbWidth);
+  QDialogButtonBox *buttonBox =
+    new QDialogButtonBox(QDialogButtonBox::Ok|QDialogButtonBox::Cancel);
+  lo->addWidget(p);
+  lo->addWidget(buttonBox);
+  d->setLayout(lo);
+
+  d->connect(buttonBox, SIGNAL(accepted()), SLOT(accept()));
+  d->connect(buttonBox, SIGNAL(rejected()), SLOT(reject()));
+
+  d->resize(width, height);
+
+  if (d->exec() == QDialog::Rejected) {
+    delete d;
+    return 0;
+  }
+
+  lua_createtable(L, p->count(), 0);
+  for (int i = 1; i <= p->count(); ++i) {
+    lua_pushinteger(L, p->pageAt(i-1) + 1);
+    lua_rawseti(L, -2, i);
+  }
+  delete d;
+  return 1;
+}
+
+// --------------------------------------------------------------------
+
 void AppUi::showPathStylePopup(Vector v)
 {
   luaShowPathStylePopup(v);
@@ -854,6 +951,60 @@ void AppUi::showWindow(int width, int height)
   show();
 }
 
+int AppUi::setClipboard(lua_State *L)
+{
+  QString data = QString::fromUtf8(luaL_checkstring(L, 2));
+  QClipboard *cb = QApplication::clipboard();
+  cb->setText(data);
+  return 0;
+}
+
+static int getImage(lua_State *L, QImage &img)
+{
+  QImage im1 = img.convertToFormat(QImage::Format_ARGB32);
+  int w = im1.width();
+  int h = im1.height();
+  bool isGray = im1.allGray();
+
+  Bitmap::TColorSpace cs = isGray ? Bitmap::EDeviceGray : Bitmap::EDeviceRGB;
+  int datalen = w * h * (isGray ? 1 : 3);
+  Buffer data(datalen);
+  char *d = data.data();
+  for (int y = 0; y < h; ++y) {
+    uint *p = (uint *) im1.scanLine(y);
+    for (int x = 0; x < w; ++x) {
+      if (isGray) {
+	*d++ = qRed(*p++);
+      } else {
+	*d++ = qRed(*p);
+	*d++ = qGreen(*p);
+	*d++ = qBlue(*p++);
+      }
+    }
+  }
+
+  Bitmap bitmap(w, h, cs, 8, data, Bitmap::EDirect, true);
+  Rect r(Vector::ZERO, Vector(w, h));
+  Image *im = new Image(r, bitmap);
+  push_object(L, im);
+  return 1;
+}
+
+int AppUi::clipboard(lua_State *L)
+{
+  bool allow_bitmap = lua_toboolean(L, 2);
+  QClipboard *cb = QApplication::clipboard();
+  if (allow_bitmap) {
+    QImage img = cb->image();
+    if (!img.isNull()) {
+      return getImage(L, img);
+    }
+  }
+  QString data = cb->text();
+  lua_pushstring(L, data.toUtf8());
+  return 1;
+}
+
 AppUiBase *createAppUi(lua_State *L0, int model)
 {
   return new AppUi(L0, model);
