@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -71,7 +71,11 @@ class PathView : public QWidget {
   Q_OBJECT
 
 public:
+#if QT_VERSION < 0x050000
   PathView(QWidget* parent = 0, Qt::WFlags f = 0);
+#else
+  PathView(QWidget* parent = 0, Qt::WindowFlags f = 0);
+#endif
 
   void set(const AllAttributes &all, Cascade *sheet);
 
