@@ -4,7 +4,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -37,7 +37,7 @@ function MODEL:action(a)
   local result, err = xpcall(function () self:paction(a) end,
 			     debug.traceback)
   if not result then
-    ipeui.messageBox(nil, "critical",
+    messageBox(nil, "critical",
 		     "Lua error\n\n"..
 		       "Data may have been corrupted. \n" ..
 		       "Save your file!",
@@ -46,9 +46,10 @@ function MODEL:action(a)
 end
 
 function MODEL:paction(a)
+  -- print("MODEL:paction(" .. a .. ")")
   if a:sub(1,5) == "mode_" then
     self.mode = a:sub(6)
-    if prefs.tablet_mode then
+    if prefs.no_ink_snap then
       -- disable snapping in ink mode
       if self.mode == "ink" then
 	self.savedsnap = {}
@@ -67,10 +68,13 @@ function MODEL:paction(a)
 	self.ui:setActionState(e, self.snap[e])
       end
       self.ui:setSnap(self.snap)
+    end
+    if prefs.no_ink_highlight then
       self.ui:setFifiVisible(self.mode ~= "ink")
       self.ui:setSelectionVisible(self.mode ~= "ink")
-      self.ui:setCursor()
     end
+    -- TODO: This seems redundant?
+    self.ui:setCursor()
   elseif a:sub(1,4) == "snap" then
     self.snap[a] = self.ui:actionState(a)
     self.ui:setSnap(self.snap)
@@ -123,6 +127,9 @@ function MODEL:selector(prop, value)
   self.ui:setAttributes(self.doc:sheets(), self.attributes)
   if self:page():hasSelection() then
     self:setAttribute(prop, value)
+    if prop == "textsize" then
+      self:autoRunLatex()
+    end
   end
   -- self:print_attributes()
 end
@@ -181,6 +188,7 @@ function MODEL:absoluteButton(button)
 			     10, 2, 1000)
     if d then
       self:set_absolute(button, d)
+      self:autoRunLatex()
     end
   elseif button == "symbolsize" then
     local old = self.doc:sheets():find(button, self.attributes[button])
@@ -250,6 +258,10 @@ function MODEL:action_mark_page(m)
   self:register(t)
 end
 
+function MODEL:action_stop()
+  self.ui:finishTool()
+end
+
 ----------------------------------------------------------------------
 
 function MODEL:showPathStylePopup(v)
@@ -299,7 +311,10 @@ function MODEL:showLayerBoxPopup(v, layer)
   local p = self:page()
   local m = ipeui.Menu(self.ui:win())
   local active = p:active(self.vno)
-  m:add("checkfrom", "Check " .. layer .. " from view " .. self.vno)
+  m:add("visible", "Make " .. layer .. " visible from view "
+	.. self.vno .. " onwards")
+  m:add("invisible", "Make " .. layer .. " invisible from view "
+	.. self.vno .. " onwards")
   m:add("rename", "Rename " .. layer)
   m:add("delete", "Delete " .. layer)
   if layer ~= active then
@@ -337,9 +352,17 @@ end
 
 ----------------------------------------------------------------------
 
-function MODEL:layeraction_checkfrom(layer, arg)
+function MODEL:layeraction_make_visible(layer, arg, vis)
   local p = self:page()
-  local t = { label="make layer " .. layer .. " visible from view " .. self.vno,
+  local label
+  if vis then
+    label = "make layer " .. layer .. " visible from view "
+      .. self.vno .. " onwards"
+  else
+    label = "make layer " .. layer .. " invisible from view "
+      .. self.vno .. " onwards"
+  end
+  local t = { label=label,
 	      pno=self.pno,
 	      vno=self.vno,
 	      original=p:clone(),
@@ -348,12 +371,20 @@ function MODEL:layeraction_checkfrom(layer, arg)
 	    }
   t.redo = function (t, doc)
 	     for j = t.vno, doc[t.pno]:countViews() do
-	       doc[t.pno]:setVisible(j, t.layer, true)
+	       doc[t.pno]:setVisible(j, t.layer, vis)
 	     end
 	   end
   self:register(t)
 end
 
+function MODEL:layeraction_visible(layer, arg)
+  self:layeraction_make_visible(layer, arg, true)
+end
+
+function MODEL:layeraction_invisible(layer, arg)
+  self:layeraction_make_visible(layer, arg, false)
+end
+
 function MODEL:layeraction_select(layer, arg)
   local p = self:page()
   local t = { label="set visibility of layer " .. layer,
@@ -465,7 +496,8 @@ end
 
 function MODEL:layeraction_rename(layer)
   local p = self:page()
-  local name = self:getString("Enter new name for layer '" .. layer .. "'")
+  local name = self:getString("Enter new name for layer '" .. layer .. "'",
+			      nil, layer)
   if not name then return end
   name = string.gsub(name, "%s+", "_")
   if indexOf(name, p:layers()) then
@@ -588,10 +620,6 @@ function MODEL:action_close()
   self.ui:close()
 end
 
-function MODEL:action_quit()
-  ipeui.closeAllWindows()
-end
-
 function MODEL:action_auto_latex()
   self.auto_latex = self.ui:actionState("auto_latex")
   print("Auto is", self.auto_latex)
@@ -606,8 +634,7 @@ end
 
 function MODEL:action_open()
   if not self:checkModified() then return end
-  local s, f = ipeui.fileDialog(self.ui:win(), "open", "Open file",
-	  "Ipe files (*.ipe *.pdf *.eps *.xml);;All files (*.*)")
+  local s, f = ipeui.fileDialog(self.ui:win(), "open", "Open file", filter_ipe)
   if s then
     self:loadDocument(s)
   else
@@ -625,23 +652,31 @@ end
 
 function MODEL:action_save_as()
   local dir
-  if self.file_name then dir = self.file_name:match("^(.+)/[^/]+") end
+  if self.file_name then dir = self.file_name:match(prefs.dir_pattern) end
   if not dir then dir = prefs.save_as_directory end
+  local name
+  local filter
+  if self.file_name then
+    name = self.file_name
+    local fmt = formatFromFileName(self.file_name)
+    if fmt then
+      filter = indexOf(fmt, { "xml", "pdf" })
+      name = self.file_name:sub(1,-5)
+    end
+  end
   local s, f =
     ipeui.fileDialog(self.ui:win(), "save", "Save file as",
-		     "XML (*.ipe *.xml);;PDF (*.pdf);;EPS (*.eps)",
-		     dir, self.dialog_filter)
-  local fmap = { XML=".ipe", PDF=".pdf", EPS=".eps" }
+		     filter_save, dir, name, filter)
+  local fmap = { ".ipe", ".pdf" }
   if s then
-    self.dialog_filter = f
     if not formatFromFileName(s) then
-      s = s .. fmap[f:sub(1,3)]
+      s = s .. fmap[f]
     end
     if ipe.fileExists(s) then
-      local b = ipeui.messageBox(self.ui:win(), "question",
-				 "File already exists!",
-				 "Do you wish to overwrite?<br>" .. s,
-				 "okcancel")
+      local b = messageBox(self.ui:win(), "question",
+			   "File already exists!",
+			   "Do you wish to overwrite?<br>" .. s,
+			   "okcancel")
       if b ~= 1 then
 	self.ui:explain("File not saved")
 	return
@@ -651,12 +686,109 @@ function MODEL:action_save_as()
   end
 end
 
+function MODEL:compute_rect(w, h, res)
+  if res == V(0,0) then res = V(72.0, 72.0) end
+
+  local frame = self.doc:sheets():find("layout").framesize
+
+  local dx = (w * 72.0) / res.x
+  local dy = (h * 72.0) / res.y
+
+  local xfactor = 1.0
+  if (dx > frame.x) then
+    xfactor = frame.x / dx
+  end
+  local  yfactor = 1.0
+  if (dy > frame.y) then
+    yfactor = frame.y / dy
+  end
+  local factor = yfactor
+  if (xfactor < yfactor) then factor = xfactor end
+  dx = dx * factor
+  dy = dy * factor
+
+  local v = 0.5 * V(frame.x - dx, frame.y - dy)
+  local rect = ipe.Rect()
+  rect:add(v)
+  rect:add(v + V(dx, dy))
+  return rect
+end
+
+function MODEL:action_export_png()
+  local dir
+  if self.file_name then dir = self.file_name:match(prefs.dir_pattern) end
+  if not dir then dir = prefs.save_as_directory end
+  local s, f =
+    ipeui.fileDialog(self.ui:win(), "save", "Export to PNG", filter_png, dir)
+  if s then
+    if ipe.fileExists(s) then
+      local b = messageBox(self.ui:win(), "question",
+			   "File already exists!",
+			   "Do you wish to overwrite?<br>" .. s,
+			   "okcancel")
+      if b ~= 1 then
+	self.ui:explain("Document not exported")
+	return
+      end
+    end
+    self.ui:renderPage(self.doc, self.pno, self.vno,
+		       "png", s, self.ui:zoom(),
+		       false, false) -- transparent, nocrop
+  end
+end
+
+function MODEL:action_export_eps()
+  local dir
+  if self.file_name then dir = self.file_name:match(prefs.dir_pattern) end
+  if not dir then dir = prefs.save_as_directory end
+  local s, f =
+    ipeui.fileDialog(self.ui:win(), "save", "Export to EPS", filter_eps, dir)
+  if s then
+    if ipe.fileExists(s) then
+      local b = messageBox(self.ui:win(), "question",
+			   "File already exists!",
+			   "Do you wish to overwrite?<br>" .. s,
+			   "okcancel")
+      if b ~= 1 then
+	self.ui:explain("Document not exported")
+	return
+      end
+    end
+    self.ui:renderPage(self.doc, self.pno, self.vno,
+		       "eps", s, 1.0, false, false)
+  end
+end
+
+function MODEL:action_insert_image()
+  local s, f = ipeui.fileDialog(self.ui:win(), "open", "Insert image",
+				filter_images)
+  if s then
+    local format = "png"
+    if s:sub(-4):lower() == ".jpg" or s:sub(-5):lower() == ".jpeg"
+    then format = "jpeg" end
+    local bitmap, res = ipe.readImage(s, format)
+    if not bitmap then
+      messageBox(self.ui:win(), "warning", "Could not read image",
+		 res, "ok")
+    else
+      local info = bitmap:info()
+      local r = self:compute_rect(info.width, info.height, res)
+      local obj = ipe.Image(r, bitmap)
+      self:creation("insert image", obj)
+    end
+  else
+    self.ui:explain("Insert image canceled")
+  end
+end
+
 ----------------------------------------------------------------------
 
 function MODEL:action_manual()
   local url = "file:///" .. config.docdir .. "/manual.html"
   if ipeui.startBrowser then
-    ipeui.startBrowser(url)
+    if not ipeui.startBrowser(url) then
+      messageBox(nil, "warning", "Manual not available!", url)
+    end
   else
     os.execute(prefs.browser:format(url))
   end
@@ -670,32 +802,74 @@ function MODEL:action_show_configuration()
   s = s .. "\n * Styles for new documents: " ..
     table.concat(prefs.styles, ", ")
   s = s .. "\n * Autosave file: " .. prefs.autosave_filename
+  s = s .. "\n * Save-as directory: " .. prefs.save_as_directory
   s = s .. "\n * Documentation: " .. config.docdir
   s = s .. "\n * Ipelets:\n   - " .. table.concat(config.ipeletDirs, "\n   - ")
   s = s .. "\n * Latex directory: " .. config.latexdir
   s = s .. "\n * Fontmap: " .. config.fontmap
   s = s .. "\n * Icons: " .. config.icons
-  ipeui.messageBox(self.ui:win(), "information", "Ipe configuration", s)
+  s = s .. "\n * External editor: " .. (prefs.external_editor or "none")
+  messageBox(self.ui:win(), "information", "Ipe configuration", s)
 end
 
 function MODEL:action_about_ipelets()
   local s = ""
-  for _,v in ipairs(ipelets) do
-    s = s .. " * " .. v.label .. "\n   " .. v.about .. "\n"
+  for i,v in ipairs(ipelets) do
+    if v.label then
+      s = s .. "### " .. v.label .. " ###\n\n" .. v.about .. "\n"
+    end
   end
-  ipeui.messageBox(self.ui:win(), "information", "About the ipelets", s)
+  local d = ipeui.Dialog(self.ui:win(), "Ipe: About the ipelets")
+  d:add("text", "text", { read_only=true }, 1, 1)
+  d:set("text", s)
+  d:addButton("ok", "Ok", "accept")
+  d:execute(prefs.latexlog_size)
 end
 
 function MODEL:action_keyboard()
   local s = prefs.keyboard
   if s then
-    os.execute(s)
+    if ipeui.startBrowser then
+      ipeui.startBrowser(s)
+    else
+      os.execute(s)
+    end
   else
     self:warning("No onscreen keyboard defined.",
 		 "Edit preferences to define an onscreen keyboard.")
   end
 end
 
+function MODEL:action_developer_reload_ipelets()
+  load_ipelets()
+  self.ui:explain("Ipelets reloaded")
+end
+
+function MODEL:action_developer_list_shortcuts()
+  local keys = {}
+  for n,k in pairs(shortcuts) do
+    if type(k) == "table" then
+      for _,k1 in ipairs(k) do
+	keys[#keys+1] = k1
+      end
+    else
+      keys[#keys+1] = k
+    end
+  end
+  table.sort(keys)
+  printTable(keys)
+  local dupl = 0
+  local prev = nil
+  for _, k in ipairs(keys) do
+    if k == prev then
+      print("Duplicate shortcut assignment: ", k)
+      dupl = dupl + 1
+    end
+    prev = k
+  end
+  print(dupl, " duplicate shortcuts found")
+end
+
 ----------------------------------------------------------------------
 
 function MODEL:action_set_origin_snap()
@@ -839,6 +1013,11 @@ function MODEL:action_grid_visible()
   self.ui:update()
 end
 
+function MODEL:action_pretty_display()
+  self.ui:setPretty(self.ui:actionState("pretty_display"));
+  self.ui:update()
+end
+
 function MODEL:action_zoom_in()
   local nzoom = prefs.zoom_factor * self.ui:zoom()
   if nzoom > prefs.max_zoom then nzoom = prefs.max_zoom end
@@ -936,7 +1115,7 @@ end
 
 function MODEL:action_normal_size()
   local p = self:page()
-  self.ui:setZoom(1)
+  self.ui:setZoom(prefs.normal_resolution / 72.0)
   local layout = self.doc:sheets():find("layout")
   local paper = ipe.Rect()
   paper:add(-layout.origin)
@@ -1108,8 +1287,8 @@ function MODEL:action_edit_effects()
   local label = "Choose effect for view " .. self.vno
   d:add("label", "label", { label=label }, 1, 1)
   d:add("combo", "combo", list, 2, 1)
-  d:addButton("cancel", "&Cancel", "reject")
   d:addButton("ok", "&Ok", "accept")
+  d:addButton("cancel", "&Cancel", "reject")
   d:set("combo", indexOf(effect, list))
   if not d:execute() then return end
   local final = list[d:get("combo")]
@@ -1154,7 +1333,7 @@ end
 
 function MODEL:action_copy_page()
   local data = self:page():xml("ipepage")
-  ipeui.setClipboard(data)
+  self.ui:setClipboard(data)
   self.ui:explain("page copied to clipboard")
 end
 
@@ -1183,7 +1362,7 @@ function MODEL:action_cut_page()
   self:registerOnly(t)
   self.ui:explain(t.label .. " and copied to clipboard")
   local data = t.original:xml("ipepage")
-  ipeui.setClipboard(data)
+  self.ui:setClipboard(data)
 end
 
 function MODEL:action_new_page()
@@ -1205,7 +1384,7 @@ function MODEL:action_new_page()
 end
 
 function MODEL:action_paste_page()
-  local data = ipeui.clipboard()
+  local data = self.ui:clipboard()
   if data:sub(1,9) ~= "<ipepage>" then
     self:warning("No Ipe page to paste")
     return
@@ -1282,8 +1461,8 @@ function MODEL:action_edit_title()
 	  action=function (d) update_dialog(d, "subsection") end
 	} , 6, 2)
   d:add("subsection", "input", {}, 7, 2, 1, 3)
-  d:addButton("cancel", "&Cancel", "reject")
   d:addButton("ok", "&Ok", "accept")
+  d:addButton("cancel", "&Cancel", "reject")
   d:setStretch("row", 2, 1)
   d:setStretch("column", 2, 1)
   -- setup original values
@@ -1326,14 +1505,15 @@ function MODEL:action_edit_title()
 	     doc[t.pno]:setTitles(t.final)
 	   end
   self:register(t)
+  self:autoRunLatex()
 end
 
 function MODEL:action_edit_notes()
   local d = ipeui.Dialog(self.ui:win(), "Ipe: Edit page notes")
   d:add("label1", "label", { label="Page notes"}, 1, 1)
   d:add("notes", "text", {}, 2, 1)
-  d:addButton("cancel", "&Cancel", "reject")
   d:addButton("ok", "&Ok", "accept")
+  d:addButton("cancel", "&Cancel", "reject")
   d:setStretch("row", 2, 1)
   -- setup original values
   local n = self:page():notes()
@@ -1412,6 +1592,14 @@ function MODEL:action_page_sorter()
   self:register(t)
 end
 
+function MODEL:action_toggle_notes()
+  self.ui:showTool("notes", self.ui:actionState("toggle_notes"))
+end
+
+function MODEL:action_toggle_bookmarks()
+  self.ui:showTool("bookmarks", self.ui:actionState("toggle_bookmarks"))
+end
+
 ----------------------------------------------------------------------
 
 function MODEL:action_new_layer()
@@ -1457,32 +1645,37 @@ function MODEL:saction_edit_as_xml()
   local prim = self:page():primarySelection()
   local xml = self:page()[prim]:xml()
   local d = ipeui.Dialog(self.ui:win(), "Edit as XML")
-  d:add("xml", "text", { syntax="xml" }, 1, 1)
+  d:add("xml", "text", { syntax="xml", focus=true }, 1, 1)
   addEditorField(d, "xml")
-  d:addButton("cancel", "&Cancel", "reject")
   d:addButton("ok", "&Ok", "accept")
+  d:addButton("cancel", "&Cancel", "reject")
   d:setStretch("row", 1, 1);
   d:set("xml", xml)
-  if not d:execute() then return end
-  local obj = ipe.Object(d:get("xml"))
-  if not obj then self:warning("Cannot parse XML") return end
+  if prefs.auto_external_editor then
+    externalEditor(d, "xml")
+  end
+  if ((prefs.auto_external_editor and prefs.editor_closes_dialog)
+    or d:execute(prefs.editor_size)) then
+    local obj = ipe.Object(d:get("xml"))
+    if not obj then self:warning("Cannot parse XML") return end
 
-  local t = { label="edit object in XML",
-	      pno = self.pno,
-	      vno = self.vno,
-	      primary = prim,
-	      original = self:page()[prim]:clone(),
-	      final = obj,
-	    }
-  t.undo = function (t, doc)
-	     doc[t.pno]:replace(t.primary, t.original)
-	   end
-  t.redo = function (t, doc)
-	     doc[t.pno]:replace(t.primary, t.final)
-	   end
-  self:register(t)
+    local t = { label="edit object in XML",
+		pno = self.pno,
+		vno = self.vno,
+		primary = prim,
+		original = self:page()[prim]:clone(),
+		final = obj,
+	      }
+    t.undo = function (t, doc)
+	       doc[t.pno]:replace(t.primary, t.original)
+	     end
+    t.redo = function (t, doc)
+	       doc[t.pno]:replace(t.primary, t.final)
+	     end
+    self:register(t)
+    if obj:type() == "text" then self:autoRunLatex() end
+  end
 end
-
 ----------------------------------------------------------------------
 
 function MODEL:saction_group()
@@ -1748,13 +1941,13 @@ end
 
 function MODEL:action_copy()
   local data = self:selectionWithCursor()
-  ipeui.setClipboard(data)
+  self.ui:setClipboard(data)
   self.ui:explain("selection copied to clipboard")
 end
 
 function MODEL:action_cut()
   local data = self:selectionWithCursor()
-  ipeui.setClipboard(data)
+  self.ui:setClipboard(data)
   local t = { label="cut selection",
 	      pno = self.pno,
 	      vno = self.vno,
@@ -1772,15 +1965,24 @@ function MODEL:action_cut()
 end
 
 function MODEL:action_paste()
-  local data = ipeui.clipboard()
-  if data:sub(1,13) ~= "<ipeselection" then
-    self:warning("No Ipe selection to paste")
+  local data = self.ui:clipboard(true)  -- allow a bitmap
+  if not data then
+    self:warning("Nothing to paste")
     return
   end
-  local elements = ipe.Object(data)
-  if not elements then
-    self:warning("Could not parse Ipe selection on clipboard")
-    return
+  local elements
+  if type(data) == "string" then
+    if data:sub(1,13) ~= "<ipeselection" then
+      self:warning("No Ipe selection to paste")
+      return
+    end
+    elements = ipe.Object(data)
+    if not elements then
+      self:warning("Could not parse Ipe selection on clipboard")
+      return
+    end
+  else
+    elements = { data }
   end
   local p = self:page()
   local t = { label="paste objects",
@@ -1818,7 +2020,7 @@ end
 
 function MODEL:action_update_style_sheets()
   local dir = nil
-  if self.file_name then dir = string.gsub(self.file_name, "[^/]+$", "") end
+  if self.file_name then dir = string.gsub(self.file_name, "[^/\\]+$", "") end
   if dir == "" then dir = "." end
   local sheets = self.doc:sheets()
   local qlog = ""
@@ -1845,7 +2047,7 @@ function MODEL:action_update_style_sheets()
     end
   end
 
-  ipeui.messageBox(self.ui:win(), "information",
+  messageBox(self.ui:win(), "information",
 		   "Result of updating stylesheets:", qlog)
 
   local t = { label="update style sheets",
@@ -1876,7 +2078,7 @@ local function sheets_add(d, dd)
   local i = d:get("list")
   if not i then i = 1 end
   local s, f = ipeui.fileDialog(dd.model.ui:win(), "open", "Add stylesheet",
-				"Ipe stylesheets (*.isy)")
+				filter_stylesheets)
   if not s then return end
   local sheet, msg = ipe.Sheet(s)
   if not sheet then
@@ -1904,12 +2106,12 @@ local function sheets_edit(d, dd)
   f:close()
   local sheet, msg
   while not sheet do
-    ipeui.WaitDialog(string.format(prefs.external_editor, fname))
+    ipeui.waitDialog(d, string.format(prefs.external_editor, fname))
     sheet, msg = ipe.Sheet(fname)
     if not sheet then
-      local r = ipeui.messageBox(dd.model.ui:win(), "question",
-				 "Cannot load stylesheet - try again?",
-				 msg, "okcancel")
+      local r = messageBox(dd.model.ui:win(), "question",
+			   "Cannot load stylesheet - try again?",
+			   msg, "okcancel")
       if r <= 0 then
 	os.remove(fname)
 	return
@@ -1962,11 +2164,11 @@ local function sheets_save(d, dd)
   local i = d:get("list")
   if not i then return end
   local s, f = ipeui.fileDialog(dd.model.ui:win(), "save", "Save stylesheet",
-				"Ipe stylesheets (*.isy)")
+				filter_stylesheets)
   if s then
     if s:sub(-4) ~= ".isy" then s = s .. ".isy" end
     local data = dd.list[i]:xml(true)
-    local f, msg = io.open(ipeui.local8bit(s), "wb")
+    local f, msg = ipe.openFile(s, "wb")
     if not f then
       dd.model:warning("Cannot save stylesheet", msg)
       return
@@ -2003,8 +2205,8 @@ function MODEL:action_style_sheets()
 	{ label="&Down", action=function (d) sheets_down(d, dd) end }, 6, 4)
   d:add("save", "button",
 	{ label="&Save", action=function (d) sheets_save(d, dd) end }, 7, 4)
-  d:addButton("cancel", "&Cancel", "reject")
   d:addButton("ok", "&Ok", "accept")
+  d:addButton("cancel", "&Cancel", "reject")
   d:setStretch("column", 2, 1)
   if not d:execute() or not dd.modified then return end
   local t = { label="modify style sheets",
@@ -2048,8 +2250,8 @@ function MODEL:action_document_properties()
   d:add("modified", "label",  { label="" }, 9, 2)
   d:add("label9", "label", { label="Creator" }, 10, 1)
   d:add("creator", "label",  { label="" }, 10, 2)
-  d:addButton("cancel", "&Cancel", "reject")
   d:addButton("ok", "&Ok", "accept")
+  d:addButton("cancel", "&Cancel", "reject")
   d:setStretch("column", 5, 1)
   d:setStretch("row", 6, 1)
   for n in pairs(p) do d:set(n, p[n]) end
