@@ -4,7 +4,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -33,12 +33,11 @@ EDITTOOL.__index = EDITTOOL
 
 local VERTEX = 1
 local SPLINECP = 2
-local BEZIERCP = 3
-local CENTER = 4
-local RADIUS = 5
-local MINOR = 6
-local CURRENT = 7
-local SCISSOR = 8
+local CENTER = 3
+local RADIUS = 4
+local MINOR = 5
+local CURRENT = 6
+local SCISSOR = 7
 
 ----------------------------------------------------------------------
 
@@ -152,6 +151,7 @@ function EDITTOOL:new(model, prim, obj)
   transformShape(obj:matrix(), tool.shape)
   model.ui:shapeTool(tool)
   tool.setColor(1.0, 0, 0)
+  tool.setSnapping(true, false)
   tool:resetCP()
   tool:setShapeMarks()
   return tool
@@ -175,8 +175,7 @@ function EDITTOOL:type()
   end
   cur = cur[self.segNum]
   if self.cpNum == 1 or self.cpNum == #cur then return VERTEX end
-  if cur.type == "spline" then return SPLINECP end
-  if cur.type == "bezier" or cur.type == "quad" then return BEZIERCP end
+  if cur.type == "spline" or cur.type == "oldspline" then return SPLINECP end
   -- must be center of arc
   return CENTER
 end
@@ -241,10 +240,10 @@ function EDITTOOL:setShapeMarks()
       for _,seg in ipairs(sp) do
 	m[#m + 1] = seg[#seg]
 	m[#m + 1] = VERTEX
-	if seg.type=="quad" or seg.type=="bezier" or seg.type=="spline" then
+	if seg.type=="spline" or seg.type=="oldspline" then
 	  for i = 2,#seg-1 do
 	    m[#m + 1] = seg[i]
-	    m[#m + 1] = seg.type=="spline" and SPLINECP or BEZIERCP
+	    m[#m + 1] = SPLINECP
 	  end
 	  local as = { type="curve", closed=false }
 	  for i = 2,#seg do
@@ -411,7 +410,7 @@ function EDITTOOL:action_duplicate()
     local seg = sp[self.segNum]
     local cp = seg[self.cpNum]
     if 1 < self.cpNum and self.cpNum < #seg then
-      assert(seg.type == "spline")
+      assert(seg.type == "spline" or seg.type == "oldspline")
       table.insert(seg, self.cpNum, cp)
       local pcp = seg[self.cpNum - 1]
       local ncp = seg[self.cpNum + 2]
@@ -505,7 +504,8 @@ function EDITTOOL:action_convert_spline()
     beziers.closed = false
     self.shape[self.spNum] = beziers
   else
-    local beziers = ipe.splineToBeziers(sp[self.segNum], false)
+    local beziers = ipe.splineToBeziers(sp[self.segNum], false,
+					sp[self.segNum].type == "oldspline")
     table.remove(sp, self.segNum)
     for i=1,#beziers do
       table.insert(sp, self.segNum + i - 1, beziers[i])
@@ -514,6 +514,125 @@ function EDITTOOL:action_convert_spline()
   self.undo[#self.undo + 1] = t
 end
 
+function EDITTOOL:action_insert_knot()
+  local sp = self.shape[self.spNum]
+  local seg = sp[self.segNum]
+  -- Knot sequence: [0, 0, 0, 1, 2, 3, 4, 5, 6, ..., k-2, k-1, k, k, k]
+  -- It has k + 5 = n + 2 knots, so there are k = n-3  knot intervals.
+  -- cps are [0,0,0], [0,0,1], [0,1,2]...[k-2,k-1,k],[k-1,k,k],[k,k,k]
+  -- we are going to insert a vertex at knot t, with 1 <= t <= k-1
+  local k = #seg - 3
+  -- we need k >= 3, so handle case n = 5 separately:
+  if k < 3 then
+    self:action_convert_spline()
+    return
+  end
+  local t = self.cpNum - 2
+  if t < 1 then t = 1 end
+  if t > k - 1 then t = k-1 end
+  local ls = { type = "spline" }
+  local rs = { type = "spline" }
+  if t == 1 then
+    -- we need [0,1,1], [1,1,1] and [1,1,2]
+    -- [0,1,1] = 1/2 [0,0,1] + 1/2 [0,2,1]
+    local q = 0.5 * (seg[2] + seg[3])
+    -- [1,1,2] = 2/3 [0,1,2] + 1/3 [3,1,2]
+    local r = (1.0/3.0) * (2 * seg[3] + seg[4])
+    local p = 0.5 * (q + r) -- [1,1,1]
+    ls[1] = seg[1]
+    ls[2] = seg[2]
+    ls[3] = q
+    ls[4] = p
+    rs[1] = p
+    rs[2] = r
+    for i=4,#seg do
+      rs[#rs+1] = seg[i]
+    end
+  elseif t == k-1 then
+    -- we need [k-2,k-1,k-1], [k-1,k-1,k-1] and [k-1,k-1,k]
+    -- [k-2,k-1,k-1] = 1/3 [k-2,k-1,k-3] + 2/3 [k-2,k-1,k]
+    local q = (1.0/3.0) * (seg[k] + 2 * seg[k+1])
+    -- [k-1,k-1,k] = 1/2 [k-2,k-1,k] + 1/2 [k,k-1,k]
+    local r = 0.5 * (seg[k+1] + seg[k+2])
+    local p = 0.5 * (q + r) -- [k-1,k-1,k-1]
+    for i = 1,#seg-3 do ls[i] = seg[i] end
+    ls[#ls+1] = q
+    ls[#ls+1] = p
+    rs[1] = p
+    rs[2] = r
+    rs[3] = seg[#seg-1]
+    rs[4] = seg[#seg]
+  else -- 2 <= t <= k-2
+    -- we need [t-1,t,t], [t,t,t], [t,t,t+1]
+    -- [t-1,t,t] = 1/3 [t-1,t,t-2] + 2/3 [t-1,t,t+1]
+    local q = (1.0/3.0) * (seg[t+1] + 2 * seg[t+2])
+    -- [t,t,t+1] = 2/3 [t-1,t,t+1] + 1/3 [t+2,t,t+1]
+    local r = (1.0/3.0) * (2 * seg[t+2] + seg[t+3])
+    -- [t,t,t] = 1/2 [t,t,t-1] + 1/2[t,t,t+1]
+    local p = 0.5 * (q + r)
+
+    -- copy [0,0,0]...[t-2,t-1,t]
+    for i=1,t+1 do
+      ls[i] = seg[i]
+    end
+    ls[t+2] = q
+    ls[t+3] = p
+    rs[1] = p
+    rs[2] = r
+    -- copy [t,t+1,t+2] ... [k,k,k]
+    for i=t+3,#seg do
+      rs[#rs+1] = seg[i]
+    end
+  end
+
+  local it = { label = "insert vertex at knot " .. t,
+	       spNum = self.spNum,
+	       original = cloneSubpath(sp),
+	       undo = revertSubpath }
+  table.remove(sp, self.segNum)
+  table.insert(sp, self.segNum, ls)
+  table.insert(sp, self.segNum + 1, rs)
+  self.undo[#self.undo + 1] = it
+end
+
+
+function EDITTOOL:action_insert_knot_closed()
+  local sp = self.shape[self.spNum]
+  local n = #sp
+  local t = self.cpNum
+  local it = { label = "insert vertex at knot " .. t,
+	       spNum = self.spNum,
+	       original = cloneSubpath(sp),
+	       undo = revertSubpath }
+
+  -- knot sequence: 0 1 2 3 4 ... n-1 n n+1 n+2
+  -- where [n n+1 n+2] = [0 1 2], etc.
+  -- rotate knot to the front
+  for i=1,t-1 do
+    local v = table.remove(sp, 1)
+    sp[#sp+1] = v
+  end
+  -- cut at knot 1
+  -- need [0,1,1], [1,1,1], and [1,1,2]
+  -- [0,1,1] = 1/3 [-1,0,1] + 2/3 [2,0,1]
+  local q = (1.0/3.0) * (sp[n] + 2 * sp[1])
+  -- [1,1,2] = 2/3 [0,1,2] + 1/3 [3,1,2]
+  local r = (1.0/3.0) * (2 * sp[1] + sp[2])
+  local p = 0.5 * (q + r) -- [1,1,1]
+
+  local seg = { type="spline" }
+  seg[1] = p
+  seg[2] = r
+  for i=2,n do seg[#seg+1] = sp[i] end
+  seg[#seg+1] = q
+  seg[#seg+1] = p
+  table.remove(self.shape, self.spNum)
+  table.insert(self.shape, self.spNum, { type="curve", closed=false, seg })
+  self.cpNum = 1
+
+  self.undo[#self.undo + 1] = it
+end
+
 ----------------------------------------------------------------------
 
 -- subdivide at parameter t (0 <= t <= 1)
@@ -619,15 +738,13 @@ function EDITTOOL:cutter_bezier()
   local param, _ = self.scissor(self.model.ui:pos())
   local seg = sp[self.segNum]
   local nseg1, nseg2
-  if seg.type == "bezier" then
+  if #seg == 4 then
     nseg1, nseg2 = subdivideBezier(seg, param)
-    nseg1.type = "bezier"
-    nseg2.type = "bezier"
   else
     nseg1, nseg2 = subdivideQuad(seg, param)
-    nseg1.type = "quad"
-    nseg2.type = "quad"
   end
+  nseg1.type = "spline"
+  nseg2.type = "spline"
   if not sp.closed then
     local nsp = { type = "curve", closed = false, nseg2 }
     for i = self.segNum+1,#sp do nsp[#nsp + 1] = sp[i] end
@@ -665,7 +782,7 @@ function EDITTOOL:action_cut_bezier()
   local sp = self.shape[self.spNum]
   local seg = sp[self.segNum]
   local bezier
-  if seg.type == "bezier" then
+  if #seg == 4 then
     bezier = ipe.Bezier(seg[1], seg[2], seg[3], seg[4])
   else
     bezier = ipe.Quad(seg[1], seg[2], seg[3])
@@ -694,6 +811,8 @@ end
 ----------------------------------------------------------------------
 
 function EDITTOOL:action_undo()
+  self.moving = false
+  self.scissor = false
   if #self.undo == 0 then
     self.model:warning("Nothing to undo")
     return
@@ -724,10 +843,24 @@ function EDITTOOL:showMenu()
     m:add("action_delete_subpath", "Delete subpath")
   end
   if t == SPLINECP then
-    m:add("action_convert_spline", "Convert spline to Beziers")
-  end
-  if t == BEZIERCP then
-    m:add("action_cut_bezier", "Cut Bezier spline")
+    local sp = self.shape[self.spNum]
+    if sp.type == "closedspline" then
+      m:add("action_convert_spline", "Convert spline to Bezier segments")
+      m:add("action_insert_knot_closed", "Cut closed spline at knot " ..
+	    self.cpNum)
+    else
+      local n = #(sp[self.segNum])
+      if n > 4 then
+	m:add("action_convert_spline", "Convert spline to Bezier segments")
+	local t = self.cpNum - 2
+	if t < 1 then t = 1 end
+	if t > n - 4 then t = n - 4 end
+	m:add("action_insert_knot", "Insert vertex at knot " .. t ..
+	      " (of " .. (n-3) .. " knot intervals)")
+      else
+	m:add("action_cut_bezier", "Cut inside Bezier curve")
+      end
+    end
   end
   if t == CENTER then
     if self.shape[self.spNum].type == "ellipse" then
@@ -737,7 +870,14 @@ function EDITTOOL:showMenu()
     end
   end
   m:add("action_undo", "Undo")
+  m:add("accept", "Accept")
   local item = m:execute(gp.x, gp.y)
+  if item == "accept" then
+    self.moving = false
+    self:acceptEdit()
+    self.model.ui:finishTool()
+    return
+  end
   if item then
     self[item](self)
     self:setShapeMarks()
@@ -759,23 +899,31 @@ function EDITTOOL:mouseButton(button, modifiers, press)
   end
   self.scissor = nil
   if not press then
-    if button == 2 then
-      self.spNum, self.segNum, self.cpNum =
-	self:find(self.model.ui:unsnappedPos())
-      self:showMenu()
-      return
-    end
-  else -- press
-    if button == 1 then
-      self.spNum, self.segNum, self.cpNum =
-	self:find(self.model.ui:unsnappedPos())
-      self.moving = true
-      local t = { label = "move",
-		  original = cloneSubpath(self.shape[self.spNum]),
-		  spNum = self.spNum,
-		  undo = revertSubpath }
-      self.undo[#self.undo + 1] = t
-    end
+    self.setSnapping(true, false)
+    return
+  end
+  if button == 0x81 then
+    -- previous click must have created a move action, discard
+    table.remove(self.undo)
+    button = 2
+  end
+  if button == 2 then
+    self.spNum, self.segNum, self.cpNum =
+      self:find(self.model.ui:unsnappedPos())
+    self:showMenu()
+    return
+  end
+  if button == 1 then
+    self.spNum, self.segNum, self.cpNum =
+      self:find(self.model.ui:unsnappedPos())
+    self.moving = true
+    -- TODO: avoid the moving vertex snapping to itself
+    self.setSnapping(false, false)
+    local t = { label = "move",
+		original = cloneSubpath(self.shape[self.spNum]),
+		spNum = self.spNum,
+		undo = revertSubpath }
+    self.undo[#self.undo + 1] = t
   end
   self:setShapeMarks()
   self.model.ui:update(false) -- update tool
@@ -799,11 +947,11 @@ end
 
 function EDITTOOL:explain()
   if self.scissor then
-    self.model.ui:explain("Left: cut | Right: menu | Ctrl+Z: undo | "
-			  .. "Space: accept")
+    self.model.ui:explain("Left: cut | Right or double-left: menu | "
+			  .. "Ctrl+Z: undo | Space: accept")
   else
-    self.model.ui:explain("Left: move | Right: menu | Ctrl+Z: undo | "
-			  .. "Space: accept")
+    self.model.ui:explain("Left: move | Right or double-left: menu | "
+			  .. "Ctrl+Z: undo | Space: accept")
   end
 end
 
@@ -826,19 +974,17 @@ function EDITTOOL:acceptEdit()
   self.model:register(t)
 end
 
-function EDITTOOL:key(code, modifiers, text)
-  -- print("Key:", code, text)
+function EDITTOOL:key(text, modifiers)
+  -- print("EditTool Key:", text)
   if text == "\027" then  -- Esc
     self.model.ui:finishTool()
     return true
-  elseif code == 90 and modifiers.control then  -- Ctrl+Z
-    self.moving = false
-    self.scissor = false
+  elseif text == "\026" and modifiers.control then  -- Ctrl+Z
     self:action_undo()
     self:setShapeMarks()
     self.model.ui:update(false) -- update tool
     return true
-  elseif code == 0x20 then -- Space: accept edit
+  elseif text == " " then -- Space: accept edit
     self.moving = false
     self:acceptEdit()
     self.model.ui:finishTool()
