@@ -4,7 +4,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -80,6 +80,7 @@ function MODEL:init(fname)
     snapangle = prefs.snap.angle,
     snapauto = prefs.snap.autoangle,
     grid_visible = prefs.grid_visible,
+    pretty_display = false,
     gridsize = prefs.grid_size,
     anglesize = prefs.angle_size,
     snap_distance = prefs.snap_distance,
@@ -93,9 +94,10 @@ function MODEL:init(fname)
   self.pristine = false
   self.first_show = true
 
+  local err = nil
   if fname then
     if ipe.fileExists(fname) then
-      self:loadDocument(fname)
+      err = self:tryLoadDocument(fname)
     else
       self:newDocument()
       self.file_name = fname
@@ -124,11 +126,8 @@ function MODEL:init(fname)
     self:runLatex()
   end
 
-  if #self:getBookmarks() == 0 then
-    self.ui:showTool("bookmarks", false)
-  end
-  if self:page():notes() == "" then
-    self.ui:showTool("notes", false)
+  for i, dock in ipairs(prefs.hide_docks) do
+    self.ui:showTool(dock, false)
   end
 
   if prefs.autosave_interval then
@@ -136,6 +135,9 @@ function MODEL:init(fname)
     self.timer:setInterval(1000 * prefs.autosave_interval) -- millisecs
     self.timer:start()
   end
+  if err then
+    self:warning("Document '" .. fname .. "' could not be opened", err)
+  end
 end
 
 function MODEL:sizeChanged()
@@ -175,9 +177,9 @@ function MODEL:getString(msg, caption, start)
   if caption == nil then caption = "Ipe" end
   local d = ipeui.Dialog(self.ui:win(), caption)
   d:add("label", "label", {label=msg}, 1, 1)
-  d:add("text", "input", {}, 2, 1)
-  d:addButton("cancel", "Cancel", "reject")
+  d:add("text", "input", {select_all=true}, 2, 1)
   d:addButton("ok", "Ok", "accept")
+  d:addButton("cancel", "Cancel", "reject")
   if start then d:set("text", start) end
   if d:execute() then
     return d:get("text")
@@ -249,7 +251,7 @@ end
 
 -- show a warning messageBox
 function MODEL:warning(text, details)
-  ipeui.messageBox(self.ui:win(), "warning", text, details)
+  messageBox(self.ui:win(), "warning", text, details)
 end
 
 -- Set canvas to current page
@@ -418,16 +420,25 @@ end
 
 ----------------------------------------------------------------------
 
+local function showSource(d)
+  local fname = config.latexdir .. "ipetemp.tex"
+  ipeui.waitDialog(d, string.format(prefs.external_editor, fname))
+end
+
 function MODEL:latexErrorBox(log)
   local d = ipeui.Dialog(self.ui:win(), "Ipe: error running Latex")
   d:add("label", "label",
 	{ label="An error occurred during the Pdflatex run. " ..
 	  "Please consult the logfile below." },
 	1, 1)
-  d:add("text", "text", { read_only=true, syntax="logfile" }, 2, 1)
+  d:add("text", "text", { read_only=true, syntax="logfile", focus=true }, 2, 1)
+  if prefs.external_editor then
+    d:addButton("editor", "&Source", function (d) showSource(d) end)
+  end
   d:addButton("ok", "Ok", "accept")
   d:set("text", log)
-  d:execute()
+  d:setStretch("row", 2, 1)
+  d:execute(prefs.latexlog_size)
 end
 
 function MODEL:runLatex()
@@ -454,7 +465,7 @@ end
 -- returns true if object is unmodified or user confirmed discarding
 function MODEL:checkModified()
   if self:isModified() then
-    return ipeui.messageBox(self.ui:win(), "question",
+    return messageBox(self.ui:win(), "question",
 				 "The document has been modified",
 				 "Do you wish to discard the current document?",
 				 "discardcancel") == 0
@@ -493,6 +504,14 @@ function MODEL:newDocument()
 end
 
 function MODEL:loadDocument(fname)
+  local err = self:tryLoadDocument(fname)
+  if err then
+    self:warning("Document '" .. fname .. "' could not be opened", err)
+  end
+end
+
+-- returns error message if there was an error
+function MODEL:tryLoadDocument(fname)
   local doc, err = ipe.Document(fname)
   if doc then
     self.doc = doc
@@ -519,11 +538,11 @@ function MODEL:loadDocument(fname)
     if #syms > 0 then
       self:warning("The document contains symbolic attributes " ..
 		   "that are not defined in the style sheet:",
-		 "<qt><ul><li>" .. table.concat(syms, "<li>")
-		 .. "</qt>")
+		 "* " .. table.concat(syms, "\n* "))
     end
+    return nil
   else
-    self:warning("Document '" .. fname .. "' could not be opened", err)
+    return err
   end
 end
 
@@ -534,49 +553,18 @@ function MODEL:saveDocument(fname)
   local fm = formatFromFileName(fname)
   if not fm then
     self:warning("File not saved!",
-		 "You must save as *.xml, *.ipe, *.pdf, or *.eps")
-    return
-  end
-
-  if fm == "eps" and self.doc:countTotalViews() > 1 then
-    self:warning("File not saved!",
-		 "Only single-view documents can be saved in EPS format")
+		 "You must save as *.xml, *.ipe, or *.pdf")
     return
   end
 
   -- run Latex if format is not XML
-  if (fm ~= "xml" or prefs.auto_export_xml_to_eps or
-      prefs.auto_export_xml_to_pdf) then
+  if fm ~= "xml" then
     if not self:runLatex() then
       self.ui:explain("Latex error - file not saved")
       return
     end
   end
 
-  if fm == "eps" then
-    local ok = true
-    local msg = "<qt>The document uses the following features:<ul>"
-    if self.doc:has("truetype") then
-      ok = false
-      msg = msg .. "<li>Truetype fonts"
-    end
-    if self.doc:has("gradients") then
-      ok = false
-      msg = msg .. "<li>Gradients"
-    end
-    if self.doc:has("transparency") then
-      ok = false
-      msg = msg .. "<li>Transparency"
-    end
-    if not ok then
-      msg = msg .. "</ul><p>These cannot be saved in EPS format.</p>"
-	.. "<p>You can save as PDF, and use pdftops to create a "
-	.. "Postscript file.</p></qt>"
-      self:warning("File not saved!", msg)
-      return
-    end
-  end
-
   local props = self.doc:properties()
   props.modified = "D:" .. ipeui.currentDateTime()
   if props.created == "" then props.created = props.modified end
@@ -588,20 +576,6 @@ function MODEL:saveDocument(fname)
     return
   end
 
-  -- auto-exporting
-  if fm == "xml" and prefs.auto_export_xml_to_eps then
-    self:auto_export("eps", fname)
-  end
-  if fm == "xml" and prefs.auto_export_xml_to_pdf then
-    self:auto_export("pdf", fname)
-  end
-  if fm == "eps" and prefs.auto_export_eps_to_pdf then
-    self:auto_export("pdf", fname)
-  end
-  if fm == "pdf" and prefs.auto_export_pdf_to_eps then
-    self:auto_export("eps", fname)
-  end
-
   self:markAsUnmodified()
   self.ui:explain("Saved document '" .. fname .. "'")
   self.file_name = fname
@@ -609,24 +583,6 @@ function MODEL:saveDocument(fname)
   return true
 end
 
-function MODEL:auto_export(fm, fname)
-  local ename = fname:sub(1,-4) .. fm
-  if prefs.auto_export_only_if_exists and not ipe.fileExists(ename) then
-    return
-  end
-  local result
-  if fm == "eps" then
-    result = self.doc:exportView(ename, fm, nil, 1, 1)
-  else
-    result = self.doc:save(ename, fm, { export=true } )
-  end
-  if not result then
-    self:warning("Auto-exporting failed",
-		 "I could not export in format '" .. fm ..
-		   "' to file '" .. ename .. "'.")
-  end
-end
-
 ----------------------------------------------------------------------
 
 function MODEL:autosave()
@@ -635,7 +591,7 @@ function MODEL:autosave()
   local f
   if prefs.autosave_filename:find("%%s") then
     if self.file_name then
-      f = self.file_name:match("/([^/]+)$") or self.file_name
+      f = self.file_name:match(prefs.basename_pattern) or self.file_name
     else
       f = "unnamed"
     end
@@ -645,8 +601,8 @@ function MODEL:autosave()
   end
   self.ui:explain("Autosaving to " .. f .. "...")
   if not self.doc:save(f, "xml") then
-    ipeui.messageBox(self.ui:win(), "critical",
-		     "Autosaving failed!")
+    messageBox(self.ui:win(), "critical",
+	       "Autosaving failed!\nFilename: " .. f)
   end
 end
 
@@ -655,10 +611,10 @@ end
 function MODEL:closeEvent()
   if self == first_model then first_model = nil end
   if self:isModified() then
-    local r = ipeui.messageBox(self.ui:win(), "question",
-				    "The document has been modified",
-				    "Do you wish to save the document?",
-				    "savediscardcancel")
+    local r = messageBox(self.ui:win(), "question",
+			 "The document has been modified",
+			 "Do you wish to save the document?",
+			 "savediscardcancel")
     if r == 1 then
       return self:action_save()
     else
@@ -692,6 +648,7 @@ function MODEL:register(t)
   if t.style_sheets_changed then
     self.ui:setupSymbolicNames(self.doc:sheets())
     self.ui:setAttributes(self.doc:sheets(), self.attributes)
+    self:resetGridSize()
   end
 end
 
@@ -770,7 +727,7 @@ function HELPER:message(m)
 end
 
 function HELPER:messageBox(text, details, buttons)
-  return ipeui.messageBox(self.model.ui:win(), nil, text, details, buttons)
+  return messageBox(self.model.ui:win(), nil, text, details, buttons)
 end
 
 function HELPER:getString(m)
@@ -835,6 +792,7 @@ function MODEL:action_undo()
   if t.style_sheets_changed then
     self.ui:setupSymbolicNames(self.doc:sheets())
     self.ui:setAttributes(self.doc:sheets(), self.attributes)
+    self:resetGridSize()
   end
 end
 
@@ -863,6 +821,7 @@ function MODEL:action_redo()
   if t.style_sheets_changed then
     self.ui:setupSymbolicNames(self.doc:sheets())
     self.ui:setAttributes(self.doc:sheets(), self.attributes)
+    self:resetGridSize()
   end
 end
 
