@@ -4,7 +4,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -33,9 +33,10 @@
 shortcuts = {
   new_window = nil,
   open = "Ctrl+O",
-  quit = "Ctrl+Q",
+  close = "Ctrl+W",
   save = "Ctrl+S",
   save_as = nil,
+  insert_image = "Ctrl+Shift+O",
   run_latex = "Ctrl+L",
   absolute = "Ctrl+T",
   undo = "Ctrl+Z",
@@ -49,8 +50,8 @@ shortcuts = {
   ungroup = "Ctrl+U",
   front = "Ctrl+F",
   back = "Ctrl+B",
-  forward = "Shift+Ctrl+F",
-  backward = "Shift+Ctrl+B",
+  forward = "Ctrl+Shift+F",
+  backward = "Ctrl+Shift+B",
   before = nil,
   behind = nil,
   duplicate = "D",
@@ -59,7 +60,7 @@ shortcuts = {
   insert_text_box = "F10",
   edit = "Ctrl+E",
   edit_as_xml = nil,
-  change_width = "Ctrl+W",
+  change_width = "Alt+W",
   document_properties = "Ctrl+Shift+P",
   style_sheets = "Ctrl+Shift+S",
   update_style_sheets = "Ctrl+Shift+U",
@@ -86,6 +87,7 @@ shortcuts = {
   mode_splinegons = "Shift+I",
   mode_ink = "K",
   snapvtx = "F4",
+  snapctl = "Shift+F4",
   snapbd = "F5",
   snapint = "F6",
   snapgrid = "F7",
@@ -163,9 +165,7 @@ shortcuts = {
 
 shortcuts_linestool = {
   spline = "s",
-  quadratic_bezier = "q",
   arc = "a",
-  cubic_bezier = "c",
   set_axis = "y",
 }
 
