@@ -1,10 +1,10 @@
 // --------------------------------------------------------------------
-// Special widgets for Win32
+// PathView widget for Win32
 // --------------------------------------------------------------------
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -37,7 +37,7 @@
 
 using namespace ipe;
 
-const char PathView::className[] = "ipePathViewClass";
+const wchar_t PathView::className[] = L"ipePathViewClass";
 
 // --------------------------------------------------------------------
 
@@ -46,15 +46,15 @@ PathView::PathView(HWND parent, int id)
   iCascade = 0;
   idBase = id;
 
-  HWND hwnd = CreateWindowEx(WS_EX_CLIENTEDGE, className, "",
-			     WS_CHILD | WS_VISIBLE,
-			     0, 0, 0, 0,
-			     parent, NULL,
-			     (HINSTANCE) GetWindowLong(parent, GWL_HINSTANCE),
-			     this);
+  HWND hwnd = CreateWindowExW(WS_EX_CLIENTEDGE, className, L"",
+			      WS_CHILD | WS_VISIBLE,
+			      0, 0, 0, 0,
+			      parent, NULL,
+			      (HINSTANCE) GetWindowLong(parent, GWL_HINSTANCE),
+			      this);
   if (hwnd == NULL) {
-    MessageBox(NULL, "PathView creation failed!", "Error!",
-	       MB_ICONEXCLAMATION | MB_OK);
+    MessageBoxA(NULL, "PathView creation failed!", "Error!",
+		MB_ICONEXCLAMATION | MB_OK);
     exit(9);
   }
   assert(GetWindowLongPtr(hwnd, GWLP_USERDATA));
@@ -216,8 +216,8 @@ void PathView::init(HINSTANCE hInstance)
   wc.hIconSm = NULL;
 
   if (!RegisterClassEx(&wc)) {
-    MessageBox(NULL, "PathView control registration failed!", "Error!",
-	       MB_ICONEXCLAMATION | MB_OK);
+    MessageBoxA(NULL, "PathView control registration failed!", "Error!",
+		MB_ICONEXCLAMATION | MB_OK);
     exit(9);
   }
 }
