@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -125,17 +125,16 @@ void LuaTool::mouseMove()
   lua_call(L, 1, 0);
 }
 
-bool LuaTool::key(int code, int modifiers, String text)
+bool LuaTool::key(String text, int modifiers)
 {
   lua_rawgeti(L, LUA_REGISTRYINDEX, iLuaTool);
   lua_getfield(L, -1, "key");
   lua_pushvalue(L, -2); // model
   lua_remove(L, -3);
-  lua_pushnumber(L, code);
-  push_modifiers(L, modifiers);
   push_string(L, text);
+  push_modifiers(L, modifiers);
   lua_State *L0 = L; // need to save L since
-  lua_call(L, 4, 1); // this may delete tool
+  lua_call(L, 3, 1); // this may delete tool
   return lua_toboolean(L0, 1);
 }
 
@@ -145,6 +144,8 @@ ShapeTool::ShapeTool(CanvasBase *canvas, lua_State *L0, int luatool)
   : LuaTool(canvas, L0, luatool)
 {
   iPen = 1.0;
+  iSnap = false;
+  iSkipLast = false;
 }
 
 void ShapeTool::draw(Painter &painter) const
@@ -172,7 +173,6 @@ void ShapeTool::draw(Painter &painter) const
     case ESplineCP:
       painter.setFill(Attribute(Color(0, 0, 800)));
       break;
-    case EBezierCP:
     case EMinor:
       painter.setFill(Attribute(Color(0, 800, 0)));
       break;
@@ -206,7 +206,6 @@ void ShapeTool::draw(Painter &painter) const
       painter.drawPath(EStrokedOnly);
       break;
     case ESplineCP:
-    case EBezierCP:
     case ERadius:
     case EMinor:
       painter.newPath();
@@ -231,6 +230,12 @@ void ShapeTool::draw(Painter &painter) const
   }
 }
 
+void ShapeTool::setSnapping(bool snap, bool skipLast)
+{
+  iSnap = snap;
+  iSkipLast = skipLast;
+}
+
 void ShapeTool::setShape(Shape shape, int which, double pen)
 {
   if (which == 1)
@@ -253,6 +258,24 @@ void ShapeTool::addMark(const Vector &v, TMarkType t)
   iMarks.push_back(m);
 }
 
+void ShapeTool::snapVtx(const Vector &mouse, Vector &pos,
+			double &bound, bool cp) const
+{
+  if (!iSnap)
+    return;
+  Matrix m;
+  if (iSkipLast && iShape.countSubPaths() == 1 &&
+      iShape.subPath(0)->asCurve()) {
+    const Curve *c = iShape.subPath(0)->asCurve();
+    if (!cp)
+      c->segment(0).cp(0).snap(mouse, pos, bound);
+    // skip last vertex of curve
+    for (int i = 0; i < c->countSegments() - 1; ++i)
+      c->segment(i).snapVtx(mouse, m, pos, bound, cp);
+  } else
+    iShape.snapVtx(mouse, m, pos, bound, cp);
+}
+
 // --------------------------------------------------------------------
 
 PasteTool::PasteTool(CanvasBase *canvas, lua_State *L0, int luatool,
