@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -64,7 +64,7 @@ public:
 
   virtual void mouseButton(int button, bool press);
   virtual void mouseMove();
-  virtual bool key(int code, int modifiers, String text);
+  virtual bool key(String text, int modifiers);
 protected:
   lua_State *L;
   int iLuaTool;
@@ -73,17 +73,20 @@ protected:
 
 class ShapeTool : public LuaTool {
 public:
-  enum TMarkType { EVertex = 1, ESplineCP, EBezierCP, ECenter,
+  enum TMarkType { EVertex = 1, ESplineCP, ECenter,
 		   ERadius, EMinor, ECurrent, EScissor,
 		   ENumMarkTypes };
 
   ShapeTool(CanvasBase *canvas, lua_State *L0, int luatool);
 
   void setShape(Shape shape, int which = 0, double pen=1.0);
+  void setSnapping(bool snap, bool skipLast);
   void clearMarks();
   void addMark(const Vector &v, TMarkType t);
 
   virtual void draw(Painter &painter) const;
+  virtual void snapVtx(const Vector &mouse, Vector &pos,
+		       double &bound, bool cp) const;
 private:
   double iPen;
   Shape iShape;
@@ -93,6 +96,8 @@ private:
     TMarkType t;
   };
   std::vector<SMark> iMarks;
+  bool iSnap;
+  bool iSkipLast;
 };
 
 class PasteTool : public LuaTool {
