@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -34,22 +34,11 @@
 
 #include "ipelua.h"
 
+#include "ipethumbs.h"
+
 #include <cstdio>
 #include <cstdlib>
 
-#ifdef IPEUI_QT
-#include "ipeselector_qt.h"
-#include "controls_qt.h"
-
-#include <QApplication>
-#include <QStatusBar>
-#include <QPainter>
-
-#include <QDialog>
-#include <QVBoxLayout>
-#include <QDialogButtonBox>
-#endif
-
 using namespace ipe;
 using namespace ipelua;
 
@@ -191,6 +180,7 @@ static int appui_setSnap(lua_State *L)
   luaL_checktype(L, 2, LUA_TTABLE);
   Snap snap = canvas->snap();
   snapFlag(L, snap.iSnap, "snapvtx", Snap::ESnapVtx);
+  snapFlag(L, snap.iSnap, "snapctl", Snap::ESnapCtl);
   snapFlag(L, snap.iSnap, "snapbd", Snap::ESnapBd);
   snapFlag(L, snap.iSnap, "snapint", Snap::ESnapInt);
   snapFlag(L, snap.iSnap, "snapgrid", Snap::ESnapGrid);
@@ -202,7 +192,7 @@ static int appui_setSnap(lua_State *L)
   lua_pop(L, 1);
   lua_getfield(L, -1, "gridsize");
   if (!lua_isnil(L, -1))
-    snap.iGridSize = luaL_checkint(L, -1);
+    snap.iGridSize = (int)luaL_checkinteger(L, -1);
   lua_pop(L, 1);
   lua_getfield(L, -1, "anglesize");
   if (!lua_isnil(L, -1))
@@ -210,7 +200,7 @@ static int appui_setSnap(lua_State *L)
   lua_pop(L, 1);
   lua_getfield(L, -1, "snap_distance");
   if (!lua_isnil(L, -1))
-    snap.iSnapDistance = luaL_checkint(L, -1);
+    snap.iSnapDistance = (int)luaL_checkinteger(L, -1);
   lua_pop(L, 1);
   lua_getfield(L, -1, "with_axes");
   if (!lua_isnil(L, -1))
@@ -239,20 +229,29 @@ static int appui_setAutoOrigin(lua_State *L)
 static int appui_update(lua_State *L)
 {
   CanvasBase *canvas = check_canvas(L, 1);
-  bool all = true;
-  if (lua_gettop(L) == 2)
-    all = lua_toboolean(L, 2);
-  if (all)
-    canvas->update();
-  else
-    canvas->updateTool();
+  if (lua_isnone(L, 2) || lua_isboolean(L, 2)) {
+    if (lua_isnone(L, 2) || lua_toboolean(L, 2))
+      canvas->update();
+    else
+      canvas->updateTool();
+  } else {
+    /*
+    Rect *r = check_rect(L, 2);
+    Vector bl = canvas->userToDev(r->topLeft());
+    Vector tr = canvas->userToDev(r->bottomRight());
+    canvas->invalidate(int(bl.x-1), int(bl.y-1),
+		       int(tr.x - bl.x + 2),
+		       int(tr.y - bl.y + 2));
+    */
+  }
   return 0;
 }
 
 static int appui_finishTool(lua_State *L)
 {
-  CanvasBase *canvas = check_canvas(L, 1);
-  canvas->finishTool();
+  AppUiBase **ui = check_appui(L, 1);
+  (*ui)->canvas()->finishTool();
+  (*ui)->explain("", 0);
   return 0;
 }
 
@@ -304,6 +303,16 @@ static int appui_setNumbering(lua_State *L)
   return 0;
 }
 
+static int appui_setPretty(lua_State *L)
+{
+  CanvasBase *canvas = check_canvas(L, 1);
+  bool t = lua_toboolean(L, 2);
+  CanvasBase::Style s = canvas->canvasStyle();
+  s.pretty = t;
+  canvas->setCanvasStyle(s);
+  return 0;
+}
+
 // --------------------------------------------------------------------
 
 static int appui_pantool(lua_State *L)
@@ -368,7 +377,7 @@ static int shapetool_setshape(lua_State *L)
   Shape shape = check_shape(L, 1);
   int which = 0;
   if (!lua_isnoneornil(L, 2))
-    which = luaL_checkint(L, 2);
+    which = (int)luaL_checkinteger(L, 2);
   double pen = 1.0;
   if (lua_isnumber(L, 3))
     pen = luaL_checknumber(L, 3);
@@ -376,6 +385,15 @@ static int shapetool_setshape(lua_State *L)
   return 0;
 }
 
+static int shapetool_setsnapping(lua_State *L)
+{
+  ShapeTool *tool = (ShapeTool *) lua_touserdata(L, lua_upvalueindex(1));
+  bool snap = lua_toboolean(L, 1);
+  bool skipLast = lua_toboolean(L, 2);
+  tool->setSnapping(snap, skipLast);
+  return 0;
+}
+
 static int shapetool_setmarks(lua_State *L)
 {
   ShapeTool *tool = (ShapeTool *) lua_touserdata(L, lua_upvalueindex(1));
@@ -423,6 +441,9 @@ static int appui_shapetool(lua_State *L)
   lua_pushlightuserdata(L, tool);
   lua_pushcclosure(L, shapetool_setmarks, 1);
   lua_setfield(L, -2, "setMarks");
+  lua_pushlightuserdata(L, tool);
+  lua_pushcclosure(L, shapetool_setsnapping, 1);
+  lua_setfield(L, -2, "setSnapping");
   canvas->setTool(tool);
   return 0;
 }
@@ -462,12 +483,25 @@ static int appui_close(lua_State *L)
   return 0;
 }
 
-static int appui_actionId(lua_State *L)
+static int appui_clipboard(lua_State *L)
+{
+  AppUiBase **ui = check_appui(L, 1);
+  return (*ui)->clipboard(L);
+}
+
+static int appui_setClipboard(lua_State *L)
+{
+  AppUiBase **ui = check_appui(L, 1);
+  return (*ui)->setClipboard(L);
+}
+
+static int appui_actionInfo(lua_State *L)
 {
   AppUiBase **ui = check_appui(L, 1);
   const char *action = luaL_checkstring(L, 2);
   lua_pushinteger(L, (*ui)->actionId(action));
-  return 1;
+  lua_pushboolean(L, (*ui)->actionAlwaysOn(action));
+  return 2;
 }
 
 static int appui_actionState(lua_State *L)
@@ -596,9 +630,8 @@ static int appui_explain(lua_State *L)
 
 // --------------------------------------------------------------------
 
-#if IPEUI_QT
-static void get_page_sorter_size(lua_State *L, int &width, int &height,
-				 int &thumbWidth)
+void get_page_sorter_size(lua_State *L, int &width, int &height,
+			  int &thumbWidth)
 {
   thumbWidth = 300;
   width = 600;
@@ -622,13 +655,10 @@ static void get_page_sorter_size(lua_State *L, int &width, int &height,
     thumbWidth = lua_tointeger(L, -1);
   lua_pop(L, 1); // thumbnail_width, prefs
 }
-#endif
 
 static int appui_selectPage(lua_State *L)
 {
-#if IPEUI_QT
-  AppUiBase **ui = check_appui(L, 1);
-  (void) ui;  // not actually used now
+  check_appui(L, 1);  // not actually used now
   Document **doc = check_document(L, 2);
   int page = -1;
   if (lua_isnumber(L, 3)) {
@@ -649,57 +679,44 @@ static int appui_selectPage(lua_State *L)
   int width, height, thumbWidth;
   get_page_sorter_size(L, width, height, thumbWidth);
 
-  int sel = PageSelector::selectPageOrView(*doc, page - 1, startIndex - 1,
-					   thumbWidth, width, height);
+  int sel = CanvasBase::selectPageOrView(*doc, page - 1, startIndex - 1,
+					 thumbWidth, width, height);
   if (sel >= 0) {
     lua_pushinteger(L, sel + 1);
     return 1;
   } else
-#endif
     return 0;
 }
 
 static int appui_pageSorter(lua_State *L)
 {
-#ifdef IPEUI_QT
   AppUiBase **ui = check_appui(L, 1);
-  (void) ui;  // not actually used now
   Document **doc = check_document(L, 2);
 
   int width, height, thumbWidth;
   get_page_sorter_size(L, width, height, thumbWidth);
 
-  QDialog *d = new QDialog();
-  d->setWindowTitle("Ipe Page Sorter");
+  return (*ui)->pageSorter(L, *doc, width, height, thumbWidth);
+}
 
-  QLayout *lo = new QVBoxLayout;
-  PageSorter *p = new PageSorter(*doc, thumbWidth);
-  QDialogButtonBox *buttonBox =
-    new QDialogButtonBox(QDialogButtonBox::Ok|QDialogButtonBox::Cancel);
-  lo->addWidget(p);
-  lo->addWidget(buttonBox);
-  d->setLayout(lo);
+static const char *const render_formats[] = {"svg", "png", "eps", "pdf", NULL };
 
-  d->connect(buttonBox, SIGNAL(accepted()), SLOT(accept()));
-  d->connect(buttonBox, SIGNAL(rejected()), SLOT(reject()));
-
-  d->resize(width, height);
-
-  if (d->exec() == QDialog::Rejected) {
-    delete d;
-    return 0;
-  }
-
-  lua_createtable(L, p->count(), 0);
-  for (int i = 1; i <= p->count(); ++i) {
-    lua_pushinteger(L, p->pageAt(i-1) + 1);
-    lua_rawseti(L, -2, i);
-  }
-  delete d;
-  return 1;
-#else
+static int appui_renderPage(lua_State *L)
+{
+  // AppUiBase **ui = check_appui(L, 1);  // not used
+  Document **doc = check_document(L, 2);
+  int pageno = luaL_checkinteger(L, 3);
+  int viewno = luaL_checkinteger(L, 4);
+  Thumbnail::TargetFormat fm =
+    Thumbnail::TargetFormat(luaL_checkoption(L, 5, NULL, render_formats));
+  const char *dst = luaL_checkstring(L, 6);
+  double zoom = luaL_checknumber(L, 7);
+  bool transparent = lua_toboolean(L, 8);
+  bool nocrop = lua_toboolean(L, 9);
+  Thumbnail tn(*doc, 0);
+  const Page *page = (*doc)->page(pageno - 1);
+  tn.saveRender(fm, dst, page, viewno - 1, zoom, transparent, nocrop);
   return 0;
-#endif
 }
 
 // --------------------------------------------------------------------
@@ -727,6 +744,7 @@ static const struct luaL_Reg appui_methods[] = {
   { "canvasSize", appui_canvasSize },
   { "setCursor", appui_setCursor },
   { "setNumbering", appui_setNumbering },
+  { "setPretty", appui_setPretty },
   // --------------------------------------------------------------------
   { "panTool", appui_pantool },
   { "selectTool", appui_selecttool },
@@ -736,9 +754,11 @@ static const struct luaL_Reg appui_methods[] = {
   // --------------------------------------------------------------------
   { "win", appui_win},
   { "close", appui_close},
+  { "setClipboard", appui_setClipboard },
+  { "clipboard", appui_clipboard },
   { "setActionState", appui_setActionState },
   { "actionState", appui_actionState },
-  { "actionId", appui_actionId },
+  { "actionInfo", appui_actionInfo },
   { "explain", appui_explain },
   { "setWindowTitle", appui_setWindowTitle },
   { "setupSymbolicNames", appui_setupSymbolicNames },
@@ -751,6 +771,7 @@ static const struct luaL_Reg appui_methods[] = {
   { "showTool", appui_showTool },
   { "selectPage", appui_selectPage },
   { "pageSorter", appui_pageSorter },
+  { "renderPage", appui_renderPage },
   { 0, 0},
 };
 
