@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -30,6 +30,7 @@
 
 #include "ipexml.h"
 #include "ipeattributes.h"
+#include <cstdlib>
 
 using namespace ipe;
 
@@ -146,7 +147,6 @@ private:
   void writeTag(String tag, const XmlAttributes &att, String data = String());
 private:
   Stream &iStream;
-  double iTextStretch;
   Rect iMediaBox;
   bool iUseCropBox;
 };
@@ -643,12 +643,12 @@ int main(int argc, char *argv[])
       dst = s + ".ipe7";
   }
 
-  std::FILE *fd = std::fopen(src, "rb");
+  std::FILE *fd = Platform::fopen(src, "rb");
   if (!fd) {
     std::fprintf(stderr, "Could not open '%s'\n", src);
     exit(1);
   }
-  std::FILE *out = std::fopen(dst.z(), "wb");
+  std::FILE *out = Platform::fopen(dst.z(), "wb");
   if (!out) {
     fprintf(stderr, "Could not open '%s' for writing.\n", dst.z());
     exit(3);
