@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -203,9 +203,9 @@ void Engine::findStandardFonts()
 {
   iFontMapLoaded = true;
   String fm = Platform::fontmapFile();
-  int i = fm.rfind(Platform::pathSeparator());
+  int i = fm.rfind(IPESEP);
   String fmb = fm.left(i+1);
-  std::FILE *file = std::fopen(fm.z(), "rb");
+  std::FILE *file = Platform::fopen(fm.z(), "rb");
   if (!file) {
     ipeDebug("Could not open fontmap '%s'", fm.z());
     return;
@@ -218,7 +218,7 @@ void Engine::findStandardFonts()
       if (!parser.iGlyphs[i].empty()) {
 	String fname = parser.iGlyphs[i];
 	// make relative paths based on fmb
-#ifdef __MINGW32__
+#ifdef WIN32
 	if (fname[0] != '\\' && fname[1] != ':')
 	  fname = fmb + fname;
 #else
