@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -41,12 +41,17 @@ namespace ipe {
 
   class Thumbnail {
   public:
+    enum TargetFormat { ESVG, EPNG, EPS, EPDF };
+
     Thumbnail(const Document *doc, int width);
     ~Thumbnail();
 
     int height() const { return iHeight; }
     Buffer render(const Page *page, int view);
-
+    void saveRender(TargetFormat fm, const char *dst,
+		    const Page *page, int view, double zoom,
+		    bool transparent, bool nocrop);
+    static void savePNG(cairo_surface_t* surface, const char *dst);
   private:
     const Document *iDoc;
     int iWidth;
