@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -45,6 +45,9 @@ using namespace ipe;
 
   These classes are not in Ipelib, but in a separate library
   libipecanvas.
+
+  On a Mac OS Retina display, the pixel coordinates used by Ipe refer
+  to the actual device pixels.
 */
 
 // --------------------------------------------------------------------
@@ -73,10 +76,12 @@ CanvasBase::CanvasBase()
   iDimmed = false;
   iWidth = 0;   // not yet known (canvas is not yet mapped)
   iHeight = 0;
+  iRetina = 1;
   iRepaintObjects = false;
   iFonts = 0;
   iAutoSnap = false;
   iFifiVisible = false;
+  iFifiMode = Snap::ESnapNone;
   iSelectionVisible = true;
 
   iAdditionalModifiers = 0;
@@ -157,6 +162,13 @@ void CanvasBase::setZoom(double zoom)
   iZoom = zoom;
 }
 
+//! Set MacOS Retina display.
+void CanvasBase::setRetina(int r)
+{
+  iRetina = r;
+  update();
+}
+
 //! Set the snapping information.
 void CanvasBase::setSnap(const Snap &s)
 {
@@ -394,6 +406,94 @@ void CanvasBase::drawObjects(cairo_t *cc)
 
 // --------------------------------------------------------------------
 
+static void draw_plus(const Vector &p, cairo_t *cr)
+{
+  cairo_move_to(cr, p.x - 8, p.y);
+  cairo_line_to(cr, p.x + 8, p.y);
+  cairo_move_to(cr, p.x, p.y - 8);
+  cairo_line_to(cr, p.x, p.y + 8);
+  cairo_stroke(cr);
+}
+
+static void draw_rhombus(const Vector &p, cairo_t *cr)
+{
+  cairo_move_to(cr, p.x - 8, p.y);
+  cairo_line_to(cr, p.x, p.y + 8);
+  cairo_line_to(cr, p.x + 8, p.y);
+  cairo_line_to(cr, p.x, p.y - 8);
+  cairo_close_path(cr);
+  cairo_stroke(cr);
+}
+
+static void draw_square(const Vector &p, cairo_t *cr)
+{
+  cairo_move_to(cr, p.x - 7, p.y - 7);
+  cairo_line_to(cr, p.x + 7, p.y - 7);
+  cairo_line_to(cr, p.x + 7, p.y + 7);
+  cairo_line_to(cr, p.x - 7, p.y + 7);
+  cairo_close_path(cr);
+  cairo_stroke(cr);
+}
+
+static void draw_x(const Vector &p, cairo_t *cr)
+{
+  cairo_move_to(cr, p.x - 5.6, p.y - 5.6);
+  cairo_line_to(cr, p.x + 5.6, p.y + 5.6);
+  cairo_move_to(cr, p.x - 5.6, p.y + 5.6);
+  cairo_line_to(cr, p.x + 5.6, p.y - 5.6);
+  cairo_stroke(cr);
+}
+
+static void draw_star(const Vector &p, cairo_t *cr)
+{
+  cairo_move_to(cr, p.x - 8, p.y);
+  cairo_line_to(cr, p.x + 8, p.y);
+  cairo_move_to(cr, p.x - 4, p.y + 7);
+  cairo_line_to(cr, p.x + 4, p.y - 7);
+  cairo_move_to(cr, p.x - 4, p.y - 7);
+  cairo_line_to(cr, p.x + 4, p.y + 7);
+  cairo_stroke(cr);
+}
+
+void CanvasBase::drawFifi(cairo_t *cr)
+{
+  Vector p = userToDev(iMousePos);
+  switch (iFifiMode) {
+  case Snap::ESnapNone:
+    // don't draw at all
+    break;
+  case Snap::ESnapVtx:
+    cairo_set_source_rgb(cr, 1.0, 0.0, 0.0);
+    draw_rhombus(p, cr);
+    break;
+  case Snap::ESnapCtl:
+    cairo_set_source_rgb(cr, 1.0, 0.0, 0.0);
+    draw_square(p, cr);
+    break;
+  case Snap::ESnapBd:
+    cairo_set_source_rgb(cr, 1.0, 0.0, 0.0);
+    draw_plus(p, cr);
+    break;
+  case Snap::ESnapInt:
+    cairo_set_source_rgb(cr, 1.0, 0.0, 0.0);
+    draw_x(p, cr);
+    break;
+  case Snap::ESnapGrid:
+    cairo_set_source_rgb(cr, 0.0, 0.5, 0.0);
+    draw_plus(p, cr);
+    break;
+  case Snap::ESnapAngle:
+  case Snap::ESnapAuto:
+  default:
+    cairo_set_source_rgb(cr, 1.0, 0.0, 0.0);
+    draw_star(p, cr);
+    break;
+  }
+  iOldFifi = p;
+}
+
+// --------------------------------------------------------------------
+
 //! Draw the current canvas tool.
 /*! If no tool is set, it draws the selected objects. */
 void CanvasBase::drawTool(Painter &painter)
@@ -453,7 +553,7 @@ void CanvasBase::finishTool()
 
 // --------------------------------------------------------------------
 
-bool CanvasBase::snapToPaperAndFrame()
+void CanvasBase::snapToPaperAndFrame()
 {
   double snapDist = iSnap.iSnapDistance / iZoom;
   double d = snapDist;
@@ -477,7 +577,8 @@ bool CanvasBase::snapToPaperAndFrame()
   // Return if snapping has occurred
   if (d < snapDist) {
     iMousePos = fifi;
-    return true;
+    iFifiMode = Snap::ESnapVtx;
+    return;
   }
 
   // boundary
@@ -494,10 +595,8 @@ bool CanvasBase::snapToPaperAndFrame()
 
   if (d < snapDist) {
     iMousePos = fifi;
-    return true;
+    iFifiMode = Snap::ESnapBd;
   }
-
-  return true;
 }
 
 // --------------------------------------------------------------------
@@ -547,7 +646,7 @@ void CanvasBase::updateTool()
   snapping is enabled, and stores snapped position in iMousePos. */
 void CanvasBase::computeFifi(double x, double y)
 {
-  iUnsnappedMousePos = devToUser(Vector(x, y));
+  iUnsnappedMousePos = devToUser(Vector(iRetina * x, iRetina * y));
   iMousePos = iUnsnappedMousePos;
 
   if (!iPage)
@@ -555,19 +654,23 @@ void CanvasBase::computeFifi(double x, double y)
 
   int mask = iAutoSnap ? 0 : Snap::ESnapAuto;
   if (iSnap.iSnap & ~mask) {
-    if (!iSnap.snap(iMousePos, iPage, iSnap.iSnapDistance / iZoom,
-		    (iAutoSnap ? &iAutoOrigin : 0)))
+    iFifiMode = iSnap.snap(iMousePos, iPage, iSnap.iSnapDistance / iZoom,
+			   iTool, (iAutoSnap ? &iAutoOrigin : 0));
+    if (iFifiMode == Snap::ESnapNone)
       snapToPaperAndFrame();
 
     // convert fifi coordinates back into device space
     Vector fifi = userToDev(iMousePos);
     if (iFifiVisible && fifi != iOldFifi) {
-      invalidate(int(iOldFifi.x - 10), int(iOldFifi.y - 10), 21, 21);
-      invalidate(int(fifi.x - 10), int(fifi.y - 10), 21, 21);
+      invalidate(int(iOldFifi.x / iRetina - 10),
+		 int(iOldFifi.y / iRetina - 10), 21, 21);
+      invalidate(int(fifi.x / iRetina - 10),
+		 int(fifi.y / iRetina - 10), 21, 21);
     }
   } else if (iFifiVisible) {
     // remove old fifi
-    invalidate(int(iOldFifi.x - 10), int(iOldFifi.y - 10), 21, 21);
+    invalidate(int(iOldFifi.x / iRetina - 10),
+	       int(iOldFifi.y / iRetina - 10), 21, 21);
     iFifiVisible = false;
   }
 }
