@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -124,6 +124,7 @@ namespace ipe {
 
     Matrix canvasTfm() const;
 
+    void setRetina(int r);
     void setObserver(CanvasObserver *observer);
 
     void setFifiVisible(bool visible);
@@ -132,6 +133,8 @@ namespace ipe {
     void setTool(Tool *tool);
     void finishTool();
 
+    Tool *tool() { return iTool; }
+
     void update();
     void updateTool();
 
@@ -141,6 +144,13 @@ namespace ipe {
     virtual void setCursor(TCursor cursor, double w = 1.0,
 			   Color *color = 0) = 0;
 
+    static int selectPageOrView(Document *doc, int page = -1,
+				int startIndex = 0,
+				int pageWidth = 240,
+				int width = 600, int height = 480);
+
+    virtual void invalidate(int x, int y, int w, int h) = 0;
+
   protected:
     CanvasBase();
     void drawPaper(cairo_t *cc);
@@ -149,12 +159,12 @@ namespace ipe {
     void drawGrid(cairo_t *cc);
     void drawObjects(cairo_t *cc);
     void drawTool(Painter &painter);
-    bool snapToPaperAndFrame();
+    void snapToPaperAndFrame();
     void refreshSurface();
     void computeFifi(double x, double y);
+    void drawFifi(cairo_t *cr);
 
     virtual void invalidate() = 0;
-    virtual void invalidate(int x, int y, int w, int h) = 0;
 
   protected:
     CanvasObserver *iObserver;
@@ -174,6 +184,8 @@ namespace ipe {
     Vector iAutoOrigin;
     int iAdditionalModifiers;
 
+    int iRetina;
+
     bool iRepaintObjects;
     int iWidth, iHeight;
     cairo_surface_t *iSurface;
@@ -183,6 +195,7 @@ namespace ipe {
     Vector iGlobalPos;
     Vector iOldFifi;  // last fifi position that has been drawn
     bool iFifiVisible;
+    Snap::TSnapModes iFifiMode;
     bool iSelectionVisible;
 
     Fonts *iFonts;
