@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -42,9 +42,14 @@ void Canvas::invalidate()
 {
   GdkRectangle r;
   r.x = r.y = 0;
+#if GTK_MAJOR_VERSION >= 3
+  r.width = gtk_widget_get_allocated_width(iWindow);
+  r.height = gtk_widget_get_allocated_height(iWindow);
+#else
   r.width = iWindow->allocation.width;
   r.height = iWindow->allocation.height;
-  gdk_window_invalidate_rect(iWindow->window, &r, FALSE);
+#endif
+  gdk_window_invalidate_rect(gtk_widget_get_window(iWindow), &r, FALSE);
 }
 
 void Canvas::invalidate(int x, int y, int w, int h)
@@ -52,7 +57,7 @@ void Canvas::invalidate(int x, int y, int w, int h)
   GdkRectangle r;
   r.x = x; r.y = y;
   r.width = w; r.height = h;
-  gdk_window_invalidate_rect(iWindow->window, &r, FALSE);
+  gdk_window_invalidate_rect(gtk_widget_get_window(iWindow), &r, FALSE);
 }
 
 // --------------------------------------------------------------------
@@ -89,31 +94,33 @@ void Canvas::scrollHandler(GdkEventScroll *event)
     iObserver->canvasObserverWheelMoved(zDelta);
 }
 
-void Canvas::exposeHandler(GdkEventExpose *event)
+#if GTK_MAJOR_VERSION >= 3
+void Canvas::exposeHandler(cairo_t *cr)
 {
+  iWidth = gtk_widget_get_allocated_width(iWindow);
+  iHeight = gtk_widget_get_allocated_height(iWindow);
+#else
+void Canvas::exposeHandler(GdkEventExpose *event)
+ {
   iWidth = iWindow->allocation.width;
   iHeight = iWindow->allocation.height;
+#endif
 
   refreshSurface();
 
+#if GTK_MAJOR_VERSION < 3
   cairo_t *cr = gdk_cairo_create(iWindow->window);
   cairo_rectangle(cr, event->area.x, event->area.y,
 		  event->area.width, event->area.height);
   cairo_clip(cr);
+#endif
 
   cairo_set_source_surface(cr, iSurface, 0.0, 0.0);
   cairo_paint(cr);
 
-  if (iFifiVisible) {
-    Vector p = userToDev(iMousePos);
-    cairo_set_source_rgb(cr, 1.0, 0.0, 0.0);
-    cairo_move_to(cr, p.x - 8, p.y);
-    cairo_line_to(cr, p.x + 8, p.y);
-    cairo_move_to(cr, p.x, p.y - 8);
-    cairo_line_to(cr, p.x, p.y + 8);
-    cairo_stroke(cr);
-    iOldFifi = p;
-  }
+  if (iFifiVisible)
+    drawFifi(cr);
+
   if (iPage) {
     CairoPainter cp(iCascade, iFonts, cr, iZoom, false);
     cp.transform(canvasTfm());
@@ -121,7 +128,9 @@ void Canvas::exposeHandler(GdkEventExpose *event)
     drawTool(cp);
     cp.popMatrix();
   }
+#if GTK_MAJOR_VERSION < 3
   cairo_destroy(cr);
+#endif
 }
 
 // --------------------------------------------------------------------
@@ -132,11 +141,19 @@ gboolean Canvas::button_cb(GtkWidget *widget, GdkEvent *event, Canvas *canvas)
   return TRUE;
 }
 
+#if GTK_MAJOR_VERSION < 3
 gboolean Canvas::expose_cb(GtkWidget *widget, GdkEvent *event, Canvas *canvas)
 {
   canvas->exposeHandler((GdkEventExpose *) event);
   return TRUE;
 }
+#else
+gboolean Canvas::expose_cb(GtkWidget *widget, cairo_t *cr, Canvas *canvas)
+{
+  canvas->exposeHandler(cr);
+  return TRUE;
+}
+#endif
 
 gboolean Canvas::motion_cb(GtkWidget *widget, GdkEvent *event, Canvas *canvas)
 {
@@ -170,8 +187,13 @@ Canvas::Canvas(GtkWidget *parent)
 		   G_CALLBACK(button_cb), this);
   g_signal_connect(G_OBJECT(iWindow), "button-press-event",
 		   G_CALLBACK(button_cb), this);
+#if GTK_MAJOR_VERSION < 3
   g_signal_connect(G_OBJECT(iWindow), "expose-event",
 		   G_CALLBACK(expose_cb), this);
+#else
+  g_signal_connect(G_OBJECT(iWindow), "draw",
+		   G_CALLBACK(expose_cb), this);
+#endif
   g_signal_connect(G_OBJECT(iWindow), "motion-notify-event",
 		   G_CALLBACK(motion_cb), this);
   g_signal_connect(G_OBJECT(iWindow), "scroll-event",
