@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -114,7 +114,11 @@ void IpeQtPainter::doDrawPath(TPathMode mode)
 // --------------------------------------------------------------------
 
 //! Construct a new canvas.
+#if QT_VERSION < 0x050000
 Canvas::Canvas(QWidget* parent, Qt::WFlags f)
+#else
+Canvas::Canvas(QWidget* parent, Qt::WindowFlags f)
+#endif
   : QWidget(parent, f)
 {
   setAttribute(Qt::WA_NoBackground);
@@ -193,33 +197,38 @@ static int convertModifiers(int qmod)
 
 void Canvas::keyPressEvent(QKeyEvent *ev)
 {
-  String s = IpeQ(ev->text());
   if (iTool &&
-      iTool->key(ev->key(),
-		 (convertModifiers(ev->modifiers()) | iAdditionalModifiers),
-		 s))
+      iTool->key(IpeQ(ev->text()),
+		 (convertModifiers(ev->modifiers()) | iAdditionalModifiers)))
     ev->accept();
   else
     ev->ignore();
 }
 
-void Canvas::mousePressEvent(QMouseEvent *ev)
+void Canvas::mouseButton(QMouseEvent *ev, int button, bool press)
 {
   iGlobalPos = Vector(ev->globalPos().x(), ev->globalPos().y());
   computeFifi(ev->x(), ev->y());
   int mod = convertModifiers(ev->modifiers()) | iAdditionalModifiers;
   if (iTool)
-    iTool->mouseButton(ev->button() | mod, true);
-  else if (iObserver)
-    iObserver->canvasObserverMouseAction(ev->button() | mod);
+    iTool->mouseButton(button | mod, press);
+  else if (press && iObserver)
+    iObserver->canvasObserverMouseAction(button | mod);
+}
+
+void Canvas::mouseDoubleClickEvent(QMouseEvent *ev)
+{
+  mouseButton(ev, 0x81, true);
+}
+
+void Canvas::mousePressEvent(QMouseEvent *ev)
+{
+  mouseButton(ev, ev->button(), true);
 }
 
 void Canvas::mouseReleaseEvent(QMouseEvent *ev)
 {
-  iGlobalPos = Vector(ev->globalPos().x(), ev->globalPos().y());
-  computeFifi(ev->x(), ev->y());
-  if (iTool)
-    iTool->mouseButton(ev->button(), false);
+  mouseButton(ev, ev->button(), false);
 }
 
 void Canvas::mouseMoveEvent(QMouseEvent *ev)
@@ -233,8 +242,13 @@ void Canvas::mouseMoveEvent(QMouseEvent *ev)
 
 void Canvas::tabletEvent(QTabletEvent *ev)
 {
+#if QT_VERSION < 0x050000
   Vector globalPos(ev->hiResGlobalPos().x(), ev->hiResGlobalPos().y());
   QPointF hiPos = ev->hiResGlobalPos() - (ev->globalPos() - ev->pos());
+#else
+  Vector globalPos(ev->hiResGlobalX(), ev->hiResGlobalY());
+  QPointF hiPos = ev->posF();
+#endif
   ev->accept();
 
   switch (ev->type()) {
@@ -280,29 +294,109 @@ void Canvas::wheelEvent(QWheelEvent *ev)
   ev->accept();
 }
 
+static void draw_plus(const Vector &p, QPainter &q, double s)
+{
+  q.drawLine(QPt(p - s * Vector(8, 0)), QPt(p + s * Vector(8, 0)));
+  q.drawLine(QPt(p - s * Vector(0, 8)), QPt(p + s * Vector(0, 8)));
+}
+
+static void draw_rhombus(const Vector &p, QPainter &q, double s)
+{
+ QPainterPath path;
+ path.moveTo(QPt(p - s * Vector(8, 0)));
+ path.lineTo(QPt(p + s * Vector(0, 8)));
+ path.lineTo(QPt(p + s * Vector(8, 0)));
+ path.lineTo(QPt(p + s * Vector(0, -8)));
+ path.closeSubpath();
+ q.drawPath(path);
+}
+
+static void draw_square(const Vector &p, QPainter &q, double s)
+{
+ QPainterPath path;
+ path.moveTo(QPt(p + s * Vector(-7, -7)));
+ path.lineTo(QPt(p + s * Vector(7, -7)));
+ path.lineTo(QPt(p + s * Vector(7, 7)));
+ path.lineTo(QPt(p + s * Vector(-7, 7)));
+ path.closeSubpath();
+ q.drawPath(path);
+}
+
+static void draw_x(const Vector &p, QPainter &q, double s)
+{
+  q.drawLine(QPt(p - s * Vector(5.6, 5.6)),
+	     QPt(p + s * Vector(5.6, 5.6)));
+  q.drawLine(QPt(p - s * Vector(5.6, -5.6)),
+	     QPt(p + s * Vector(5.6, -5.6)));
+}
+
+static void draw_star(const Vector &p, QPainter &q, double s)
+{
+  q.drawLine(QPt(p - s * Vector(8, 0)), QPt(p + s * Vector(8, 0)));
+  q.drawLine(QPt(p + s * Vector(-4, 7)), QPt(p + s * Vector(4, -7)));
+  q.drawLine(QPt(p + s * Vector(-4, -7)), QPt(p + s * Vector(4, 7)));
+}
+
+void Canvas::drawFifi(QPainter &q)
+{
+  Vector p = userToDev(iMousePos);
+  switch (iFifiMode) {
+  case Snap::ESnapNone:
+    // don't draw at all
+    break;
+  case Snap::ESnapVtx:
+    q.setPen(QColor(255, 0, 0, 255));
+    draw_rhombus(p, q, iRetina);
+    break;
+  case Snap::ESnapCtl:
+    q.setPen(QColor(255, 0, 0, 255));
+    draw_square(p, q, iRetina);
+    break;
+    break;
+  case Snap::ESnapBd:
+    q.setPen(QColor(255, 0, 0, 255));
+    draw_plus(p, q, iRetina);
+    break;
+  case Snap::ESnapInt:
+    q.setPen(QColor(255, 0, 0, 255));
+    draw_x(p, q, iRetina);
+    break;
+  case Snap::ESnapGrid:
+    q.setPen(QColor(0, 128, 0, 255));
+    draw_plus(p, q, iRetina);
+    break;
+  case Snap::ESnapAngle:
+  case Snap::ESnapAuto:
+  default:
+    q.setPen(QColor(255, 0, 0, 255));
+    draw_star(p, q, iRetina);
+    break;
+  }
+  iOldFifi = p;
+}
+
 void Canvas::paintEvent(QPaintEvent * ev)
 {
-  iWidth = width();
-  iHeight = height();
+  iWidth = iRetina * width();
+  iHeight = iRetina * height();
 
   refreshSurface();
 
   QPainter qPainter;
   qPainter.begin(this);
   QRect r = ev->rect();
+  QRect source(iRetina * r.left(), iRetina * r.top(),
+	       iRetina * r.width(), iRetina * r.height());
   QImage bits(cairo_image_surface_get_data(iSurface),
 	      iWidth, iHeight, QImage::Format_RGB32);
-  qPainter.drawImage(r.left(), r.top(), bits,
-		     r.left(), r.top(), r.width(), r.height());
+  // make sure that when the display is not a retina display,
+  // we still get a reasonable picture:
+  qPainter.setRenderHint(QPainter::SmoothPixmapTransform, true);
+  qPainter.drawImage(r, bits, source);
   qPainter.translate(-1, -1);
-  if (iFifiVisible) {
-    Vector p = userToDev(iMousePos);
-    qPainter.setPen(QColor(255, 0, 0, 255));
-    qPainter.drawLine(QPt(p - Vector(8, 0)), QPt(p + Vector(8, 0)));
-    qPainter.drawLine(QPt(p - Vector(0, 8)), QPt(p + Vector(0, 8)));
-    iOldFifi = p;
-  }
-  // -----------------------------------
+  qPainter.scale(1.0/iRetina, 1.0/iRetina);
+  if (iFifiVisible)
+    drawFifi(qPainter);
   if (iPage) {
     IpeQtPainter qp(iCascade, &qPainter);
     qp.transform(canvasTfm());
