@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -73,7 +73,11 @@ namespace ipe {
   class Canvas : public QWidget, public CanvasBase {
     Q_OBJECT
   public:
+#if QT_VERSION < 0x050000
     Canvas(QWidget* parent, Qt::WFlags f=0);
+#else
+    Canvas(QWidget* parent, Qt::WindowFlags f=0);
+#endif
 
     virtual void setCursor(TCursor cursor, double w = 1.0,
 			   Color *color = 0);
@@ -81,9 +85,12 @@ namespace ipe {
   protected:
     virtual void invalidate();
     virtual void invalidate(int x, int y, int w, int h);
+    void drawFifi(QPainter &q);
 
   protected:
     virtual void paintEvent(QPaintEvent *ev);
+    void mouseButton(QMouseEvent *ev, int button, bool press);
+    virtual void mouseDoubleClickEvent(QMouseEvent *ev);
     virtual void mousePressEvent(QMouseEvent *ev) ;
     virtual void mouseReleaseEvent(QMouseEvent *ev);
     virtual void mouseMoveEvent(QMouseEvent *ev);
