@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -28,6 +28,10 @@
 
 */
 
+// for touch gestures:  need at least Windows 7
+#define WINVER 0x0601
+#define _WIN32_WINNT 0x0601
+
 #include "ipecanvas_win.h"
 #include "ipecairopainter.h"
 #include "ipetool.h"
@@ -35,11 +39,16 @@
 #include <cairo.h>
 #include <cairo-win32.h>
 
+#include <windows.h>
 #include <windowsx.h>
 
 using namespace ipe;
 
-const char Canvas::className[] = "ipeCanvasWindowClass";
+typedef WINBOOL WINAPI (*LPFNGETGESTUREINFO)(HGESTUREINFO hGestureInfo,
+					     PGESTUREINFO pGestureInfo);
+static LPFNGETGESTUREINFO pGetGestureInfo = 0;
+
+const wchar_t Canvas::className[] = L"ipeCanvasWindowClass";
 
 // --------------------------------------------------------------------
 
@@ -61,28 +70,31 @@ void Canvas::invalidate(int x, int y, int w, int h)
 
 // --------------------------------------------------------------------
 
-int Canvas::getModifiers()
+void Canvas::key(WPARAM wParam, LPARAM lParam)
 {
   int mod = 0;
-  if (GetKeyState(VK_SHIFT))
+  if (GetKeyState(VK_SHIFT) < 0)
     mod |= EShift;
-  if (GetKeyState(VK_CONTROL))
+  if (GetKeyState(VK_CONTROL) < 0)
     mod |= EControl;
-  if (GetKeyState(VK_MENU))
+  if (GetKeyState(VK_MENU) < 0)
     mod |= EAlt;
-  return mod;
+  ipeDebug("Canvas::key(%x, %x) %x", wParam, lParam, mod);
+  String t;
+  t.append(char(wParam));
+  iTool->key(t, mod);
 }
 
-void Canvas::button(int button, bool down, int xPos, int yPos)
+void Canvas::button(int button, bool down, int mod, int xPos, int yPos)
 {
-  // ipeDebug("Canvas::button %d %d %d %d", button, down, xPos, yPos);
+  ipeDebug("Canvas::button %d %d %d %d", button, down, xPos, yPos);
   POINT p;
   p.x = xPos;
   p.y = yPos;
   ClientToScreen(hwnd, &p);
   iGlobalPos = Vector(p.x, p.y);
   computeFifi(xPos, yPos);
-  int mod = getModifiers() | iAdditionalModifiers;
+  mod |= iAdditionalModifiers;
   if (iTool)
     iTool->mouseButton(button | mod, down);
   else if (down && iObserver)
@@ -99,16 +111,15 @@ void Canvas::mouseMove(int xPos, int yPos)
     iObserver->canvasObserverPositionChanged();
 }
 
-void Canvas::wheel(int zDelta)
-{
-  // ipeDebug("Canvas::wheel %d", zDelta);
-  if (iObserver)
-    iObserver->canvasObserverWheelMoved(zDelta);
-}
-
 void Canvas::setCursor(TCursor cursor, double w, Color *color)
 {
-  // TODO
+  if (cursor == EDotCursor) {
+    // Windows switches to a dot automatically when using the pen,
+    // not implemented
+  } else {
+    // HandCursor, CrossCursor not implemented,
+    // but they are also not used in Ipe now.
+  }
 }
 
 // --------------------------------------------------------------------
@@ -145,16 +156,9 @@ void Canvas::wndPaint()
   cairo_set_source_surface(cr, iSurface, 0.0, 0.0);
   cairo_paint(cr);
 
-  if (iFifiVisible) {
-    Vector p = userToDev(iMousePos);
-    cairo_set_source_rgb(cr, 1.0, 0.0, 0.0);
-    cairo_move_to(cr, p.x - 8, p.y);
-    cairo_line_to(cr, p.x + 8, p.y);
-    cairo_move_to(cr, p.x, p.y - 8);
-    cairo_line_to(cr, p.x, p.y + 8);
-    cairo_stroke(cr);
-    iOldFifi = p;
-  }
+  if (iFifiVisible)
+    drawFifi(cr);
+
   if (iPage) {
     CairoPainter cp(iCascade, iFonts, cr, iZoom, false);
     cp.transform(canvasTfm());
@@ -172,15 +176,58 @@ void Canvas::wndPaint()
 
 // --------------------------------------------------------------------
 
-void Canvas::mouseButton(Canvas *canvas, int button, bool down, LPARAM lParam)
+void Canvas::mouseButton(Canvas *canvas, int button, bool down,
+			 WPARAM wParam, LPARAM lParam)
 {
   if (canvas) {
     int xPos = GET_X_LPARAM(lParam);
     int yPos = GET_Y_LPARAM(lParam);
-    canvas->button(button, down, xPos, yPos);
+    int mod = 0;
+    if (wParam & MK_SHIFT)
+      mod |= EShift;
+    if (wParam & MK_CONTROL)
+      mod |= EControl;
+    if (GetKeyState(VK_MENU) < 0)
+      mod |= EAlt;
+    canvas->button(button, down, mod, xPos, yPos);
   }
 }
 
+LRESULT Canvas::handlePanGesture(DWORD flags, POINTS &p)
+{
+  Vector v(p.x, -p.y);
+  if (flags & GF_BEGIN) {
+    iGestureStart = v;
+    iGesturePan = pan();
+  } else {
+    v = v - iGestureStart;
+    setPan(iGesturePan - (1.0/zoom()) * v);
+    update();
+  }
+  return 0;
+}
+
+LRESULT Canvas::handleZoomGesture(DWORD flags, POINTS &p, ULONG d)
+{
+  if (flags & GF_BEGIN) {
+    iGestureDist = d;
+    iGestureZoom = zoom();
+  } else {
+    POINT q = { p.x, p.y };
+    ScreenToClient(hwnd, &q);
+    Vector origin = devToUser(Vector(q.x, q.y));
+    Vector offset = iZoom * (pan() - origin);
+    double nzoom = iGestureZoom * d / iGestureDist;
+    setZoom(nzoom);
+    setPan(origin + (1.0/nzoom) * offset);
+    update();
+    if (iObserver)
+      // scroll wheel hasn't moved, but update display of ppi
+      iObserver->canvasObserverWheelMoved(0);
+  }
+  return 0;
+}
+
 LRESULT CALLBACK Canvas::wndProc(HWND hwnd, UINT message, WPARAM wParam,
 				 LPARAM lParam)
 {
@@ -195,10 +242,43 @@ LRESULT CALLBACK Canvas::wndProc(HWND hwnd, UINT message, WPARAM wParam,
       SetWindowLongPtr(hwnd, GWLP_USERDATA, (LONG_PTR) canvas);
     }
     break;
+  case WM_MOUSEACTIVATE:
+    if (canvas)
+      SetFocus(canvas->hwnd);
+    return MA_ACTIVATE;
+  case WM_CHAR:
+    if (canvas && canvas->iTool) {
+      canvas->key(wParam, lParam);
+      return 0;
+    }
+    break;
+  case WM_MOUSEWHEEL:
+    if (canvas && canvas->iObserver)
+      canvas->iObserver->canvasObserverWheelMoved
+	(GET_WHEEL_DELTA_WPARAM(wParam));
+    return 0;
   case WM_PAINT:
     if (canvas)
       canvas->wndPaint();
     return 0;
+  case WM_SIZE:
+    if (canvas)
+      canvas->updateSize();
+    break;
+  case WM_GESTURE: {
+    if (!pGetGestureInfo)
+      break;  // GetGestureInfo not available (Windows Vista or older)
+    GESTUREINFO gi = {0};
+    gi.cbSize = sizeof(gi);
+    BOOL res = pGetGestureInfo((HGESTUREINFO) lParam, &gi);
+    if (res && canvas) {
+      if (gi.dwID == GID_PAN)
+	return canvas->handlePanGesture(gi.dwFlags, gi.ptsLocation);
+      else if (gi.dwID == GID_ZOOM)
+	return canvas->handleZoomGesture(gi.dwFlags, gi.ptsLocation,
+					 gi.ullArguments);
+    }
+    break; }
   case WM_MOUSEMOVE:
     if (canvas) {
       int xPos = GET_X_LPARAM(lParam);
@@ -206,40 +286,33 @@ LRESULT CALLBACK Canvas::wndProc(HWND hwnd, UINT message, WPARAM wParam,
       canvas->mouseMove(xPos, yPos);
     }
     break;
-  case WM_SIZE:
-    if (canvas)
-      canvas->updateSize();
-    break;
   case WM_LBUTTONDOWN:
-    mouseButton(canvas, 1, true, lParam);
-    break;
   case WM_LBUTTONUP:
-    mouseButton(canvas, 1, false, lParam);
+    mouseButton(canvas, 1, message == WM_LBUTTONDOWN, wParam, lParam);
+    break;
+  case WM_LBUTTONDBLCLK:
+    mouseButton(canvas, 0x81, true, wParam, lParam);
     break;
   case WM_RBUTTONDOWN:
-    mouseButton(canvas, 2, true, lParam);
-    break;
   case WM_RBUTTONUP:
-    mouseButton(canvas, 2, false, lParam);
+    mouseButton(canvas, 2, message == WM_RBUTTONDOWN, wParam, lParam);
     break;
   case WM_MBUTTONDOWN:
-    mouseButton(canvas, 3, true, lParam);
-    break;
   case WM_MBUTTONUP:
-    mouseButton(canvas, 3, false, lParam);
-    break;
-  case WM_MOUSEWHEEL:
-    if (canvas)
-      canvas->wheel(GET_WHEEL_DELTA_WPARAM(wParam));
-    break;
-  case WM_GETMINMAXINFO:
-    {
-      LPMINMAXINFO mm = LPMINMAXINFO(lParam);
-      fprintf(stderr, "Canvas MINMAX %ld %ld\n", mm->ptMinTrackSize.x,
-	      mm->ptMinTrackSize.y);
-    }
+    mouseButton(canvas, 4, message == WM_MBUTTONDOWN, wParam, lParam);
     break;
+  case WM_XBUTTONDOWN:
+  case WM_XBUTTONUP: {
+    int but = (HIWORD(wParam) == XBUTTON2) ? 0x10 : 0x08;
+    mouseButton(canvas, but, message == WM_XBUTTONDOWN, wParam, lParam);
+    break; }
+  case WM_GETMINMAXINFO: {
+    LPMINMAXINFO mm = LPMINMAXINFO(lParam);
+    ipeDebug("Canvas MINMAX %ld %ld", mm->ptMinTrackSize.x,
+	     mm->ptMinTrackSize.y);
+    break; }
   case WM_DESTROY:
+    ipeDebug("Windows canvas is destroyed");
     delete canvas;
     break;
   default:
@@ -253,15 +326,15 @@ LRESULT CALLBACK Canvas::wndProc(HWND hwnd, UINT message, WPARAM wParam,
 
 Canvas::Canvas(HWND parent)
 {
-  HWND hwnd = CreateWindowEx(WS_EX_CLIENTEDGE, className, "",
-			     WS_CHILD | WS_VISIBLE,
-			     0, 0, 0, 0,
-			     parent, NULL,
-			     (HINSTANCE) GetWindowLong(parent, GWL_HINSTANCE),
-			     this);
+  HWND hwnd = CreateWindowExW(WS_EX_CLIENTEDGE, className, L"",
+			      WS_CHILD | WS_VISIBLE,
+			      0, 0, 0, 0,
+			      parent, NULL,
+			      (HINSTANCE) GetWindowLong(parent, GWL_HINSTANCE),
+			      this);
   if (hwnd == NULL) {
-    MessageBox(NULL, "Canvas creation failed!", "Error!",
-	       MB_ICONEXCLAMATION | MB_OK);
+    MessageBoxA(NULL, "Canvas creation failed!", "Error!",
+		MB_ICONEXCLAMATION | MB_OK);
     exit(9);
   }
   assert(GetWindowLongPtr(hwnd, GWLP_USERDATA));
@@ -271,7 +344,7 @@ void Canvas::init(HINSTANCE hInstance)
 {
   WNDCLASSEX wc;
   wc.cbSize = sizeof(WNDCLASSEX);
-  wc.style = 0; // CS_VREDRAW | CS_HREDRAW;
+  wc.style = CS_DBLCLKS; // CS_VREDRAW | CS_HREDRAW;
   wc.lpfnWndProc = wndProc;
   wc.cbClsExtra	 = 0;
   wc.cbWndExtra	 = 0;
@@ -283,16 +356,17 @@ void Canvas::init(HINSTANCE hInstance)
   wc.hIcon = NULL;
   wc.hIconSm = NULL;
 
-  if (!RegisterClassEx(&wc)) {
-    MessageBox(NULL, "Canvas control registration failed!", "Error!",
-	       MB_ICONEXCLAMATION | MB_OK);
+  if (!RegisterClassExW(&wc)) {
+    MessageBoxA(NULL, "Canvas control registration failed!", "Error!",
+		MB_ICONEXCLAMATION | MB_OK);
     exit(9);
   }
-}
 
-void ipe_init_canvas(HINSTANCE hInstance)
-{
-  Canvas::init(hInstance);
+  // Check if GetGestureInfo is available (i.e. at least Windows 7)
+  HMODULE hDll = LoadLibraryA("user32.dll");
+  if (hDll)
+    pGetGestureInfo = (LPFNGETGESTUREINFO)
+      GetProcAddress(hDll, "GetGestureInfo");
 }
 
 // --------------------------------------------------------------------
