@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -46,30 +46,37 @@ namespace ipe {
 
     HWND windowId() const { return hwnd; }
 
+    static HBITMAP createBitmap(unsigned char *p, int w, int h);
+
   private:
-    int getModifiers();
-    void button(int button, bool down, int xPos, int yPos);
+    void button(int button, bool down, int mod, int xPos, int yPos);
+    void key(WPARAM wParam, LPARAM lParam);
     void mouseMove(int xPos, int yPos);
-    void wheel(int zDelta);
     void wndPaint();
     void updateSize();
 
+    LRESULT handlePanGesture(DWORD flags, POINTS &p);
+    LRESULT handleZoomGesture(DWORD flags, POINTS &p, ULONG d);
+
     virtual void invalidate();
     virtual void invalidate(int x, int y, int w, int h);
     virtual void setCursor(TCursor cursor, double w, Color *color);
   private:
-    static const char className[];
+    static const wchar_t className[];
     static LRESULT CALLBACK wndProc(HWND hwnd, UINT Message,
 				    WPARAM wParam, LPARAM lParam);
     static void mouseButton(Canvas *canvas, int button, bool down,
-			    LPARAM lParam);
+			    WPARAM wParam, LPARAM lParam);
   private:
     HWND hwnd;
+    // Gesture support
+    Vector iGestureStart;
+    int iGestureDist;
+    Vector iGesturePan;
+    double iGestureZoom;
   };
 
 } // namespace
 
-extern void ipe_init_canvas(HINSTANCE hInstance);
-
 // --------------------------------------------------------------------
 #endif
