@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -32,6 +32,8 @@
 
 #include "ipethumbs.h"
 
+#include "ipecanvas.h"
+
 #include <QDialog>
 #include <QVBoxLayout>
 
@@ -39,8 +41,8 @@ using namespace ipe;
 
 // --------------------------------------------------------------------
 
-/*! \class ipeqt::PageSelector
-  \ingroup qtcanvas
+/*! \class ipe::PageSelector
+  \ingroup canvas
   \brief A Qt widget that displays a list of Ipe pages.
 */
 
@@ -127,18 +129,19 @@ void PageSelector::pageSelected(QListWidgetItem *item)
 
     If \a page is non-negative, all views of this page are shown, and
     the selected view number is returned. */
-int PageSelector::selectPageOrView(Document *doc, int page, int startIndex,
-				   int pageWidth, int width, int height)
+int CanvasBase::selectPageOrView(Document *doc, int page, int startIndex,
+				 int pageWidth, int width, int height)
 {
   QDialog *d = new QDialog();
-  d->setWindowTitle("Select page");
+  d->setWindowTitle((page >= 0) ? "Ipe: Select view" :
+		    "Ipe: Select page");
 
   QLayout *lo = new QVBoxLayout;
   PageSelector *p = new PageSelector(doc, page, startIndex, pageWidth);
   lo->addWidget(p);
   d->setLayout(lo);
 
-  connect(p, SIGNAL(selectionMade()), d, SLOT(accept()));
+  QWidget::connect(p, SIGNAL(selectionMade()), d, SLOT(accept()));
 
   d->resize(width, height);
   int result = d->exec();
