@@ -1,11 +1,11 @@
 // -*- C++ -*-
 // --------------------------------------------------------------------
-// ipeqt::PageSelector
+// ipe::PageSelector
 // --------------------------------------------------------------------
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -51,11 +51,6 @@ namespace ipe {
 
     int selectedIndex() const { return currentRow(); }
 
-    static int selectPageOrView(Document *doc, int page = -1,
-				int startIndex = 0,
-				int pageWidth = 240,
-				int width = 600, int height = 480);
-
   signals:
     void selectionMade();
 
