@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -32,31 +32,13 @@
 #ifndef IPETOOL_H
 #define IPETOOL_H
 
-#include "ipelib.h"
-
-using namespace ipe;
+#include "ipepage.h"
+#include "ipetoolbase.h"
 
 namespace ipe {
 
   class CanvasBase;
 
-  class Tool {
-  public:
-    virtual ~Tool();
-
-  public:
-    virtual void draw(Painter &painter) const = 0;
-    virtual void mouseButton(int button, bool press);
-    virtual void mouseMove();
-    virtual bool key(int code, int modifiers, String text);
-
-  protected:
-    Tool(CanvasBase *canvas);
-
-  protected:
-    CanvasBase *iCanvas;
-  };
-
   // --------------------------------------------------------------------
 
   class PanTool : public Tool {
@@ -82,7 +64,7 @@ namespace ipe {
     virtual void draw(Painter &painter) const;
     virtual void mouseButton(int button, bool press);
     virtual void mouseMove();
-    virtual bool key(int code, int modifiers, String text);
+    virtual bool key(String text, int modifiers);
 
   private:
     void ensurePrimary();
