@@ -4,7 +4,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -264,7 +264,7 @@ function parabola(model)
     local q1 = V(0.5*(a + b), a*b)
     local q2 = V(b, b*b)
 
-    local curve = { type="curve", closed=false; { type="quad", q0, q1, q2 } }
+    local curve = { type="curve", closed=false; { type="spline", q0, q1, q2 } }
 
     local obj = ipe.Path(model.attributes, { curve } )
     local stretch = 2.0 * mrk.y;
