@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@
 */
 
 #include <stdio.h>
-#include <stdlib.h>
+#include <cstdlib>
 
 extern "C" {
 #include "qhull_a.h"
@@ -140,11 +140,11 @@ void CollectVisitor::visitReference(const Reference *obj)
 // readpoints put points into structure for qhull
 //
 // returns:
-//  number of points, array of point coordinates, ismalloc True
+//  number of points, array of point coordinates
 //
 
 static coordT *readpoints(const std::vector<Vector> &sites, int mode,
-			  int *numpoints, boolT *ismalloc)
+			  int *numpoints)
 {
   coordT *points, *coords;
 
@@ -155,9 +155,8 @@ static coordT *readpoints(const std::vector<Vector> &sites, int mode,
   else if (mode == 3)
     *numpoints = sites.size() * (sites.size() - 1) * (sites.size() - 2) / 6;
 
-  qh normal_size= 3 * sizeof(coordT); /* for tracing with qh_printpoint */
-  *ismalloc= True; 		      /* use malloc since memory not setup */
-  coords=points=(coordT*)malloc(*numpoints*3*sizeof(coordT));
+  qh normal_size = 3 * sizeof(coordT); /* for tracing with qh_printpoint */
+  coords = points = new coordT[*numpoints * 3];
 
   if (mode < 2 || mode == 4) {
     for (uint i = 0; i < sites.size(); i++) {
@@ -326,6 +325,7 @@ void VoronoiIpelet::delaunayTreatFacet(facetT *facet)
 
 bool VoronoiIpelet::run(int function, IpeletData *data, IpeletHelper *helper)
 {
+  ipeDebug("VoronoiIpelet::run(%d)", function);
   if (function == 5) {
     char buf[32];
     std::sprintf(buf, "%g", iInfiniteEdgeLength);
@@ -348,26 +348,22 @@ bool VoronoiIpelet::run(int function, IpeletData *data, IpeletHelper *helper)
     return false;
   }
 
-  int numpoints = 0;
-  coordT *points;
-  boolT ismalloc;
-
+  ipeDebug("qh_meminit");
   qh_meminit(stderr);
   qh_initqhull_start(stdin, stdout, stderr);
 
+  int numpoints = 0;
+  coordT *points = 0;
+
   if (!setjmp(qh errexit)) {
-    // the command(s) for qhull
 
-    // strcpy (qh qhull_command, qhull_flags);
-    // fprintf(stderr, "Qhull flags = %s\n", qh qhull_command);
-    // qh_initflags (qh qhull_command);
-
-    points = readpoints(iSites, function, &numpoints, &ismalloc);
-    qh_initqhull_globals (points, numpoints, 3, ismalloc);
+    points = readpoints(iSites, function, &numpoints);
+    qh_initqhull_globals(points, numpoints, 3, False);
+    ipeDebug("qh_initqhull_mem()");
     qh_initqhull_mem();
     /* mem.c and set.c are initialized */
     qh_initqhull_buffers();
-    qh_initthresholds (qh qhull_command);
+    qh_initthresholds(qh qhull_command);
     if (qh SCALEinput)
       qh_scaleinput();
     if (qh ROTATErandom >= 0) {
@@ -377,7 +373,11 @@ bool VoronoiIpelet::run(int function, IpeletData *data, IpeletHelper *helper)
     }
     qh_qhull();
     qh_check_output();
+#ifndef WIN32
+    // this crashes on Windows.  Why?
+    ipeDebug("qh_produce_output()");
     qh_produce_output();
+#endif
     if (qh VERIFYoutput && !qh FORCEoutput && !qh STOPpoint && !qh STOPcone)
       qh_check_points();
 
@@ -409,10 +409,15 @@ bool VoronoiIpelet::run(int function, IpeletData *data, IpeletHelper *helper)
     }
     data->iPage->append(ESecondarySelected, data->iLayer, group);
   }
-  qh NOerrexit= True;  /* no more setjmp */
+  ipeDebug("qh_freehull(True)");
+  qh_freeqhull(True);
+
+  delete [] points;
+
   iEdges.clear();
   iSites.clear();
   iDelaunay.clear();
+
   return true;
 }
 
