@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -30,6 +30,7 @@
 
 #include "ipebase.h"
 #include <cmath>
+#include <cstdlib>
 
 using namespace ipe;
 
@@ -364,7 +365,7 @@ String Lex::nextToken()
 {
   skipWhitespace();
   uint mark = iPos;
-  while (!eos() && iString[iPos] > ' ')
+  while (!(eos() || uchar(iString[iPos]) <= ' '))
     ++iPos;
   return iString.substr(mark, iPos - mark);
 }
@@ -426,14 +427,13 @@ Fixed Lex::getFixed()
 //! Extract double token (skipping whitespace).
 double Lex::getDouble()
 {
-  String str = nextToken();
-  return std::atof(str.z());
+  return Platform::toDouble(nextToken());
 }
 
 //! Skip over whitespace.
 void Lex::skipWhitespace()
 {
-  while (!eos() && iString[iPos] <= ' ')
+  while (!eos() && uchar(iString[iPos]) <= ' ')
     ++iPos;
 }
 
