@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -37,7 +37,6 @@
 #include "ipefontpool.h"
 #include "ipepdfparser.h"
 #include "ipepdfwriter.h"
-#include "ipepswriter.h"
 #include "ipelatex.h"
 
 #include <errno.h>
@@ -318,7 +317,7 @@ Document *doParseXml(DataSource &source, int &reason)
 Document *doParsePs(DataSource &source, int &reason)
 {
   PsSource psSource(source);
-  reason = -3; // could not find Xml stream
+  reason = Document::EFileOpenError; // could not find Xml stream
   if (!psSource.skipToXml())
     return 0;
 
@@ -337,12 +336,11 @@ Document *doParsePs(DataSource &source, int &reason)
 Document *doParsePdf(DataSource &source, int &reason)
 {
   PdfFile loader;
-  reason = -2; // could not parse
-  if (!loader.parse(source))
+  reason = Document::ENotAnIpeFile;
+  if (!loader.parse(source))  // could not parse PDF container
     return 0;
-
-  reason = -3; // not an Ipe document
   const PdfObj *obj = loader.object(1);
+  // was the object really created by Ipe?
   if (!obj || !obj->dict())
     return 0;
   const PdfObj *type = obj->dict()->get("Type", 0);
@@ -381,13 +379,14 @@ Document *Document::load(DataSource &source, TFormat format,
   if (format == EEps)
     return doParsePs(source, reason);
 
+  reason = (format == EIpe5) ? EVersionTooOld : ENotAnIpeFile;
   return 0;
 }
 
 Document *Document::load(const char *fname, int &reason)
 {
   reason = EFileOpenError;
-  std::FILE *fd = std::fopen(fname, "rb");
+  std::FILE *fd = Platform::fopen(fname, "rb");
   if (!fd)
     return 0;
   FileSource source(fd);
@@ -418,6 +417,9 @@ Document *Document::loadWithErrorReport(const char *fname)
   case Document::EFileOpenError:
     perror("Error opening the file");
     break;
+  case Document::ENotAnIpeFile:
+    fprintf(stderr, "The document was not created by Ipe.\n");
+    break;
   default:
     fprintf(stderr, "Error parsing the document at position %d\n.", reason);
     break;
@@ -466,26 +468,12 @@ bool Document::save(TellStream &stream, TFormat format, uint flags) const
     return true;
   }
 
-  if (format == EEps) {
-    if (countTotalViews() != 1)
-      return false;
-
-    PsWriter writer(stream, this, (flags & ENoColor));
-    if (!writer.createHeader())
-      return false;
-    writer.createPageView(0, 0);
-    if (!(flags & EExport))
-      writer.createXml(compresslevel);
-    writer.createTrailer();
-    return true;
-  }
-
   return false;
 }
 
 bool Document::save(const char *fname, TFormat format, uint flags) const
 {
-  std::FILE *fd = std::fopen(fname, "wb");
+  std::FILE *fd = Platform::fopen(fname, "wb");
   if (!fd)
     return false;
   FileStream stream(fd);
@@ -494,37 +482,26 @@ bool Document::save(const char *fname, TFormat format, uint flags) const
   return result;
 }
 
-//! Export a single view to PDF or EPS.
+//! Export a single view to PDF
 bool Document::exportView(const char *fname, TFormat format, uint flags,
 			  int pno, int vno) const
 {
-  if (format != EPdf && format != EEps)
+  if (format != EPdf)
     return false;
 
   int compresslevel = 9;
   if (flags & ENoZip)
     compresslevel = 0;
 
-  std::FILE *fd = std::fopen(fname, "wb");
+  std::FILE *fd = Platform::fopen(fname, "wb");
   if (!fd)
     return false;
   FileStream stream(fd);
 
-  if (format == EPdf) {
-    PdfWriter writer(stream, this, iFontPool, (flags & EMarkedView),
-		     pno, pno, compresslevel);
-    writer.createPageView(pno, vno);
-    writer.createTrailer();
-  } else {
-    // Postscript
-    PsWriter writer(stream, this, (flags & ENoColor));
-    if (!writer.createHeader(pno, vno)) {
-      std::fclose(fd);
-      return false;
-    }
-    writer.createPageView(pno, vno);
-    writer.createTrailer();
-  }
+  PdfWriter writer(stream, this, iFontPool, (flags & EMarkedView),
+		   pno, pno, compresslevel);
+  writer.createPageView(pno, vno);
+  writer.createTrailer();
   std::fclose(fd);
   return true;
 }
@@ -536,7 +513,7 @@ bool Document::exportPages(const char *fname, uint flags,
   int compresslevel = 9;
   if (flags & ENoZip)
     compresslevel = 0;
-  std::FILE *fd = std::fopen(fname, "wb");
+  std::FILE *fd = Platform::fopen(fname, "wb");
   if (!fd)
     return false;
   FileStream stream(fd);
@@ -827,7 +804,7 @@ int Document::runLatex(String &texLog)
 
   String encoding = cascade()->findEncoding();
 
-#ifdef IPE_USE_ICONV
+#if defined(IPE_USE_ICONV) || defined(WIN32)
   String utf8;
   StringStream stream(utf8);
   int err = converter.createLatexSource(stream, properties().iPreamble);
@@ -835,18 +812,26 @@ int Document::runLatex(String &texLog)
     return ErrWritingSource;
 
   if (encoding.empty()) {
-    std::FILE *file = std::fopen(texFile.z(), "wb");
+    std::FILE *file = Platform::fopen(texFile.z(), "wb");
     if (!file)
       return ErrWritingSource;
     FileStream fstream(file);
     fstream.putRaw(utf8.data(), utf8.size());
     std::fclose(file);
   } else {
+#ifdef WIN32
+    std::FILE *file = Platform::fopen(texFile.z(), "wb");
+    if (!file)
+      return ErrWritingSource;
+    String raw = Platform::winConv(utf8, encoding);
+    std::fputs(raw.z(), file);
+    std::fclose(file);
+#else
     iconv_t conv = iconv_open(encoding.z(), "UTF-8");
     if (conv == iconv_t(-1))
       return ErrWritingSource;
 
-    std::FILE *file = std::fopen(texFile.z(), "wb");
+    std::FILE *file = Platform::fopen(texFile.z(), "wb");
     if (!file)
       return ErrWritingSource;
 
@@ -870,12 +855,13 @@ int Document::runLatex(String &texLog)
     }
     iconv_close(conv);
     std::fclose(file);
+#endif
   }
 #else
   if (!encoding.empty())
     return ErrNoIconv;
 
-  std::FILE *file = std::fopen(texFile.z(), "wb");
+  std::FILE *file = Platform::fopen(texFile.z(), "wb");
   if (!file)
     return ErrWritingSource;
   FileStream stream(file);
@@ -886,6 +872,7 @@ int Document::runLatex(String &texLog)
     return ErrWritingSource;
 #endif
 
+
   int result = Platform::runPdfLatex(latexDir);
 
   if (result != 0 && result != 1)
@@ -905,7 +892,7 @@ int Document::runLatex(String &texLog)
   if (texLog.find("\n!") >= 0)
     return ErrLatex;
 
-  std::FILE *pdfF = std::fopen(pdfFile.z(), "rb");
+  std::FILE *pdfF = Platform::fopen(pdfFile.z(), "rb");
   if (!pdfF)
     return ErrLatex;
   FileSource source(pdfF);
