@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -37,6 +37,8 @@
 
 */
 
+const double BEZIER_INTERSECT_PRECISION = 1.0;
+
 #include "ipegeo.h"
 
 using namespace ipe;
@@ -477,7 +479,7 @@ Linear::Linear(String str)
 //! Return inverse.
 Linear Linear::inverse() const
 {
-  double t = a[0]*a[3]-a[1]*a[2];
+  double t = determinant();
   assert(t != 0);
   t = 1.0/t;
   return Linear(a[3]*t, -a[1]*t, -a[2]*t, a[0]*t);
@@ -507,7 +509,7 @@ Matrix::Matrix(String str)
 //! Return inverse.
 Matrix Matrix::inverse() const
 {
-  double t = a[0]*a[3]-a[1]*a[2];
+  double t = determinant();
   assert(t != 0);
   t = 1.0/t;
   return Matrix(a[3]*t, -a[1]*t, -a[2]*t, a[0]*t,
@@ -618,11 +620,14 @@ Bezier Bezier::quadBezier(const Vector &p0, const Vector &p1,
   return Bezier(p0, q1, q2, p2);
 }
 
-//! Convert a uniform cubic B-spline to a series of Bezier splines.
-/*! First and last control point are given multiplicity 3.
+//! Convert an old-style Ipe B-spline to a series of Bezier splines.
+/*! For some reason lost in the mist of time, this was the definition
+  of splines in Ipe for many years.  It doesn't use knots.  The first
+  and last control point are simply given multiplicity 3.
+
   Bezier splines are appended to \a result.
 */
-void Bezier::spline(int n,  const Vector *v, std::vector<Bezier> &result)
+void Bezier::oldSpline(int n,  const Vector *v, std::vector<Bezier> &result)
 {
   Vector p0, p1, p2, p3, q0, q1, q2, q3;
   // First segment (p1 = p2 = p0 => q1 = q2 = q0 = p0)
@@ -667,19 +672,102 @@ void Bezier::spline(int n,  const Vector *v, std::vector<Bezier> &result)
   result.push_back(Bezier(q3, p3, p3, p3));
 }
 
+//! Convert a clamped uniform B-spline to a series of Bezier splines.
+/*! See Thomas Sederberg, Computer-Aided Geometric Design, Chapter 6.
+
+  In polar coordinates, a control point is defined by three knots, so
+  n control points need n + 2 knots.  To clamp the spline to the first
+  and last control point, the first and last knot are repeated three
+  times. This leads to k knot intervals and the knot sequence
+  [0, 0, 0, 1, 2, 3, 4, 5, 6, ..., k-2, k-1, k, k, k]
+  There are k + 5 = n + 2 knots in this sequence, so k = n-3 is the
+  number of knot intervals and therefore the number of output Bezier
+  curves.
+
+  If n = 4, the knot sequence is [0, 0, 0, 1, 1, 1] and the result is
+  simply the Bezier curve on the four control points.
+
+  When n in {2, 3}, returns a single Bezier curve that is a segment or
+  quadratic Bezier spline.  This is different from the behaviour of
+  the "old" Ipe splines.
+
+  Bezier splines are appended to \a result.
+*/
+void Bezier::spline(int n,  const Vector *v, std::vector<Bezier> &result)
+{
+  if (n == 2) {
+    result.push_back(Bezier(v[0], v[0], v[1], v[1]));
+  } else if (n == 3) {
+    result.push_back(quadBezier(v[0], v[1], v[2]));
+  } else if (n == 4) {
+    result.push_back(Bezier(v[0], v[1], v[2], v[3]));
+  } else if (n == 5) {
+    // Given are [0,0,0], [0,0,1], [0,1,2], [1,2,2], [2,2,2]
+    // Interval 0-1: [0,0,0], [0,0,1], [0,1,1], [1,1,1]
+    Vector q0 = v[0];  // [0,0,0]
+    Vector q1 = v[1];  // [0,0,1]
+    Vector q2 = midpoint(q1, v[2]); // [0,1,1] = 1/2 [0,1,0] + 1/2 [0,1,2]
+    Vector r = midpoint(v[2], v[3]); // [1,1,2] = 1/3 [0,1,2] + 1/2 [2,1,2]
+    Vector q3 = midpoint(q2, r);  // [1,1,1] = 1/2 [1,0,1] + 1/2 [1,2,1]
+    result.push_back(Bezier(q0, q1, q2, q3));
+    // Interval 1-2: [1,1,1], [1,1,2], [1,2,2], [2,2,2]
+    result.push_back(Bezier(q3, r, v[3], v[4]));
+  } else {
+    int k = n-3;
+    // Interval 0-1: [0,0,0], [0,0,1], [0,1,1], [1,1,1]
+    Vector q0 = v[0];  // [0,0,0]
+    Vector q1 = v[1];  // [0,0,1]
+    Vector q2 = midpoint(q1, v[2]); // [0,1,1] = 1/2 [0,1,0] + 1/2 [0,1,2]
+    Vector r = thirdpoint(v[2], v[3]);  // [1,2,1] = 2/3 [0,1,2] + 1/3 [3,1,2]
+    Vector q3 = midpoint(q2, r);  // [1,1,1] = 1/2 [1,0,1] + 1/2 [1,2,1]
+    result.push_back(Bezier(q0, q1, q2, q3));
+    for (int i = 1; i < k-2; ++i) {
+      // Interval i-i+1: [i,i,i], [i,i,i+1], [i,i+1,i+1], [i+1,i+1,i+1]
+      q0 = q3; // [i,i,i]
+      q1 = r;  // [i,i,i+1]
+      // [i,i+1,i+1] = 1/2 [i,i+1,i] + 1/2 [i, i+1, i+2]
+      q2 = midpoint(q1, v[i+2]);
+      // [i+1,i+1,i+2] = 2/3 [i,i+1,i+2] + [i+3,i+1,i+2]
+      r = thirdpoint(v[i+2], v[i+3]);
+      // [i+1,i+1,i+1] = 1/2 [i+1,i+1,i] + 1/2 [i+1,i+1,i+2]
+      q3 = midpoint(q2, r);
+      result.push_back(Bezier(q0, q1, q2, q3));
+    }
+    // Interval (k-2)-(k-1):
+    // [k-2,k-2,k-2], [k-2,k-2,k-1], [k-2,k-1,k-1], [k-1,k-1,k-1]
+    q0 = q3;
+    q1 = r;  // [k-2,k-2,k-1]
+    // [k-2,k-1,k-1] = 1/2 [k-2,k-1,k-2] + 1/2 [k-2,k-1,k]
+    q2 = midpoint(q1, v[k]);
+    // [k-1,k-1,k] =  1/2 [k-2,k-1,k] + 1/2 [k,k-1,k]
+    r = midpoint(v[k], v[k+1]);
+    // [k-1,k-1,k-1] = 1/2 [k-1,k-1,k-2] + 1/2 [k-1,k-1,k]
+    q3 = midpoint(q2, r);
+    result.push_back(Bezier(q0, q1, q2, q3));
+    // Interval (k-1)-k: [k-1,k-1,k-1], [k-1,k-1,k], [k-1,k,k], [k,k,k]
+    q0 = q3;
+    q1 = r;
+    q2 = v[n-2]; // [k-1,k,k]
+    q3 = v[n-1]; // [k,k,k]
+    result.push_back(Bezier(q0, q1, q2, q3));
+  }
+}
+
 //! Convert a closed uniform cubic B-spline to a series of Bezier splines.
 /*! Bezier splines are appended to \a result. */
 void Bezier::closedSpline(int n,  const Vector *v, std::vector<Bezier> &result)
 {
   for (int i = 0; i < n; ++i) {
-    Vector p0 = v[i];
-    Vector p1 = v[(i + 1) % n];
-    Vector p2 = v[(i + 2) % n];
-    Vector p3 = v[(i + 3) % n];
-    Vector q1 = thirdpoint(p1, p2);
-    Vector q2 = thirdpoint(p2, p1);
-    Vector q0 = midpoint(thirdpoint(p1, p0), q1);
-    Vector q3 = midpoint(thirdpoint(p2, p3), q2);
+    Vector p0 = v[i % n];           // [0, 1, 2]
+    Vector p1 = v[(i+1) % n];       // [1, 2, 3]
+    Vector p2 = v[(i+2) % n];       // [2, 3, 4]
+    Vector p3 = v[(i+3) % n];       // [3, 4, 5]
+    Vector r = thirdpoint(p1, p0);  // [1, 2, 2]
+    Vector u = thirdpoint(p2, p3);  // [3, 3, 4]
+    Vector q1 = thirdpoint(p1, p2); // [2, 2, 3]
+    Vector q2 = thirdpoint(p2, p1); // [2, 3, 3]
+    Vector q0 = midpoint(r, q1);    // [2, 2, 2]
+    Vector q3 = midpoint(u, q2);    // [3, 3, 3]
     result.push_back(Bezier(q0, q1, q2, q3));
   }
 }
@@ -804,8 +892,8 @@ bool Bezier::snap(const Vector &v, double &t, Vector &pos, double &bound) const
  * The found intersection points are stored in the vector intersections.
  */
 
-void intersectBeziers(std::vector<Vector> &intersections,
-                      const Bezier &a, const Bezier &b)
+static void intersectBeziers(std::vector<Vector> &intersections,
+			     const Bezier &a, const Bezier &b)
 {
   /* Recursive approximation procedure to find intersections:
    * If the bounding boxes of two Beziers overlap, both are subdivided,
@@ -814,16 +902,7 @@ void intersectBeziers(std::vector<Vector> &intersections,
    * of the partial Beziers overlap. If they do, they are subdivided
    * again and so on, until a special precision is achieved:
    * Then the Beziers are converted to Segments and checked for intersection.
-   *
-   * Intersection points that lie near the first or last control point
-   * of a bezier (have a distance of less than MIN_DIST to a cp) are ignored;
-   * this is to avoid connection points of two beziers accidentaly being
-   * misinterpreted as intersections.
    */
-
-  const double PRECISION = 1.0;
-  const double MIN_DIST = 0.05;
-
   Rect abox(a.iV[0], a.iV[1]);
   abox.addPoint(a.iV[2]);
   abox.addPoint(a.iV[3]);
@@ -834,22 +913,13 @@ void intersectBeziers(std::vector<Vector> &intersections,
   if (!abox.intersects(bbox))
     return;
 
-  if (a.straight(PRECISION) && b.straight(PRECISION)) {
-    Segment aLine = Segment(a.iV[0], a.iV[3]);
-    Segment bLine = Segment(b.iV[0], b.iV[3]);
-
+  if (a.straight(BEZIER_INTERSECT_PRECISION) &&
+      b.straight(BEZIER_INTERSECT_PRECISION)) {
+    Segment as = Segment(a.iV[0], a.iV[3]);
+    Segment bs = Segment(b.iV[0], b.iV[3]);
     Vector p;
-    if (aLine.intersects(bLine, p)) {
-      // We found an intersection point.  If this point is too close
-      // to one of the outer control points, it is ignored.
-      if ((p-a.iV[0]).len() < MIN_DIST ||
-          (p-a.iV[3]).len() < MIN_DIST ||
-          (p-b.iV[0]).len() < MIN_DIST ||
-          (p-b.iV[3]).len() < MIN_DIST) {
-        // ignoring intersection too close to control point.
-      } else // Found intersection p
-        intersections.push_back(p);
-    }
+    if (as.intersects(bs, p))
+      intersections.push_back(p);
   } else {
     Bezier leftA, rightA, leftB, rightB;
     a.subdivide(leftA, rightA);
@@ -865,24 +935,16 @@ void intersectBeziers(std::vector<Vector> &intersections,
 //! Compute intersection points of Bezier with Line.
 void Bezier::intersect(const Line &l, std::vector<Vector> &result) const
 {
-  const double PRECISION = 1.0;
-  const double MIN_DIST = 0.05;
-
   double sgn = l.side(iV[0]);
-
   if (sgn < 0 && l.side(iV[1]) < 0 && l.side(iV[2]) < 0 && l.side(iV[3]) < 0)
     return;
   if (sgn > 0 && l.side(iV[1]) > 0 && l.side(iV[2]) > 0 && l.side(iV[3]) > 0)
     return;
 
-  if (straight(PRECISION)) {
+  if (straight(BEZIER_INTERSECT_PRECISION)) {
     Vector p;
-    if (Segment(iV[0], iV[3]).intersects(l, p)) {
-      // We found an intersection point.  If this point is too close
-      // to one of the outer control points, it is ignored.
-      if ((p-iV[0]).len() > MIN_DIST && (p-iV[3]).len() > MIN_DIST)
-        result.push_back(p);
-    }
+    if (Segment(iV[0], iV[3]).intersects(l, p))
+      result.push_back(p);
   } else {
     Bezier leftA, rightA;
     subdivide(leftA, rightA);
