@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -240,6 +240,14 @@ void Group::snapVtx(const Vector &mouse, const Matrix &m,
     (*it)->snapVtx(mouse, m1, pos, bound);
 }
 
+void Group::snapCtl(const Vector &mouse, const Matrix &m,
+		    Vector &pos, double &bound) const
+{
+  Matrix m1 = m * matrix();
+  for (const_iterator it = begin(); it != end(); ++it)
+    (*it)->snapCtl(mouse, m1, pos, bound);
+}
+
 void Group::snapBnd(const Vector &mouse, const Matrix &m,
 		    Vector &pos, double &bound) const
 {
@@ -276,17 +284,16 @@ void Group::detach()
 }
 
 //! Set attribute on all children.
-bool Group::setAttribute(Property prop, Attribute value,
-			 Attribute stroke, Attribute fill)
+bool Group::setAttribute(Property prop, Attribute value)
 {
   if (prop == EPropPinned || prop == EPropTransformations)
-    return Object::setAttribute(prop, value, stroke, fill);
+    return Object::setAttribute(prop, value);
   // all others are handled by elements themselves
   detach();
   bool result = false;
   for (List::iterator it = iImp->iObjects.begin();
        it != iImp->iObjects.end(); ++it)
-    result |= (*it)->setAttribute(prop, value, stroke, fill);
+    result |= (*it)->setAttribute(prop, value);
   return result;
 }
 
