@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -166,7 +166,7 @@ void Image::addToBBox(Rect &box, const Matrix &m, bool) const
   box.addPoint(m1 * iRect.topLeft());
 }
 
-void Image::snapVtx(const Vector &mouse, const Matrix &m,
+void Image::snapCtl(const Vector &mouse, const Matrix &m,
 		    Vector &pos, double &bound) const
 {
   Matrix m1 = m * matrix();
