@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -34,6 +34,7 @@
 
 #include "ipefontpool.h"
 #include "ipelatex.h"
+#include <cstdlib>
 
 using namespace ipe;
 
@@ -161,12 +162,11 @@ int Latex::createLatexSource(Stream &stream, String preamble)
 	   << "Set IPEANCIENTPDFTEX environment variable!}\\fi\n";
   }
   stream << "\\documentclass{article}\n"
-    // << "\\newcommand{\\Ipechar}[1]{\\unichar{#1}}\n"
 	 << "\\newcommand{\\PageTitle}[1]{#1}\n"
 	 << "\\newdimen\\ipefs\n"
 	 << "\\newcommand{\\ipesymbol}[4]{\\ipefs 1ex\\pdfliteral"
 	 << "{(#1) (\\the\\ipefs) (#2) (#3) (#4) sym}}\n"
-	 << "\\usepackage{color}\n";
+	 << "\\usepackage{xcolor}\n";
   AttributeSeq colors;
   iCascade->allNames(EColor, colors);
   for (AttributeSeq::const_iterator it = colors.begin();
