@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -294,13 +294,8 @@ void Object::setTransformations(TTransformations trans)
 }
 
 //! Set an attribute on this object.
-/*! Returns true if an attribute was actually changed.  The arguments
-  \a stroke and \a fill are only used then \a prop is \e
-  EPropPathMode, and stroking or filling is added to an object that
-  didn't have it before.
- */
-bool Object::setAttribute(Property prop, Attribute value,
-			  Attribute stroke, Attribute fill)
+/*! Returns true if an attribute was actually changed.  */
+bool Object::setAttribute(Property prop, Attribute value)
 {
   switch (prop) {
   case EPropPinned:
@@ -360,6 +355,28 @@ void Object::checkSymbol(Kind kind, Attribute attr, const Cascade *sheet,
   }
 }
 
+//! Compute vertex snapping position for transformed object.
+/*! Looks only for positions closer than \a bound.
+  If successful, modify \a pos and \a bound.
+  The default implementation does nothing.
+*/
+void Object::snapVtx(const Vector &mouse, const Matrix &m,
+		     Vector &pos, double &bound) const
+{
+  // nothing
+}
+
+//! Compute control point snapping position for transformed object.
+/*! Looks only for positions closer than \a bound.
+  If successful, modify \a pos and \a bound.
+  The default implementation does nothing.
+*/
+void Object::snapCtl(const Vector &mouse, const Matrix &m,
+		     Vector &pos, double &bound) const
+{
+  // nothing
+}
+
 //! Compute boundary snapping position for transformed object.
 /*! Looks only for positions closer than \a bound.
   If successful, modify \a pos and \a bound.
