@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -208,6 +208,7 @@ void Path::makeArrowData()
     iRArrowOk = false;
   } else {
     CurveSegment seg = iShape.subPath(0)->asCurve()->segment(0);
+
     iRArrowOk = true;
     iRArrowPos = seg.cp(0);
     iRArrowArc = 0;
@@ -322,33 +323,42 @@ void Path::drawArrow(Painter &painter, Vector pos, Angle angle,
     if (cw)
       radius = -radius;
 
-    if ((shape == Attribute::ARROW_ARC() ||
-	 shape == Attribute::ARROW_FARC()) && (radius > s)) {
+    if (shape.isArcArrow() && (radius > s)) {
       Angle delta = s / radius;
       Angle alpha = atan(1.0/3.0);
       Arc arc1;
       Arc arc2;
+      Arc arc3;
       if (cw) {
 	arc1 = Arc(Matrix(radius, 0, 0, radius, 0, -radius),
 		   IpeHalfPi, IpeHalfPi + delta);
 	arc2 = Arc(Matrix(radius, 0, 0, -radius, 0, -radius),
 		   -IpeHalfPi - delta, -IpeHalfPi);
+	arc3 = Arc(Matrix(radius, 0, 0, radius, 0, -radius),
+		   IpeHalfPi, IpeHalfPi + 0.8 * delta);
       } else {
 	arc1 = Arc(Matrix(radius, 0, 0, radius, 0, radius),
 		   -IpeHalfPi - delta, -IpeHalfPi);
 	arc2 = Arc(Matrix(radius, 0, 0, -radius, 0, radius),
 		   IpeHalfPi, IpeHalfPi + delta);
+	arc3 = Arc(Matrix(radius, 0, 0, radius, 0, radius),
+		   -IpeHalfPi - 0.8 * delta, -IpeHalfPi);
       }
       arc1 = Linear(alpha) * arc1;
       arc2 = Linear(-alpha) * arc2;
       painter.setStroke(Attribute(color));
-      if (shape == Attribute::ARROW_FARC())
+      if (shape == Attribute::ARROW_FARC() ||
+	  shape == Attribute::ARROW_FPTARC())
 	painter.setFill(Attribute(Color(1000, 1000, 1000)));
       else
 	painter.setFill(Attribute(color));
       painter.newPath();
       painter.moveTo(arc1.beginp());
       painter.drawArc(arc1);
+      if (shape == Attribute::ARROW_PTARC() ||
+	  shape == Attribute::ARROW_FPTARC()) {
+	painter.lineTo(arc3.endp());
+      }
       painter.lineTo(arc2.beginp());
       painter.drawArc(arc2);
       painter.closePath();
@@ -413,8 +423,7 @@ void Path::draw(Painter &painter) const
     }
     if (iHasFArrow && iFArrowOk) {
       double r = 0.0;
-      if (iFArrowArc && (iFArrowShape == Attribute::ARROW_ARC() ||
-			 iFArrowShape == Attribute::ARROW_FARC())) {
+      if (iFArrowArc && iFArrowShape.isArcArrow()) {
 	CurveSegment seg = iShape.subPath(0)->asCurve()->segment(-1);
 	Vector center = painter.matrix() * seg.matrix().translation();
 	r = (center - painter.matrix() * iFArrowPos).len();
@@ -464,7 +473,13 @@ double Path::distance(const Vector &v, const Matrix &m, double bound) const
 void Path::snapVtx(const Vector &mouse, const Matrix &m,
 		   Vector &pos, double &bound) const
 {
-  iShape.snapVtx(mouse, m * matrix(), pos, bound);
+  iShape.snapVtx(mouse, m * matrix(), pos, bound, false);
+}
+
+void Path::snapCtl(const Vector &mouse, const Matrix &m,
+		   Vector &pos, double &bound) const
+{
+  iShape.snapVtx(mouse, m * matrix(), pos, bound, true);
 }
 
 void Path::snapBnd(const Vector &mouse, const Matrix &m,
@@ -476,12 +491,6 @@ void Path::snapBnd(const Vector &mouse, const Matrix &m,
 //! Set whether object will be stroked and filled.
 void Path::setPathMode(TPathMode pm)
 {
-  /* XXX
-  if (iPathMode == EStrokedOnly && pm != EStrokedOnly)
-    iFill = Attribute::WHITE();
-  if (iPathMode == EFilledOnly && pm != EFilledOnly)
-    iStroke = Attribute::BLACK();
-  */
   iPathMode = pm;
 }
 
@@ -568,9 +577,7 @@ void Path::setFillRule(TFillRule s)
 
 void Path::checkStyle(const Cascade *sheet, AttributeSeq &seq) const
 {
-  // XXX if (iPathMode <= EStrokedAndFilled)
   checkSymbol(EColor, iStroke, sheet, seq);
-  // XXX if (iPathMode >= EStrokedAndFilled)
   checkSymbol(EColor, iFill, sheet, seq);
   checkSymbol(EDashStyle, iDashStyle, sheet, seq);
   checkSymbol(EPen, iPen, sheet, seq);
@@ -585,22 +592,12 @@ void Path::checkStyle(const Cascade *sheet, AttributeSeq &seq) const
     checkSymbol(EGradient, iGradient, sheet, seq);
 }
 
-// nStroke and nFill attribute are not used
-// remove in future version
-bool Path::setAttribute(Property prop, Attribute value,
-			Attribute nStroke, Attribute nFill)
+bool Path::setAttribute(Property prop, Attribute value)
 {
   switch (prop) {
   case EPropPathMode:
     if (value.pathMode() != pathMode()) {
-      // XXX TPathMode old = pathMode();
       setPathMode(value.pathMode());
-      /* XXX
-      if (old == EStrokedOnly)
-	setFill(nFill);
-      if (old == EFilledOnly)
-	setStroke(nStroke);
-      */
       return true;
     }
     break;
@@ -704,7 +701,7 @@ bool Path::setAttribute(Property prop, Attribute value,
     }
     break;
   default:
-    return Object::setAttribute(prop, value, nStroke, nFill);
+    return Object::setAttribute(prop, value);
   }
   return false;
 }
