@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -30,6 +30,7 @@
 
 #include "ipepdfparser.h"
 #include "ipeutils.h"
+#include <cstdlib>
 
 using namespace ipe;
 
@@ -64,11 +65,6 @@ inline int toInt(String &s)
   return std::strtol(s.z(), 0, 10);
 }
 
-inline double toDouble(String &s)
-{
-  return std::strtod(s.z(), 0);
-}
-
 // --------------------------------------------------------------------
 
 /*! \class ipe::PdfObj
@@ -528,11 +524,11 @@ PdfArray *PdfParser::makeArray()
 	  arr->append(new PdfRef(toInt(t1.iString)));
 	  getToken();
 	} else {
-	  arr->append(new PdfNumber(toDouble(t1.iString)));
-	  arr->append(new PdfNumber(toDouble(t2.iString)));
+	  arr->append(new PdfNumber(Platform::toDouble(t1.iString)));
+	  arr->append(new PdfNumber(Platform::toDouble(t2.iString)));
 	}
       } else {
-	arr->append(new PdfNumber(toDouble(t1.iString)));
+	arr->append(new PdfNumber(Platform::toDouble(t1.iString)));
       }
     } else {
       PdfObj *obj = getObject();
@@ -597,7 +593,7 @@ PdfDict *PdfParser::makeDict()
 	} else
 	  return 0; // should be name or '>>'
       } else
-	dict->add(name, new PdfNumber(toDouble(t1.iString)));
+	dict->add(name, new PdfNumber(Platform::toDouble(t1.iString)));
     } else {
       PdfObj *obj = getObject();
       if (!obj)
@@ -615,7 +611,7 @@ PdfObj *PdfParser::getObject()
 
   switch (tok.iType) {
   case PdfToken::ENumber:
-    return new PdfNumber(std::strtod(tok.iString.z(), 0));
+    return new PdfNumber(Platform::toDouble(tok.iString));
   case PdfToken::EString:
     return new PdfString(tok.iString);
   case PdfToken::EName:
@@ -715,23 +711,31 @@ bool PdfFile::parse(DataSource &source)
       // <num> 0 obj starts an object
       int num = toInt(t.iString);
       PdfObj *obj = parser.getObjectDef();
-      if (!obj)
+      if (!obj) {
+	ipeDebug("Failed to get object %d", num);
 	return false;
+      }
       iObjects[num] = obj;
     } else if (t.iType == PdfToken::EOp) {
       if (t.iString == "trailer") {
 	iTrailer = parser.getTrailer();
-	if (!iTrailer)
+	if (!iTrailer) {
+	  ipeDebug("Failed to get trailer");
 	  return false;
+	}
 	return true;
       } else if (t.iString == "xref") {
 	parser.skipXRef();
-      } else
+      } else {
+	ipeDebug("Weird token: %s", t.iString.z());
 	// don't know what's happening
 	return false;
-    } else
+      }
+    } else {
+      ipeDebug("Weird token type: %d %s", t.iType, t.iString.z());
       // don't know what's happening
       return false;
+    }
   }
 }
 
