@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -35,15 +35,22 @@
 #include <windows.h>
 #include <shlobj.h>
 #include <direct.h>
+#include <gdiplus.h>
 #else
 #include <sys/wait.h>
+#include <xlocale.h>
+#include <dirent.h>
 #endif
+#ifdef __APPLE__
+#include <mach-o/dyld.h>
+#endif
+
 #include <cstdlib>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <stdarg.h>
+#include <cstdarg>
 #include <unistd.h>
-#include <locale.h>
+#include <clocale>
 
 using namespace ipe;
 
@@ -66,55 +73,90 @@ int Platform::libVersion()
 
 // --------------------------------------------------------------------
 
+static bool initialized = false;
 static bool showDebug = false;
 static Platform::DebugHandler debugHandler = 0;
 
+#ifdef WIN32
+static ULONG_PTR gdiplusToken = 0;
+// not yet available in MINGW RT library
+// _locale_t ipeLocale;
+#else
+locale_t ipeLocale;
+#endif
+
+#ifdef WIN32
+// Counterpart to Platform::wideToUtf8
+void Platform::utf8ToWide(const char *utf8, std::vector<wchar_t> &wbuf)
+{
+  int rw = MultiByteToWideChar(CP_UTF8, 0, utf8, -1, NULL, 0);
+  wbuf.resize(rw);
+  MultiByteToWideChar(CP_UTF8, 0, utf8, -1, &wbuf[0], rw);
+}
+#endif
+
 static void debugHandlerImpl(const char *msg)
 {
   if (showDebug) {
+    fprintf(stderr, "%s\n", msg);
 #ifdef WIN32
+    fflush(stderr);
     OutputDebugStringA(msg);
 #endif
-    fprintf(stderr, "%s\n", msg);
   }
 }
 
-static void cleanup_repository()
+static void shutdownIpelib()
 {
+#ifdef WIN32
+  Gdiplus::GdiplusShutdown(gdiplusToken);
+  // _free_locale(ipeLocale);
+#else
+  freelocale(ipeLocale);
+#endif
   Repository::cleanup();
 }
 
 //! Initialize Ipelib.
 /*! This method must be called before Ipelib is used.
 
-  It sets the LC_NUMERIC locale to 'C', which is necessary for correct
-  loading and saving of Ipe objects.  The method also checks that the
-  correct version of Ipelib is loaded, and aborts with an error
-  message if the version is not correct.  Also enables ipeDebug
+  It creates a LC_NUMERIC locale set to 'C', which is necessary for
+  correct loading and saving of Ipe objects.  The method also checks
+  that the correct version of Ipelib is loaded, and aborts with an
+  error message if the version is not correct.  Also enables ipeDebug
   messages if environment variable IPEDEBUG is defined.  (You can
   override this using setDebug).
 */
 void Platform::initLib(int version)
 {
-  // Ensure numbers are output with a period rather than a comma
-  // as the decimal point.
-  setlocale(LC_NUMERIC, "C");
+  if (initialized)
+    return;
+  initialized = true;
   showDebug = false;
-  if (getenv("IPEDEBUG") != 0)
+  if (getenv("IPEDEBUG") != 0) {
     showDebug = true;
+    fprintf(stderr, "Debug messages enabled\n");
+  }
   debugHandler = debugHandlerImpl;
-  atexit(cleanup_repository);
+#ifdef WIN32
+  // ipeLocale = ::_create_locale(LC_NUMERIC, "C");
+  Gdiplus::GdiplusStartupInput gdiplusStartupInput;
+  Gdiplus::GdiplusStartup(&gdiplusToken, &gdiplusStartupInput, 0);
+#else
+  ipeLocale = newlocale(LC_NUMERIC_MASK, "C", NULL);
+#endif
+  atexit(shutdownIpelib);
+#ifndef WIN32
   if (version == IPELIB_VERSION)
     return;
   fprintf(stderr,
 	  "Ipetoipe has been compiled with header files for Ipelib %d\n"
 	  "but is dynamically linked against libipe %d.\n"
-#ifndef WIN32
 	  "Check with 'ldd' which libipe is being loaded, and "
-	  "replace it by the correct version or set LD_LIBRARY_PATH.\n"
-#endif
-	  , version, IPELIB_VERSION);
+	  "replace it by the correct version or set LD_LIBRARY_PATH.\n",
+	  version, IPELIB_VERSION);
   exit(99);
+#endif
 }
 
 //! Enable or disable display of ipeDebug messages.
@@ -139,25 +181,15 @@ void ipeDebug(const char *msg, ...)
 
 // --------------------------------------------------------------------
 
-//! Return correct path separator for this platform.
-char Platform::pathSeparator()
-{
-#ifdef WIN32
-  return '\\';
-#else
-  return '/';
-#endif
-}
-
 //! Returns current working directory.
 /*! Returns empty string if something fails. */
 String Platform::currentDirectory()
 {
-#ifdef __MINGW32__
-  char *buffer;
-  if ((buffer = _getcwd(0, 0)) == 0)
+#ifdef WIN32
+  wchar_t *buffer;
+  if ((buffer = _wgetcwd(0, 0)) == 0)
     return String();
-  return String(buffer);
+  return Platform::wideToUtf8(buffer);
 #else
   char buffer[1024];
   if (getcwd(buffer, 1024) != buffer)
@@ -166,21 +198,50 @@ String Platform::currentDirectory()
 #endif
 }
 
-#ifdef WIN32
-String Platform::ipeDir(const char *suffix)
+#ifdef IPEBUNDLE
+String Platform::ipeDir(const char *suffix, const char *fname)
 {
-  char exename[OFS_MAXPATHNAME];
-  GetModuleFileNameA(0, exename, OFS_MAXPATHNAME);
-  String exe(exename);
-  int i = exe.rfind('\\');
-  if (i >= 0) {
-    i = exe.left(i).rfind('\\');
-    if (i >= 0)
-      return exe.left(i + 1) + suffix;
-  }
-  return String("\\") + suffix;
-}
+#ifdef WIN32
+  wchar_t exename[OFS_MAXPATHNAME];
+  GetModuleFileNameW(0, exename, OFS_MAXPATHNAME);
+  String exe = Platform::wideToUtf8(exename);
 #else
+  char path[PATH_MAX];
+  uint32_t size = sizeof(path);
+  String exe;
+  if (_NSGetExecutablePath(path, &size) == 0) {
+    char *rp = realpath(path, NULL);
+    exe = String(rp);
+    free(rp);
+  } else
+    ipeDebug("ipeDir: buffer too small; need size %u", size);
+#endif
+  int i = exe.rfind(IPESEP);
+  if (i >= 0) {
+    exe = exe.left(i);    // strip filename
+    i = exe.rfind(IPESEP);
+    if (i >= 0) {
+      exe = exe.left(i);  // strip bin directory name
+    }
+  }
+#ifdef __APPLE__
+  if (!strcmp(suffix, "doc"))
+    exe += "/SharedSupport/";
+  else
+    exe += "/Resources/";
+#else
+  exe += IPESEP;
+#endif
+  exe += suffix;
+  if (fname) {
+    exe += IPESEP;
+    exe += fname;
+  }
+  return exe;
+}
+#endif
+
+#ifndef WIN32
 static String dotIpe()
 {
   const char *home = getenv("HOME");
@@ -200,22 +261,22 @@ static String dotIpe()
  */
 String Platform::latexDirectory()
 {
-#ifdef __MINGW32__
+#ifdef WIN32
   String latexDir;
-  const char *p = getenv("IPELATEXDIR");
+  const wchar_t *p = _wgetenv(L"IPELATEXDIR");
   if (getenv("IPEWINE"))
     latexDir = "z:\\tmp\\ipelatex";
   else if (p) {
-    latexDir = p;
+    latexDir = Platform::wideToUtf8(p);
   } else {
-    TCHAR szPath[MAX_PATH];
-    if (SUCCEEDED(SHGetFolderPath(NULL, CSIDL_LOCAL_APPDATA,
-				  NULL, 0, szPath))) {
-      latexDir = String(szPath) + "\\ipe";
+    wchar_t szPath[MAX_PATH];
+    if (SUCCEEDED(SHGetFolderPathW(NULL, CSIDL_LOCAL_APPDATA,
+				   NULL, 0, szPath))) {
+      latexDir = Platform::wideToUtf8(szPath) + "\\ipe";
     } else {
-      p = getenv("LOCALAPPDATA");
+      p = _wgetenv(L"LOCALAPPDATA");
       if (p)
-	latexDir = String(p) + "\\ipe";
+	latexDir = Platform::wideToUtf8(p) + "\\ipe";
       else
 	latexDir = ipeDir("latexrun");
     }
@@ -223,7 +284,7 @@ String Platform::latexDirectory()
   if (latexDir.right(1) == "\\")
     latexDir = latexDir.left(latexDir.size() - 1);
   if (!fileExists(latexDir)) {
-    if (_mkdir(latexDir.z()) != 0)
+    if (Platform::mkdir(latexDir.z()) != 0)
       return String();
   }
   latexDir += "\\";
@@ -251,8 +312,8 @@ String Platform::fontmapFile()
   const char *p = getenv("IPEFONTMAP");
   if (p)
     return String(p);
-#ifdef WIN32
-  return ipeDir("data\\fontmap.xml");
+#ifdef IPEBUNDLE
+  return ipeDir("data", "fontmap.xml");
 #else
   return IPEFONTMAP;
 #endif
@@ -261,20 +322,54 @@ String Platform::fontmapFile()
 //! Determine whether file exists.
 bool Platform::fileExists(String fname)
 {
-#ifdef __MINGW32__
-  struct _stat statBuffer;
-  return (_stat(fname.z(), &statBuffer) == 0);
+#ifdef WIN32
+  std::vector<wchar_t> wname;
+  utf8ToWide(fname.z(), wname);
+  return (_waccess(&wname[0], F_OK) == 0);
 #else
   return (access(fname.z(), F_OK) == 0);
 #endif
 }
 
+//! List all files in directory
+/*! Return true if successful, false on error. */
+bool Platform::listDirectory(String path, std::vector<String> &files)
+{
+#ifdef WIN32
+  String pattern = path + "\\*";
+  std::vector<wchar_t> wname;
+  utf8ToWide(pattern.z(), wname);
+  struct _wfinddata_t info;
+  intptr_t h = _wfindfirst(&wname[0], &info);
+  if (h == -1L)
+    return false;
+  files.push_back(wideToUtf8(info.name));
+  while (_wfindnext(h, &info) == 0)
+    files.push_back(wideToUtf8(info.name));
+  _findclose(h);
+  return true;
+#else
+  DIR *dir = opendir(path.z());
+  if (dir == 0)
+    return false;
+  struct dirent *entry = readdir(dir);
+  while (entry != 0) {
+    String s(entry->d_name);
+    if (s != "." && s != "..")
+      files.push_back(s);
+    entry = readdir(dir);
+  }
+  closedir(dir);
+  return true;
+#endif
+}
+
 //! Read entire file into string.
 /*! Returns an empty string if file cannot be found or read.
   There is no way to distinguish an empty file from this. */
 String Platform::readFile(String fname)
 {
-  std::FILE *file = std::fopen(fname.z(), "rb");
+  std::FILE *file = Platform::fopen(fname.z(), "rb");
   if (!file)
     return String();
   String s;
@@ -292,14 +387,14 @@ int Platform::runPdfLatex(String dir)
   const char *wine = getenv("IPEWINE");
   if (wine) {
     int secs = Lex(wine).getInt();
-    std::system("/bin/sh -c \"cd /tmp/ipelatex; rm -f ipetemp.log;"
-		"pdflatex ipetemp.tex\"");
+    String s = "start /unix /home/otfried/Lib/Bin/ipelatexhelper";
+    std::system(s.z());
     fprintf(stderr, "Waiting %d seconds for pdflatex\n", secs);
     Sleep(secs * 1000);
     return 0;
   } else {
     String s = dir + "runlatex.bat";
-    std::FILE *f = std::fopen(s.z(), "wb");
+    std::FILE *f = Platform::fopen(s.z(), "wb");
     if (!f)
       return -1;
 
@@ -308,15 +403,39 @@ int Platform::runPdfLatex(String dir)
 
     // CMD.EXE input needs to be encoded in "OEM codepage",
     // which can be different from "Windows codepage"
-    Buffer oemDir(2 * dir.size() + 1);
-    CharToOemA(dir.z(), oemDir.data());
+    std::vector<wchar_t> wbuf;
+    utf8ToWide(dir.z(), wbuf);
+    Buffer oemDir(2 * wbuf.size() + 1);
+    CharToOemW(&wbuf[0], oemDir.data());
 
     fprintf(f, "cd \"%s\"\r\n", oemDir.data());
     fprintf(f, "pdflatex ipetemp.tex\r\n");
     std::fclose(f);
 
-    s = String("call \"") + dir + String("runlatex.bat\"");
-    std::system(s.z());
+    // Declare and initialize process blocks
+    PROCESS_INFORMATION processInformation;
+    STARTUPINFOW startupInfo;
+
+    memset(&processInformation, 0, sizeof(processInformation));
+    memset(&startupInfo, 0, sizeof(startupInfo));
+    startupInfo.cb = sizeof(startupInfo);
+
+    // Call the executable program
+    s = String("cmd /c call \"") + dir + String("runlatex.bat\"");
+    utf8ToWide(s.z(), wbuf);
+
+    int result = CreateProcessW(NULL, &wbuf[0], NULL, NULL, FALSE,
+				NORMAL_PRIORITY_CLASS|CREATE_NO_WINDOW,
+				NULL, NULL, &startupInfo, &processInformation);
+    if (result == 0)
+      return -1;  // failure to create process
+
+    // Wait until child process exits.
+    WaitForSingleObject(processInformation.hProcess, INFINITE);
+
+    // Close process and thread handles.
+    CloseHandle(processInformation.hProcess);
+    CloseHandle(processInformation.hThread);
     return 0;
   }
 #else
@@ -330,8 +449,92 @@ int Platform::runPdfLatex(String dir)
 #endif
 }
 
+#ifdef WIN32
+FILE *Platform::fopen(const char *fname, const char *mode)
+{
+  std::vector<wchar_t> wname;
+  utf8ToWide(fname, wname);
+  std::vector<wchar_t> wmode;
+  utf8ToWide(mode, wmode);
+  return _wfopen(&wname[0], &wmode[0]);
+}
+
+int Platform::mkdir(const char *dname)
+{
+  std::vector<wchar_t> wname;
+  utf8ToWide(dname, wname);
+  return _wmkdir(&wname[0]);
+}
+
+struct Codepage {
+  UINT cp;
+  const char *name;
+};
+
+static const Codepage codepages[] = {
+  { 28591, "iso-8859-1" },
+  { 51949, "euc-kr" },
+  { 51932, "euc-jp" },
+  { 28592, "iso-8859-2" },
+  { 21866, "koi8-u" },
+  { 20866, "koi8-r" },
+  { 932, "shift_jis" },
+  { 0, 0 }
+};
+
+// See http://msdn.microsoft.com/en-us/library/
+//  windows/desktop/dd317756%28v=vs.85%29.aspx
+static UINT codepage(const String &encoding)
+{
+  const Codepage *cp = codepages;
+  while (cp->name) {
+    if (encoding == cp->name)
+      return cp->cp;
+    ++cp;
+  }
+  String s("You requested the character encoding '");
+  s += encoding;
+  s += "' for Latex conversion.\n"
+    "There is no mapping for this encoding in Ipe.\n"
+    "This may be a bug, please report it.";
+  MessageBoxA(NULL, s.z(), "Error", MB_ICONEXCLAMATION | MB_OK);
+  return CP_UTF8;
+}
+
+String Platform::winConv(const String &utf8, const String &encoding)
+{
+  UINT cp = codepage(encoding);
+  int rw = MultiByteToWideChar(CP_UTF8, 0, utf8.z(), -1, NULL, 0);
+  wchar_t wbuf[rw];
+  MultiByteToWideChar(CP_UTF8, 0, utf8.z(), -1, wbuf, rw);
+  int rm = WideCharToMultiByte(cp, 0, wbuf, -1, NULL, 0, NULL, NULL);
+  char multi[rm];
+  WideCharToMultiByte(cp, 0, wbuf, -1, multi, rm, NULL, NULL);
+  return String(multi);
+}
+
+String Platform::wideToUtf8(const wchar_t *wbuf)
+{
+  int rm = WideCharToMultiByte(CP_UTF8, 0, wbuf, -1, NULL, 0, NULL, NULL);
+  char multi[rm];
+  WideCharToMultiByte(CP_UTF8, 0, wbuf, -1, multi, rm, NULL, NULL);
+  return String(multi);
+}
+
+#endif
+
 // --------------------------------------------------------------------
 
+double Platform::toDouble(String s)
+{
+#ifdef WIN32
+  return strtod(s.z(), 0);
+  // return _strtod_l(s.z(), 0, ipeLocale);
+#else
+  return strtod_l(s.z(), 0, ipeLocale);
+#endif
+}
+
 void ipeAssertionFailed(const char *file, int line, const char *assertion)
 {
   fprintf(stderr, "Assertion failed on line #%d (%s): '%s'\n",
