@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -275,8 +275,7 @@ void Reference::setSize(Attribute size)
 }
 
 //! \copydoc Object::setAttribute
-bool Reference::setAttribute(Property prop, Attribute value,
-			     Attribute nStroke, Attribute nFill)
+bool Reference::setAttribute(Property prop, Attribute value)
 {
   switch (prop) {
   case EPropPen:
@@ -305,17 +304,12 @@ bool Reference::setAttribute(Property prop, Attribute value,
     break;
   case EPropMarkShape:
     if ((iFlags & EIsMark) && value != name()) {
-      uint nFlags = flagsFromName(value.string());
-      if (!(iFlags & EHasStroke) && (nFlags & EHasStroke))
-	setStroke(nStroke);
-      if (!(iFlags & EHasFill) && (nFlags & EHasFill))
-	setFill(nFill);
       setName(value);
       return true;
     }
     break;
   default:
-    return Object::setAttribute(prop, value, nStroke, nFill);
+    return Object::setAttribute(prop, value);
   }
   return false;
 }
