@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -54,24 +54,26 @@ inline void snapBezier(const Vector &mouse, const Bezier &bez,
   \ingroup geo
   \brief A segment on an SubPath.
 
-  A segment is either an elliptic arc, a straight segment, a quadratic
-  Bezier spline, a cubic Bezier spline, or a B-spline curve, depending
-  on its type().  This is a lightweight object, created on the fly by
-  Curve::segment().  There is no public constructor, so the only
-  way to create such an object is through that method.
+  A segment is either an elliptic arc, a straight segment, or a
+  B-spline curve, depending on its type().  This is a lightweight
+  object, created on the fly by Curve::segment().  There is no public
+  constructor, so the only way to create such an object is through
+  that method.
 
   The type() is one of the following:
 
   - \c ESegment: the segment has two control points, and represents a
     line segment.
 
-  - \c EQuad: a quadratic Bezier spline with three control points.
-
-  - \c EBezier: a cubic Bezier spline with four control points.
-
   - \c ESpline: a B-spline curve with n control points.  The first and
     last control point's knot value is repeated three times, so the
-    curve begins and ends in these points.
+    curve begins and ends in these points. A spline with 4 control
+    points is a single Bezier curve with those control points.  A
+    spline with 3 control points is defined to be a quadratic Bezier
+    curve with those control points.
+
+  - \c EOldSpline: an incorrectly defined B-spline, used by Ipe for
+    many years.  Supported for compatibility.
 
   - \c EArc: an elliptic arc, with begin and end point.  The
     supporting ellipse is defined by the matrix(), it is the image
@@ -88,27 +90,12 @@ inline void snapBezier(const Vector &mouse, const Bezier &bez,
 //! Create a segment.
 /*! Matrix \a m defaults to null, for all segments but arcs. */
 CurveSegment::CurveSegment(Type type, int num, const Vector *cp,
-			 const Matrix *m)
+			   const Matrix *m)
   : iType(type), iCP(cp), iNumCP(num), iM(m)
 {
   // nothing
 }
 
-//! Return segment as Bezier.
-/*! Panics if segment is not a quadratic or cubic Bezier spline. */
-Bezier CurveSegment::bezier() const
-{
-  switch (type()) {
-  case EBezier:
-    return Bezier(cp(0), cp(1), cp(2), cp(3));
-  case EQuad:
-    return Bezier::quadBezier(cp(0), cp(1), cp(2));
-  default:
-    assert(false);
-    return Bezier(); // satisfy compiler
-  }
-}
-
 //! Return segment as Arc.
 /*! Panics if segment is not an arc. */
 Arc CurveSegment::arc() const
@@ -120,7 +107,10 @@ Arc CurveSegment::arc() const
 //! Convert B-spline to a sequence of Bezier splines.
 void CurveSegment::beziers(std::vector<Bezier> &bez) const
 {
-  Bezier::spline(countCP(), iCP, bez);
+  if (iType == EOldSpline)
+    Bezier::oldSpline(countCP(), iCP, bez);
+  else
+    Bezier::spline(countCP(), iCP, bez);
 }
 
 //! Draw the segment.
@@ -132,10 +122,7 @@ void CurveSegment::draw(Painter &painter) const
   case ESegment:
     painter.lineTo(cp(1));
     break;
-  case EQuad:
-  case EBezier:
-    painter.curveTo(bezier());
-    break;
+  case EOldSpline:
   case ESpline: {
     std::vector<Bezier> bez;
     beziers(bez);
@@ -163,20 +150,13 @@ void CurveSegment::addToBBox(Rect &box, const Matrix &m, bool cpf) const
     box.addPoint(m * cp(0));
     box.addPoint(m * cp(1));
     break;
-  case EBezier:
-  case EQuad:
-    if (cpf) {
-      for (int i = 0; i < countCP(); ++i)
-	box.addPoint(m * cp(i));
-    } else
-      box.addRect((m * bezier()).bbox());
-    break;
   case EArc:
     box.addRect((m * arc()).bbox());
     if (cpf)
       box.addPoint((m * matrix()).translation());
     break;
   case ESpline:
+  case EOldSpline:
     if (cpf) {
       for (int i = 0; i < countCP(); ++i)
 	box.addPoint(m * cp(i));
@@ -197,11 +177,9 @@ double CurveSegment::distance(const Vector &v, const Matrix &m,
   switch (type()) {
   case ESegment:
     return Segment(m * cp(0), m * cp(1)).distance(v, bound);
-  case EBezier:
-  case EQuad:
-    return (m * bezier()).distance(v, bound);
   case EArc:
     return (m * arc()).distance(v, bound);
+  case EOldSpline:
   case ESpline: {
     std::vector<Bezier> bez;
     beziers(bez);
@@ -221,38 +199,33 @@ double CurveSegment::distance(const Vector &v, const Matrix &m,
 /*! The method assumes that the first control point has already been
   tested. */
 void CurveSegment::snapVtx(const Vector &mouse, const Matrix &m,
-			   Vector &pos, double &bound) const
+			   Vector &pos, double &bound, bool ctl) const
 {
-  // may want to have a flag determining whether to snap to control points
-  // as well as to real vertices
   switch (type()) {
   case ESegment:
-    snapVertex(mouse, m * cp(1), pos, bound);
-    break;
-  case EQuad:
-    // snap to control points
-    snapVertex(mouse, m * cp(1), pos, bound);
-    // cp(2) is real endpoint
-    snapVertex(mouse, m * cp(2), pos, bound);
-    break;
-  case EBezier:
-    // snap to interior control points
-    snapVertex(mouse, m * cp(1), pos, bound);
-    snapVertex(mouse, m * cp(2), pos, bound);
-    // cp(3) is real endpoint
-    snapVertex(mouse, m * cp(3), pos, bound);
+    if (ctl)
+      // segment midpoint
+      snapVertex(mouse, m * (0.5 * (cp(0) + cp(1))), pos, bound);
+    else
+      snapVertex(mouse, m * cp(1), pos, bound);
     break;
   case EArc:
     // snap to center and endpoints
-    snapVertex(mouse, m * cp(1), pos, bound);
-    // center
-    snapVertex(mouse, (m * matrix()).translation(), pos, bound);
+    if (ctl)
+      // center
+      snapVertex(mouse, (m * matrix()).translation(), pos, bound);
+    else
+      snapVertex(mouse, m * cp(1), pos, bound);
     break;
   case ESpline:
+  case EOldSpline:
     // real end point is cp(countCP() - 1)
     // snap to all control points
-    for (int i = 1; i < countCP(); ++i)
-      snapVertex(mouse, m * cp(i), pos, bound);
+    if (ctl) {
+      for (int i = 1; i < countCP() - 1; ++i)
+	snapVertex(mouse, m * cp(i), pos, bound);
+    } else
+      snapVertex(mouse, m * cp(countCP() - 1), pos, bound);
     break;
   }
 }
@@ -264,10 +237,6 @@ void CurveSegment::snapBnd(const Vector &mouse, const Matrix &m,
   case ESegment:
     Segment(m * cp(0), m * cp(1)).snap(mouse, pos, bound);
     break;
-  case EBezier:
-  case EQuad:
-    snapBezier(mouse, m * bezier(), pos, bound);
-    break;
   case EArc: {
     Arc a = m * arc();
     Vector pos1;
@@ -278,6 +247,7 @@ void CurveSegment::snapBnd(const Vector &mouse, const Matrix &m,
       pos = pos1;
     }
     break; }
+  case EOldSpline:
   case ESpline: {
     std::vector<Bezier> bez;
     beziers(bez);
@@ -329,48 +299,18 @@ void Curve::appendArc(const Matrix &m, const Vector &v0, const Vector &v1)
   iSeg.push_back(seg);
 }
 
-//! Append quadratic Bezier spline.
-void Curve::appendQuad(const Vector &v0, const Vector &v1, const Vector &v2)
-{
-  if (iSeg.empty())
-    iCP.push_back(v0);
-  assert(v0 == iCP.back());
-  iCP.push_back(v1);
-  iCP.push_back(v2);
-  Seg seg;
-  seg.iType = CurveSegment::EQuad;
-  seg.iLastCP = iCP.size() - 1;
-  seg.iMatrix = iM.size() - 1;
-  iSeg.push_back(seg);
-}
-
-//! Append cubic Bezier spline.
-void Curve::appendBezier(const Vector &v0, const Vector &v1,
-			 const Vector &v2, const Vector &v3)
-{
-  if (iSeg.empty())
-    iCP.push_back(v0);
-  assert(v0 == iCP.back());
-  iCP.push_back(v1);
-  iCP.push_back(v2);
-  iCP.push_back(v3);
-  Seg seg;
-  seg.iType = CurveSegment::EBezier;
-  seg.iLastCP = iCP.size() - 1;
-  seg.iMatrix = iM.size() - 1;
-  iSeg.push_back(seg);
-}
-
 //! Append B-spline curve.
-void Curve::appendSpline(const std::vector<Vector> &v)
+void Curve::appendSpline(const std::vector<Vector> &v, CurveSegment::Type type)
 {
+  assert(type == CurveSegment::ESpline ||
+	 type == CurveSegment::EOldSpline);
   if (iSeg.empty())
     iCP.push_back(v[0]);
   assert(v[0] == iCP.back());
   for (uint i = 1; i < v.size(); ++i)
     iCP.push_back(v[i]);
   Seg seg;
-  seg.iType = CurveSegment::ESpline;
+  seg.iType = type;
   seg.iLastCP = iCP.size() - 1;
   seg.iMatrix = iM.size() - 1;
   iSeg.push_back(seg);
@@ -421,25 +361,20 @@ void Curve::save(Stream &stream) const
       assert(vtx == it->iLastCP);
       stream << iCP[vtx++] << " l\n";
       break;
-    case CurveSegment::EBezier:
-      assert(vtx + 2 == it->iLastCP);
-      stream << iCP[vtx] << " " << iCP[vtx+1] << " " << iCP[vtx+2] << " c\n";
-      vtx += 3;
-      break;
-    case CurveSegment::EQuad:
-      assert(vtx + 1 == it->iLastCP);
-      stream << iCP[vtx] << " " << iCP[vtx+1] << " q\n";
-      vtx += 2;
-      break;
     case CurveSegment::EArc:
       assert(vtx == it->iLastCP && mat == it->iMatrix);
       stream << iM[mat++] << " " << iCP[vtx++] << " a\n";
       break;
-    case CurveSegment::ESpline:
+    case CurveSegment::EOldSpline:
       while (vtx < it->iLastCP)
 	stream << iCP[vtx++] << "\n";
       stream << iCP[vtx++] << " s\n";
       break;
+    case CurveSegment::ESpline:
+      while (vtx < it->iLastCP)
+	stream << iCP[vtx++] << "\n";
+      stream << iCP[vtx++] << " c\n";
+      break;
     }
   }
   if (closed())
@@ -480,11 +415,15 @@ double Curve::distance(const Vector &v, const Matrix &m,
 }
 
 void Curve::snapVtx(const Vector &mouse, const Matrix &m,
-		    Vector &pos, double &bound) const
+		    Vector &pos, double &bound, bool ctl) const
 {
-  snapVertex(mouse, m * segment(0).cp(0), pos, bound);
+  if (!ctl)
+    snapVertex(mouse, m * segment(0).cp(0), pos, bound);
+  else if (iClosed)
+    // midpoint of closing segment
+    snapVertex(mouse, m * (0.5 * (iCP.back() + iCP.front())), pos, bound);
   for (int i = 0; i < countSegments(); ++i)
-    segment(i).snapVtx(mouse, m, pos, bound);
+    segment(i).snapVtx(mouse, m, pos, bound, ctl);
 }
 
 void Curve::snapBnd(const Vector &mouse, const Matrix &m,
@@ -570,14 +509,6 @@ const Curve *SubPath::asCurve() const
   return 0;
 }
 
-/*
-//! Return this object as an Curve, or else 0.
-Curve *SubPath::asCurve()
-{
-  return 0;
-}
-*/
-
 // --------------------------------------------------------------------
 
 /*! \class ipe::Ellipse
@@ -625,9 +556,10 @@ double Ellipse::distance(const Vector &v, const Matrix &m,
 
 //! snaps to center of ellipse.
 void Ellipse::snapVtx(const Vector &mouse, const Matrix &m,
-		      Vector &pos, double &bound) const
+		      Vector &pos, double &bound, bool ctl) const
 {
-  snapVertex(mouse, (m * iM).translation(), pos, bound);
+  if (ctl)
+    snapVertex(mouse, (m * iM).translation(), pos, bound);
 }
 
 void Ellipse::snapBnd(const Vector &mouse, const Matrix &m,
@@ -716,11 +648,13 @@ void ClosedSpline::beziers(std::vector<Bezier> &bez) const
 }
 
 void ClosedSpline::snapVtx(const Vector &mouse, const Matrix &m,
-			   Vector &pos, double &bound) const
+			   Vector &pos, double &bound, bool ctl) const
 {
-  // snap to control points
-  for (uint i = 0; i < iCP.size(); ++i)
-    snapVertex(mouse, m * iCP[i], pos, bound);
+  if (ctl) {
+    // snap to control points
+    for (uint i = 0; i < iCP.size(); ++i)
+      snapVertex(mouse, m * iCP[i], pos, bound);
+  }
 }
 
 void ClosedSpline::snapBnd(const Vector &mouse, const Matrix &m,
@@ -895,10 +829,10 @@ double Shape::distance(const Vector &v, const Matrix &m, double bound) const
 }
 
 void Shape::snapVtx(const Vector &mouse, const Matrix &m,
-		    Vector &pos, double &bound) const
+		    Vector &pos, double &bound, bool ctl) const
 {
   for (int i = 0; i < countSubPaths(); ++i)
-    subPath(i)->snapVtx(mouse, m, pos, bound);
+    subPath(i)->snapVtx(mouse, m, pos, bound, ctl);
 }
 
 void Shape::snapBnd(const Vector &mouse, const Matrix &m,
@@ -1003,43 +937,30 @@ bool Shape::load(String data)
 	sp->appendSegment(org, v);
 	org = v;
       }
-    } else if (stream.token() == "q") {
-      if (!sp || args.size() != 4)
-	return false;
-      stream.nextToken();
-      Vector v1 = getVector(args);
-      Vector v2 = getVector(args);
-      sp->appendQuad(org, v1, v2);
-      org = v2;
-    } else if (stream.token() == "c") {
-      // assert(sp && args.size() >= 6 && (args.size() % 6 == 0));
-      if (!sp || args.size() != 6)
-	return false;
-      stream.nextToken();
-      while (!args.empty()) {
-	Vector v1 = getVector(args);
-	Vector v2 = getVector(args);
-	Vector v3 = getVector(args);
-	sp->appendBezier(org, v1, v2, v3);
-	org = v3;
-      }
     } else if (stream.token() == "a") {
       if (!sp || args.size() != 8)
 	return false;
       stream.nextToken();
       Matrix m = getMatrix(args);
+      if (m.determinant() == 0)
+	return false; // don't accept zero-radius arc
       Vector v1 = getVector(args);
       sp->appendArc(m, org, v1);
       org = v1;
-    } else if (stream.token() == "s") {
+    } else if (stream.token() == "s" || stream.token() == "q"
+	       || stream.token() == "c") {
       if (!sp || args.size() < 2 || (args.size() % 2 != 0))
 	return false;
+      bool oldStyle = (stream.token() == "s");
       stream.nextToken();
       std::vector<Vector> v;
       v.push_back(org);
       while (!args.empty())
 	v.push_back(getVector(args));
-      sp->appendSpline(v);
+      if (oldStyle)
+	sp->appendOldSpline(v);
+      else
+	sp->appendSpline(v);
       org = v.back();
     } else if (stream.token() == "e") {
       if (args.size() != 6)
