@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -65,6 +65,7 @@ public:
 public:
   std::vector<Segment> iSegs;
   std::vector<Bezier> iBeziers;
+  std::vector<bool> iBeziersCont; // true if continuation of previous bezier
   std::vector<Arc> iArcs;
 
 private:
@@ -107,11 +108,16 @@ void CollectSegs::visitPath(const Path *obj)
       break;
     case SubPath::EClosedSpline: {
       std::vector<Bezier> bez;
+      bool cont = false;
       sp->asClosedSpline()->beziers(bez);
       for (uint i = 0; i < bez.size(); ++i) {
 	b = m * bez[i];
-	if (b.distance(iMouse, iDist) < iDist)
+	if (b.distance(iMouse, iDist) < iDist) {
 	  iBeziers.push_back(b);
+	  iBeziersCont.push_back(cont);
+	  cont = true;
+	} else
+	  cont = false;
       }
       break; }
     case SubPath::ECurve: {
@@ -125,24 +131,24 @@ void CollectSegs::visitPath(const Path *obj)
 	  if (seg.distance(iMouse, m, iDist) < iDist)
 	    iSegs.push_back(Segment(m * seg.cp(0), m * seg.cp(1)));
 	  break;
-	case CurveSegment::EBezier:
-	case CurveSegment::EQuad:
-	  b = m * seg.bezier();
-	  if (b.distance(iMouse, iDist) < iDist)
-	    iBeziers.push_back(b);
-	  break;
 	case CurveSegment::EArc:
 	  arc = m * seg.arc();
 	  if (arc.distance(iMouse, iDist) < iDist)
 	    iArcs.push_back(arc);
 	  break;
+	case CurveSegment::EOldSpline:
 	case CurveSegment::ESpline: {
 	  std::vector<Bezier> bez;
 	  seg.beziers(bez);
+	  bool cont = false;
 	  for (uint i = 0; i < bez.size(); ++i) {
 	    b = m * bez[i];
-	    if (b.distance(iMouse, iDist) < iDist)
+	    if (b.distance(iMouse, iDist) < iDist) {
 	      iBeziers.push_back(b);
+	      iBeziersCont.push_back(cont);
+	      cont = true;
+	    } else
+	      cont = false;
 	  }
 	  break; }
 	}
@@ -170,8 +176,8 @@ Line Snap::getLine(const Vector &mouse, const Vector &base) const
 }
 
 //! Perform intersection snapping.
-bool Snap::intersectionSnap(Vector &pos, const Page *page,
-			    double snapDist) const
+void Snap::intersectionSnap(const Vector &pos,  Vector &fifi,
+			    const Page *page, double &snapDist) const
 {
   CollectSegs segs(pos, snapDist, page);
 
@@ -188,8 +194,10 @@ bool Snap::intersectionSnap(Vector &pos, const Page *page,
 
   // 2. bezier-bezier and bezier-seg intersections
   for (uint i = 0; i < segs.iBeziers.size(); ++i) {
-    for (uint j = i + 1; j < segs.iBeziers.size(); ++j)
-      segs.iBeziers[i].intersect(segs.iBeziers[j], pts);
+    for (uint j = i + 1; j < segs.iBeziers.size(); ++j) {
+      if (j > i+1 || !segs.iBeziersCont[j])
+	segs.iBeziers[i].intersect(segs.iBeziers[j], pts);
+    }
     for (uint j = 0; j < segs.iSegs.size(); ++j)
       segs.iBeziers[i].intersect(segs.iSegs[j], pts);
   }
@@ -215,10 +223,9 @@ bool Snap::intersectionSnap(Vector &pos, const Page *page,
   }
 
   if (d < snapDist) {
-    pos = pos1;
-    return true;
+    fifi = pos1;
+    snapDist = d;
   }
-  return false;
 }
 
 //! Perform snapping to intersection of angular line and pos.
@@ -265,7 +272,8 @@ bool Snap::snapAngularIntersection(Vector &pos, const Line &l,
 
 //! Tries vertex, intersection, boundary, and grid snapping.
 /*! If snapping occurred, \a pos is set to the new user space position. */
-bool Snap::simpleSnap(Vector &pos, const Page *page, double snapDist) const
+Snap::TSnapModes Snap::simpleSnap(Vector &pos, const Page *page,
+				  double snapDist, Tool *tool) const
 {
   double d = snapDist;
   Vector fifi = pos;
@@ -276,15 +284,36 @@ bool Snap::simpleSnap(Vector &pos, const Page *page, double snapDist) const
       if (page->hasSnapping(page->layerOf(i)))
 	page->snapVtx(i, pos, fifi, d);
     }
-  }
-  if (iSnap & ESnapInt) {
-    intersectionSnap(pos, page, d);
+    if (tool)
+      tool->snapVtx(pos, fifi, d, false);
   }
 
+  double dvtx = d;
+  Vector fifiCtl = pos;
+  if (iSnap & ESnapCtl) {
+    for (int i = 0; i < page->count(); ++i) {
+      if (page->hasSnapping(page->layerOf(i)))
+	page->snapCtl(i, pos, fifiCtl, d);
+    }
+    if (tool)
+      tool->snapVtx(pos, fifiCtl, d, true);
+  }
+
+  double dctl = d;
+  Vector fifiX = pos;
+  if (iSnap & ESnapInt)
+    intersectionSnap(pos, fifiX, page, d);
+
   // Return if snapping has occurred
-  if (d < snapDist) {
+  if (d < dctl) {
+    pos = fifiX;
+    return ESnapInt;
+  } else if (d < dvtx) {
+    pos = fifiCtl;
+    return ESnapCtl;
+  } else if (d < snapDist) {
     pos = fifi;
-    return true;
+    return ESnapVtx;
   }
 
   // boundary snapping
@@ -295,19 +324,19 @@ bool Snap::simpleSnap(Vector &pos, const Page *page, double snapDist) const
     }
     if (d < snapDist) {
       pos = fifi;
-      return true;
+      return ESnapBd;
     }
   }
 
   // grid snapping: always occurs
   if (iSnap & ESnapGrid) {
     int grid = iGridSize;
-    fifi.x = grid * int(pos.x / grid + 0.5);
-    fifi.y = grid * int(pos.y / grid + 0.5);
+    fifi.x = grid * int(pos.x / grid + (pos.x > 0 ? 0.5 : -0.5));
+    fifi.y = grid * int(pos.y / grid + (pos.y > 0 ? 0.5 : -0.5));
     pos = fifi;
-    return true;
+    return ESnapGrid;
   }
-  return false;
+  return ESnapNone;
 }
 
 //! Performs snapping of position \a pos.
@@ -317,8 +346,8 @@ bool Snap::simpleSnap(Vector &pos, const Page *page, double snapDist) const
   Automatic angular snapping occurs if \a autoOrg is not null --- the
   value is then used as the origin for automatic angular snapping.
 */
-bool Snap::snap(Vector &pos, const Page *page, double snapDist,
-		Vector *autoOrg) const
+Snap::TSnapModes Snap::snap(Vector &pos, const Page *page, double snapDist,
+			    Tool *tool, Vector *autoOrg) const
 {
   // automatic angular snapping and angular snapping both on?
   if (autoOrg && (iSnap & ESnapAuto) && (iSnap & ESnapAngle)) {
@@ -326,9 +355,9 @@ bool Snap::snap(Vector &pos, const Page *page, double snapDist,
     Line angular = getLine(pos, iOrigin);
     Line automat = getLine(pos, *autoOrg);
     Vector v;
-    if (angular.intersects(automat, v)) {
+    if (angular.intersects(automat, v) && v.sqLen() < 1e10) {
       pos = v;
-      return true;
+      return ESnapAngle;
     }
     // if the two lines do not intersect, use following case
   }
@@ -344,11 +373,11 @@ bool Snap::snap(Vector &pos, const Page *page, double snapDist,
     pos = l.project(pos);
     if (iSnap & ESnapBd)
       snapAngularIntersection(pos, l, page, snapDist);
-    return true;
+    return ESnapAngle;
   }
 
   // we are not in any angular snapping mode
-  return simpleSnap(pos, page, snapDist);
+  return simpleSnap(pos, page, snapDist, tool);
 }
 
 //! Set axis origin and direction from edge near mouse.
