@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -349,6 +349,31 @@ bool StyleSheet::has(Kind kind, Attribute sym) const
   }
 }
 
+//! Removes definition for a symbolic attribute from this stylesheet.
+/*! This method also works for ESymbol, EGradient, ETiling, and
+  EEffect.  It is okay if the symbolic attribute is not defined in the
+  stylesheet, nothing happens in this case. */
+void StyleSheet::remove(Kind kind, Attribute sym)
+{
+  switch (kind) {
+  case ETiling:
+    iTilings.erase(sym.index());
+    break;
+  case ESymbol:
+    iSymbols.erase(sym.index());
+    break;
+  case EGradient:
+    iGradients.erase(sym.index());
+    break;
+  case EEffect:
+    iEffects.erase(sym.index());
+    break;
+  default:
+    iMap.erase(sym.index() | (kind << SHIFT));
+    break;
+  };
+}
+
 // --------------------------------------------------------------------
 
 //! Return all symbolic names in the style sheet cascade.
