@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -361,10 +361,14 @@ void Text::addToBBox(Rect &box, const Matrix &m, bool) const
     box.addPoint(v[i]);
 }
 
-void Text::snapVtx(const Vector &mouse, const Matrix &m,
+void Text::snapCtl(const Vector &mouse, const Matrix &m,
 		   Vector &pos, double &bound) const
 {
   (m * (matrix() * iPos)).snap(mouse, pos, bound);
+  Vector v[4];
+  quadrilateral(m, v);
+  for (int i = 0; i < 4; ++i)
+    v[i].snap(mouse, pos, bound);
 }
 
 // --------------------------------------------------------------------
@@ -433,8 +437,7 @@ void Text::setVerticalAlignment(TVerticalAlignment align)
   iVerticalAlignment = align;
 }
 
-bool Text::setAttribute(Property prop, Attribute value,
-			Attribute nStroke, Attribute nFill)
+bool Text::setAttribute(Property prop, Attribute value)
 {
   switch (prop) {
   case EPropStrokeColor:
@@ -500,7 +503,7 @@ bool Text::setAttribute(Property prop, Attribute value,
     }
     break;
   default:
-    return Object::setAttribute(prop, value, nStroke, nFill);
+    return Object::setAttribute(prop, value);
   }
   return false;
 }
