@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -30,6 +30,7 @@
 
 #include "ipelua.h"
 #include "ipedoc.h"
+#include "ipebitmap.h"
 
 #include <cerrno>
 
@@ -66,7 +67,6 @@ bool ipelua::is_type(lua_State *L, int ud, const char *tname)
 
 String ipelua::check_filename(lua_State *L, int index)
 {
-  // TODO: local encoding!
   return String(luaL_checkstring(L, index));
 }
 
@@ -103,6 +103,9 @@ static int document_constructor(lua_State *L)
     case Document::EFileOpenError:
       lua_pushfstring(L, "Error opening file: %s", strerror(errno));
       break;
+    case Document::ENotAnIpeFile:
+      lua_pushliteral(L, "The document was not created by Ipe");
+      break;
     default:
       lua_pushfstring(L, "Parsing error at position %d", reason);
       break;
@@ -137,7 +140,7 @@ static int document_tostring(lua_State *L)
 
 static int check_pageno(lua_State *L, int i, Document *d, int extra = 0)
 {
-  int n = luaL_checkint(L, i);
+  int n = (int)luaL_checkinteger(L, i);
   luaL_argcheck(L, 1 <= n && n <= d->countPages() + extra, i,
 		"invalid page number");
   return n - 1;
@@ -169,7 +172,7 @@ static int document_len(lua_State *L)
 static int document_page_iterator(lua_State *L)
 {
   Document **d = check_document(L, 1);
-  int i = luaL_checkint(L, 2);
+  int i = (int)luaL_checkinteger(L, 2);
   i = i + 1;
   if (i <= (*d)->countPages()) {
     lua_pushnumber(L, i);               // new counter
@@ -190,7 +193,7 @@ static int document_pages(lua_State *L)
   return 3;
 }
 
-// "export", "nozip", "markedview", "nocolor"
+// "export", "nozip", "markedview"
 static uint check_flags(lua_State *L, int index)
 {
   if (lua_isnoneornil(L, index))
@@ -209,10 +212,6 @@ static uint check_flags(lua_State *L, int index)
   if (lua_toboolean(L, -1))
     flags |= Document::EMarkedView;
   lua_pop(L, 1);
-  lua_getfield(L, index, "nocolor");
-  if (lua_toboolean(L, -1))
-    flags |= Document::ENoColor;
-  lua_pop(L, 1);
   return flags;
 }
 
@@ -514,7 +513,7 @@ static const struct luaL_Reg document_methods[] = {
 static int file_format(lua_State *L)
 {
   String fname = check_filename(L, 1);
-  FILE *fd = fopen(fname.z(), "rb");
+  FILE *fd = Platform::fopen(fname.z(), "rb");
   if (!fd)
     luaL_error(L, "fopen error: %s", strerror(errno));
   FileSource source(fd);
@@ -546,15 +545,18 @@ static int ipe_splinetobeziers(lua_State *L)
     lua_pop(L, 1);
   }
   bool closed = lua_toboolean(L, 2);
+  bool oldStyle = lua_toboolean(L, 3);
   std::vector<Bezier> result;
   if (closed)
     Bezier::closedSpline(v.size(), &v[0], result);
+  else if (oldStyle)
+    Bezier::oldSpline(v.size(), &v[0], result);
   else
     Bezier::spline(v.size(), &v[0], result);
   lua_createtable(L, result.size(), 0);
   for (uint i = 0; i < result.size(); ++i) {
     lua_createtable(L, 4, 1);
-    lua_pushliteral(L, "bezier");
+    lua_pushliteral(L, "spline");
     lua_setfield(L, -2, "type");
     for (int k = 0; k < 4; ++k) {
       if (k == 0 && i > 0)
@@ -575,6 +577,92 @@ static int ipe_fileExists(lua_State *L)
   return 1;
 }
 
+static int ipe_directory(lua_State *L)
+{
+  const char *path = luaL_checkstring(L, 1);
+  std::vector<String> files;
+  if (!Platform::listDirectory(path, files))
+    luaL_error(L, "cannot list directory '%s'", path);
+  lua_createtable(L, 0, files.size());
+  for (uint i = 0; i < files.size(); ++i) {
+    push_string(L, files[i]);
+    lua_rawseti(L, -2, i+1);
+  }
+  return 1;
+}
+
+#define l_checkmode(mode) \
+	(*mode != '\0' && strchr("rwa", *(mode++)) != NULL &&	\
+	(*mode != '+' || ++mode) &&  /* skip if char is '+' */	\
+	(*mode != 'b' || ++mode) &&  /* skip if char is 'b' */	\
+	(*mode == '\0'))
+
+static int ipe_fclose (lua_State *L) {
+  luaL_Stream *p = (luaL_Stream *) luaL_checkudata(L, 1, LUA_FILEHANDLE);
+  int res = fclose(p->f);
+  return luaL_fileresult(L, (res == 0), NULL);
+}
+
+// open a file for reading or writing from Lua,
+// correctly handling UTF-8 filenames on Windows
+static int ipe_openFile(lua_State *L)
+{
+  const char *filename = luaL_checkstring(L, 1);
+  const char *mode = luaL_optstring(L, 2, "r");
+
+  luaL_Stream *p = (luaL_Stream *)lua_newuserdata(L, sizeof(luaL_Stream));
+  p->closef = NULL;  /* mark file handle as 'closed' */
+  luaL_setmetatable(L, LUA_FILEHANDLE);
+  p->f = NULL;
+  p->closef = &ipe_fclose;
+
+  const char *md = mode;  /* to traverse/check mode */
+  luaL_argcheck(L, l_checkmode(md), 2, "invalid mode");
+  p->f = Platform::fopen(filename, mode);
+  return (p->f == NULL) ? luaL_fileresult(L, 0, filename) : 1;
+}
+
+static const char * const image_format_names[] = { "png", "jpeg" };
+
+static int ipe_readImage(lua_State *L)
+{
+  String s = check_filename(L, 1);
+  int fmt = luaL_checkoption(L, 2, NULL, image_format_names);
+
+  Vector dotsPerInch;
+  const char *errmsg = 0;
+
+  Bitmap bmp = fmt ?
+    Bitmap::readJpeg(s.z(), dotsPerInch, errmsg) :
+    Bitmap::readPNG(s.z(), true, dotsPerInch, errmsg);
+
+  if (bmp.isNull()) {
+    lua_pushnil(L);
+    lua_pushstring(L, errmsg);
+    return 2;
+  }
+
+  Rect r(Vector::ZERO, Vector(bmp.width(), bmp.height()));
+  Image *img = new Image(r, bmp);
+  push_object(L, img);
+  push_vector(L, dotsPerInch);
+  return 2;
+}
+
+static int image_constructor(lua_State *L)
+{
+  Rect *r = check_rect(L, 1);
+  Object *s = check_object(L, 2)->obj;
+  luaL_argcheck(L, s->type() == Object::EImage, 2, "not an image object");
+  Bitmap bm = s->asImage()->bitmap();
+
+  Image *img = new Image(*r, bm);
+  push_object(L, img);
+  return 1;
+}
+
+// --------------------------------------------------------------------
+
 static const struct luaL_Reg ipelib_functions[] = {
   { "Document", document_constructor },
   { "Page", page_constructor },
@@ -603,6 +691,10 @@ static const struct luaL_Reg ipelib_functions[] = {
   { "normalizeAngle", ipe_normalizeangle },
   { "splineToBeziers", ipe_splinetobeziers },
   { "fileExists", ipe_fileExists },
+  { "directory", ipe_directory },
+  { "openFile", ipe_openFile },
+  { "readImage", ipe_readImage },
+  { "Image", image_constructor },
   { NULL, NULL }
 };
 
