@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -49,7 +49,7 @@ void ipelua::push_page(lua_State *L, Page *page, bool owned)
 
 static int check_objno(lua_State *L, int i, Page *p, int extra = 0)
 {
-  int n = luaL_checkint(L, i);
+  int n = (int)luaL_checkinteger(L, i);
   luaL_argcheck(L, 1 <= n && n <= p->count() + extra,
 		i, "invalid object index");
   return n - 1;
@@ -65,7 +65,7 @@ int ipelua::check_layer(lua_State *L, int i, Page *p)
 
 int ipelua::check_viewno(lua_State *L, int i, Page *p, int extra)
 {
-  int n = luaL_checkint(L, i);
+  int n = (int)luaL_checkinteger(L, i);
   luaL_argcheck(L, 1 <= n && n <= p->countViews() + extra,
 		i, "invalid view index");
   return n - 1;
@@ -151,7 +151,7 @@ static TSelect check_select(lua_State *L, int index)
 {
   TSelect w = ENotSelected;
   if (!lua_isnoneornil(L, index)) {
-    if (luaL_checkint(L, index) == 1)
+    if ((int)luaL_checkinteger(L, index) == 1)
       w = EPrimarySelected;
     else
       w = ESecondarySelected;
@@ -163,7 +163,7 @@ static TSelect check_select(lua_State *L, int index)
 static int page_object_iterator(lua_State *L)
 {
   Page *p = check_page(L, 1)->page;
-  int i = luaL_checkint(L, 2);
+  int i = (int)luaL_checkinteger(L, 2);
   i = i + 1;
   if (i <= p->count()) {
     lua_pushnumber(L, i);                            // new counter
@@ -288,7 +288,7 @@ static int page_moveLayer(lua_State *L)
 {
   Page *p = check_page(L, 1)->page;
   int index = check_layer(L, 2, p);
-  int newIndex = luaL_checkint(L, 3) - 1;
+  int newIndex = (int)luaL_checkinteger(L, 3) - 1;
   luaL_argcheck(L, 0 <= newIndex && newIndex < p->countLayers(),
 		3, "invalid target index");
   p->moveLayer(index, newIndex);
@@ -407,13 +407,7 @@ static int page_setAttribute(lua_State *L)
   int n = check_objno(L, 2, p);
   Property prop = Property(luaL_checkoption(L, 3, NULL, property_names));
   Attribute value = check_property(prop, L, 4);
-  Attribute stroke = Attribute::BLACK();
-  Attribute fill = Attribute::WHITE();
-  if (!lua_isnoneornil(L, 5))
-    stroke = check_color_attribute(L, 5);
-  if (!lua_isnoneornil(L, 6))
-    fill = check_color_attribute(L, 6);
-  lua_pushboolean(L, p->setAttribute(n, prop, value, stroke, fill));
+  lua_pushboolean(L, p->setAttribute(n, prop, value));
   return 1;
 }
 
@@ -449,14 +443,6 @@ static int page_ensurePrimarySelection(lua_State *L)
   return 0;
 }
 
-static int page_textBox(lua_State *L)
-{
-  Page *p = check_page(L, 1)->page;
-  Cascade *sheets = check_cascade(L, 2)->cascade;
-  push_rect(L, p->textBox(sheets));
-  return 1;
-}
-
 static int page_titles(lua_State *L)
 {
   Page *p = check_page(L, 1)->page;
@@ -702,7 +688,6 @@ static const struct luaL_Reg page_methods[] = {
   { "setNotes", page_setNotes },
   { "marked", page_marked },
   { "setMarked", page_setMarked },
-  { "textBox", page_textBox },
   { 0, 0 }
 };
 
