@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -41,7 +41,8 @@ using namespace ipelua;
 // --------------------------------------------------------------------
 
 static const char * const set_names[] =
-  { "preamble", "linecap", "linejoin", "fillrule", "symbol", "layout", 0 };
+  { "preamble", "linecap", "linejoin", "fillrule", "symbol", "layout",
+    "gradient", 0 };
 
 int ipelua::test_option(lua_State *L, int i, const char * const *names)
 {
@@ -72,7 +73,7 @@ int ipelua::sheet_constructor(lua_State *L)
 {
   if (lua_type(L, 1) == LUA_TSTRING) {
     String fname = check_filename(L, 1);
-    FILE *fd = fopen(fname.z(), "rb");
+    FILE *fd = Platform::fopen(fname.z(), "rb");
     if (!fd) {
       lua_pushnil(L);
       lua_pushfstring(L, "fopen error: %s", strerror(errno));
@@ -88,6 +89,19 @@ int ipelua::sheet_constructor(lua_State *L)
       return 2;
     }
     push_sheet(L, sheet);
+  } else if (lua_type(L, 2)== LUA_TSTRING) {
+    size_t len;;
+    const char *s = lua_tolstring(L, 2, &len);
+    Buffer buf(s, len);
+    BufferSource source(buf);
+    ImlParser parser(source);
+    StyleSheet *sheet = parser.parseStyleSheet();
+    if (!sheet) {
+      lua_pushnil(L);
+      lua_pushfstring(L, "Parsing error at %d", parser.parsePosition());
+      return 2;
+    }
+    push_sheet(L, sheet);
   } else
     push_sheet(L, new StyleSheet());
   return 1;
@@ -172,6 +186,49 @@ static int sheet_add(lua_State *L)
   return 0;
 }
 
+static int sheet_addfrom(lua_State *L)
+{
+  StyleSheet *s = check_sheet(L, 1)->sheet;
+  StyleSheet *t = check_sheet(L, 2)->sheet;
+  Kind kind = Kind(luaL_checkoption(L, 3, NULL, kind_names));
+  const char *name = luaL_checkstring(L, 4);
+  Attribute sym(true, name);
+  switch (kind) {
+  case EGradient: {
+    const Gradient *g = t->findGradient(sym);
+    if (!g)
+      luaL_argerror(L, 4, "no such gradient");
+    s->addGradient(sym, *g);
+    break; }
+  case EEffect: {
+    const Effect *e = t->findEffect(sym);
+    if (!e)
+      luaL_argerror(L, 4, "no such effect");
+    s->addEffect(sym, *e);
+    break; }
+  case ETiling: {
+    const Tiling *g = t->findTiling(sym);
+    if (!g)
+      luaL_argerror(L, 4, "no such tiling");
+    s->addTiling(sym, *g);
+    break; }
+  default:
+    luaL_argerror(L, 3, "cannot handle this kind");
+    break;
+  }
+  return 0;
+}
+
+static int sheet_remove(lua_State *L)
+{
+  StyleSheet *s = check_sheet(L, 1)->sheet;
+  Kind kind = Kind(luaL_checkoption(L, 2, NULL, kind_names));
+  const char *name = luaL_checkstring(L, 3);
+  Attribute sym(true, name);
+  s->remove(kind, sym);
+  return 0;
+}
+
 static int sheet_isStandard(lua_State *L)
 {
   SSheet *p = check_sheet(L, 1);
@@ -241,6 +298,8 @@ static const struct luaL_Reg sheet_methods[] = {
   { "clone", sheet_clone },
   { "xml", sheet_xml },
   { "add", sheet_add },
+  { "addFrom", sheet_addfrom },
+  { "remove", sheet_remove },
   { "set", sheet_set },
   { "isStandard", sheet_isStandard },
   { "name", sheet_name },
@@ -321,6 +380,42 @@ static int push_layout(lua_State *L, const Layout &l)
   return 1;
 }
 
+static int push_gradient(lua_State *L, const Gradient &g)
+{
+  lua_createtable(L, 0, 6);
+  lua_pushstring(L, (g.iType == Gradient::EAxial) ? "axial" : "radial");
+  lua_setfield(L, -2, "type");
+  lua_createtable(L, 2, 0);
+  push_vector(L, g.iV[0]);
+  lua_rawseti(L, -2, 1);
+  push_vector(L, g.iV[1]);
+  lua_rawseti(L, -2, 2);
+  lua_setfield(L, -2, "v");
+  lua_pushboolean(L, g.iExtend);
+  lua_setfield(L, -2, "extend");
+  push_matrix(L, g.iMatrix);
+  lua_setfield(L, -2, "matrix");
+  if (g.iType == Gradient::ERadial) {
+    lua_createtable(L, 2, 0);
+    lua_pushnumber(L, g.iRadius[0]);
+    lua_rawseti(L, -2, 1);
+    lua_pushnumber(L, g.iRadius[1]);
+    lua_rawseti(L, -2, 2);
+    lua_setfield(L, -2, "radius");
+  }
+  lua_createtable(L, g.iStops.size(), 0);
+  for (int i = 0; i < int(g.iStops.size()); ++i) {
+    lua_createtable(L, 0, 2);
+    lua_pushnumber(L, g.iStops[i].offset);
+    lua_setfield(L, -2, "offset");
+    push_color(L, g.iStops[i].color);
+    lua_setfield(L, -2, "color");
+    lua_rawseti(L, -2, i+1);
+  }
+  lua_setfield(L, -2, "stops");
+  return 1;
+}
+
 // find will also work for the values that are "set"
 static int cascade_find(lua_State *L)
 {
@@ -351,6 +446,10 @@ static int cascade_find(lua_State *L)
   case 5:  // layout
     push_layout(L, *s->findLayout());
     break;
+  case 6: { // gradient
+    const char *name = luaL_checkstring(L, 3);
+    push_gradient(L, *s->findGradient(Attribute(true, name)));
+    break; }
   default: {
     Kind kind = Kind(luaL_checkoption(L, 2, NULL, kind_names));
     const char *name = luaL_checkstring(L, 3);
@@ -380,7 +479,7 @@ static int cascade_count(lua_State *L)
 static int cascade_sheet(lua_State *L)
 {
   Cascade *p = check_cascade(L, 1)->cascade;
-  int index = luaL_checkint(L, 2);
+  int index = (int)luaL_checkinteger(L, 2);
   luaL_argcheck(L, 1 <= index && index <= p->count(), 2, "index out of bounds");
   push_sheet(L, p->sheet(index - 1), false);
   return 1;
@@ -389,7 +488,7 @@ static int cascade_sheet(lua_State *L)
 static int cascade_insert(lua_State *L)
 {
   Cascade *p = check_cascade(L, 1)->cascade;
-  int index = luaL_checkint(L, 2);
+  int index = (int)luaL_checkinteger(L, 2);
   luaL_argcheck(L, 1 <= index && index <= p->count() + 1,
 		2, "index out of bounds");
   SSheet *s = check_sheet(L, 3);
@@ -404,7 +503,7 @@ static int cascade_insert(lua_State *L)
 static int cascade_remove(lua_State *L)
 {
   Cascade *p = check_cascade(L, 1)->cascade;
-  int index = luaL_checkint(L, 2);
+  int index = (int)luaL_checkinteger(L, 2);
   luaL_argcheck(L, 1 <= index && index <= p->count(),
 		2, "index out of bounds");
   p->remove(index - 1);
