@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -37,8 +37,6 @@
 using namespace ipe;
 using namespace ipelua;
 
-extern int luaopen_lfs (lua_State *L);
-
 // --------------------------------------------------------------------
 
 static int traceback (lua_State *L)
@@ -79,7 +77,6 @@ static lua_State *setup_lua()
   lua_State *L = luaL_newstate();
   luaL_openlibs(L);
   luaopen_ipe(L);
-  luaopen_lfs(L);
   return L;
 }
 
@@ -97,7 +94,11 @@ static void setup_globals(lua_State *L)
 	i += 1;
       String d = scripts.substr(j, i-j);
       if (d == "_")
+#ifdef IPEBUNDLE
+	d = Platform::ipeDir("scripts", "?.lua");
+#else
 	d = IPESCRIPTDIR;
+#endif
       d += "/?.lua";
       if (!s.empty())
 	s += ";";
@@ -107,8 +108,8 @@ static void setup_globals(lua_State *L)
     lua_pushstring(L, s.z());
   } else {
     String s = "./?.lua;";
-#ifdef WIN32
-    s += Platform::ipeDir("scripts/?.lua");
+#ifdef IPEBUNDLE
+    s += Platform::ipeDir("scripts", "?.lua");
 #else
     const char *home = getenv("HOME");
     if (home) {
@@ -131,7 +132,7 @@ static void setup_globals(lua_State *L)
 #endif
   lua_setfield(L, -2, "platform");
 
-#ifdef WIN32
+#ifdef IPEBUNDLE
   setup_config(L, "system_styles", "styles");
 #else
   setup_config(L, "system_styles", IPESTYLEDIR);
