@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -124,7 +124,6 @@ Dialog::Dialog(lua_State *L0, WINID parent, const char *caption)
   iIgnoreEscape = false;
   iNoRows = 1;
   iNoCols = 1;
-  // TODO: handle "Ctrl+Return"
 }
 
 Dialog::~Dialog()
@@ -191,14 +190,14 @@ int Dialog::add(lua_State *L)
   luaL_checktype(L, 4, LUA_TTABLE);
   m.lua_method = LUA_NOREF;
   m.flags = 0;
-  m.row = luaL_checkint(L, 5) - 1;
-  m.col = luaL_checkint(L, 6) - 1;
+  m.row = (int)luaL_checkinteger(L, 5) - 1;
+  m.col = (int)luaL_checkinteger(L, 6) - 1;
   m.rowspan = 1;
   m.colspan = 1;
   if (!lua_isnoneornil(L, 7))
-    m.rowspan = luaL_checkint(L, 7);
+    m.rowspan = (int)luaL_checkinteger(L, 7);
   if (!lua_isnoneornil(L, 8))
-    m.colspan = luaL_checkint(L, 8);
+    m.colspan = (int)luaL_checkinteger(L, 8);
   if (m.row + m.rowspan > iNoRows)
     iNoRows = m.row + m.rowspan;
   if (m.col + m.colspan > iNoCols)
@@ -240,7 +239,21 @@ void Dialog::addLabel(lua_State *L, SElement &m)
   m.text = String(lua_tostring(L, -1));
   lua_pop(L, 1); // label
   m.minHeight = 16;
-  m.minWidth = 4 * m.text.len();
+  const char *p = m.text.z();
+  int maxlen = 0;
+  int curlen = 0;
+  while (*p) {
+    if (*p++ == '\n') {
+      m.minHeight += 8;
+      if (curlen > maxlen)
+	maxlen = curlen;
+      curlen = 0;
+    }
+    ++curlen;
+  }
+  if (curlen > maxlen)
+    maxlen = curlen;
+  m.minWidth = 4 * maxlen;
 }
 
 void Dialog::addButtonItem(lua_State *L, SElement &m)
@@ -291,6 +304,13 @@ void Dialog::addInput(lua_State *L, SElement &m)
 {
   m.minHeight = 12;
   m.minWidth = 100;
+  lua_getfield(L, 4, "select_all");
+  if (lua_toboolean(L, -1))
+    m.flags |= ESelectAll;
+  lua_getfield(L, 4, "focus");
+  if (lua_toboolean(L, -1))
+    m.flags |= EFocused;
+  lua_pop(L, 2);
 }
 
 void Dialog::addTextEdit(lua_State *L, SElement &m)
@@ -298,6 +318,12 @@ void Dialog::addTextEdit(lua_State *L, SElement &m)
   lua_getfield(L, 4, "read_only");
   if (lua_toboolean(L, -1))
     m.flags |= EReadOnly;
+  lua_getfield(L, 4, "select_all");
+  if (lua_toboolean(L, -1))
+    m.flags |= ESelectAll;
+  lua_getfield(L, 4, "focus");
+  if (lua_toboolean(L, -1))
+    m.flags |= EFocused;
   lua_getfield(L, 4, "syntax");
   if (!lua_isnil(L, -1)) {
     const char *s = lua_tostring(L, -1);
@@ -306,10 +332,11 @@ void Dialog::addTextEdit(lua_State *L, SElement &m)
     } else if (!strcmp(s, "xml")) {
       m.flags |= EXml;
     } else if (!strcmp(s, "latex")) {
+      m.flags |= ELatex;
     } else
       luaL_argerror(L, 4, "unknown syntax");
   }
-  lua_pop(L, 2); // syntax, read_only
+  lua_pop(L, 4); // syntax, focus, select_all, read_only
   m.minHeight = 48;
   m.minWidth = 100;
 }
@@ -357,7 +384,7 @@ int Dialog::findElement(lua_State *L, int index)
 int Dialog::set(lua_State *L)
 {
   const char *s = luaL_checkstring(L, 2);
-  if (!strcmp(s, "ignore-cancel")) {
+  if (!strcmp(s, "ignore-escape")) {
     iIgnoreEscape = lua_toboolean(L, 3);
     return 0;
   }
@@ -385,7 +412,7 @@ void Dialog::setUnmapped(lua_State *L, int idx)
   case EList:
   case ECombo:
     if (lua_isnumber(L, 3)) {
-      int n = luaL_checkint(L, 3);
+      int n = (int)luaL_checkinteger(L, 3);
       luaL_argcheck(L, 1 <= n && n <= int(m.items.size()), 3,
 		    "list index out of bounds");
       m.value = n - 1;
@@ -465,8 +492,8 @@ int Dialog::setStretch(lua_State *L)
     iRowStretch.push_back(0);
 
   int type = luaL_checkoption(L, 2, 0, typenames);
-  int rowcol = luaL_checkint(L, 3) - 1;
-  int stretch = luaL_checkint(L, 4);
+  int rowcol = (int)luaL_checkinteger(L, 3) - 1;
+  int stretch = (int)luaL_checkinteger(L, 4);
 
   if (type == 0) {
     luaL_argcheck(L, 0 <= rowcol && rowcol < iNoRows, 3,
@@ -553,6 +580,13 @@ static int dialog_setEnabled(lua_State *L)
   return (*dlg)->setEnabled(L);
 }
 
+static int dialog_accept(lua_State *L)
+{
+  Dialog **dlg = check_dialog(L, 1);
+  (*dlg)->acceptDialog(L);
+  return 0;
+}
+
 // --------------------------------------------------------------------
 
 static const struct luaL_Reg dialog_methods[] = {
@@ -565,6 +599,7 @@ static const struct luaL_Reg dialog_methods[] = {
   { "set", dialog_set },
   { "get", dialog_get },
   { "setEnabled", dialog_setEnabled },
+  { "accept", dialog_accept },
   { 0, 0 }
 };
 
