@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -41,6 +41,7 @@ public:
   virtual bool buildAndRun(int w, int h);
   virtual void retrieveValues();
   virtual void enableItem(int idx, bool value);
+  virtual void acceptDialog(lua_State *L);
 private:
   static void setListBoxRow(GtkTreeView *w, int row);
   static GtkWidget *createListBox(const SElement &m);
@@ -62,6 +63,13 @@ PDialog::~PDialog()
   //
 }
 
+void PDialog::acceptDialog(lua_State *L)
+{
+  int accept = lua_toboolean(L, 2);
+  (void) accept; // TODO
+  // QDialog::done(accept);
+}
+
 void PDialog::itemResponse(GtkWidget *item, PDialog *dlg)
 {
   for (int i = 0; i < int(dlg->iWidgets.size()); ++i) {
@@ -447,8 +455,8 @@ PMenu::~PMenu()
 
 int PMenu::execute(lua_State *L)
 {
-  iPopupX = luaL_checkint(L, 2);
-  iPopupY = luaL_checkint(L, 3);
+  iPopupX = (int)luaL_checkinteger(L, 2);
+  iPopupY = (int)luaL_checkinteger(L, 3);
   iSelectedItem = -1;
   gtk_menu_popup(GTK_MENU(iMenu), NULL, NULL,
 		 // GtkMenuPositionFunc(positionResponse), this,
@@ -636,14 +644,13 @@ static int ipeui_fileDialog(lua_State *L)
   GtkWindow *parent = GTK_WINDOW(check_winid(L, 1));
   int type = luaL_checkoption(L, 2, 0, typenames);
   const char *caption = luaL_checkstring(L, 3);
-  // const char *filter = luaL_checkstring(L, 4); // not used
+  // GTK dialog uses no filters: args 4 and 7 are not used
   const char *dir = 0;
   if (!lua_isnoneornil(L, 5))
     dir = luaL_checkstring(L, 5);
-  // GTK dialog uses no filters:
-  // const char *selected = 0;
-  // if (!lua_isnoneornil(L, 6))
-  //   selected = luaL_checkstring(L, 6);
+  const char *name = 0;
+  if (!lua_isnoneornil(L, 6))
+    name = luaL_checkstring(L, 6);
 
   GtkWidget *dlg =
     gtk_file_chooser_dialog_new(caption, parent,
@@ -655,13 +662,15 @@ static int ipeui_fileDialog(lua_State *L)
 				NULL);
   if (dir)
     gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dlg), dir);
+  if (name)
+    gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(dlg), name);
 
   int result = gtk_dialog_run(GTK_DIALOG(dlg));
 
   if (result == GTK_RESPONSE_ACCEPT) {
     char *fn = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dlg));
     lua_pushstring(L, fn);
-    lua_pushstring(L, "ipe");
+    lua_pushinteger(L, 1); // name filters not used
     g_free(fn);
     gtk_widget_destroy(dlg);
     return 2;
@@ -688,7 +697,7 @@ static int ipeui_messageBox(lua_State *L)
     details = luaL_checkstring(L, 4);
   int buttons = 0;
   if (lua_isnumber(L, 5))
-    buttons = luaL_checkint(L, 5);
+    buttons = (int)luaL_checkinteger(L, 5);
   else if (!lua_isnoneornil(L, 5))
     buttons = luaL_checkoption(L, 5, 0, buttontype);
 
@@ -815,7 +824,7 @@ gboolean PTimer::elapsed()
 // does not update interval on running timer
 int PTimer::setInterval(lua_State *L)
 {
-  int t = luaL_checkint(L, 2);
+  int t = (int)luaL_checkinteger(L, 2);
   iInterval = t;
   return 0;
 }
@@ -874,52 +883,14 @@ static int timer_constructor(lua_State *L)
 
 // --------------------------------------------------------------------
 
-static int ipeui_closeAllWindows(lua_State *L)
-{
-  luaL_error(L, "'closeAllWindows' is not yet implemented.");
-  return 0;
-}
-
 static int ipeui_wait(lua_State *L)
 {
-  luaL_error(L, "'WaitDialog' is not yet implemented.");
+  luaL_error(L, "'waitDialog' is not yet implemented.");
   return 0;
 }
 
 // --------------------------------------------------------------------
 
-static int ipeui_setClipboard(lua_State *L)
-{
-  const char *data = luaL_checkstring(L, 1);
-  GtkClipboard *b = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
-  gtk_clipboard_set_text(b, data, -1);
-  return 0;
-}
-
-static int ipeui_clipboard(lua_State *L)
-{
-  GtkClipboard *b = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
-  char *data = gtk_clipboard_wait_for_text(b);
-  if (data) {
-    lua_pushstring(L, data);
-    g_free(data);
-    return 1;
-  }
-  return 0;
-}
-
-static int ipeui_local8bit(lua_State *L)
-{
-  const char *s = luaL_checkstring(L, 1);
-  gchar *r = g_locale_from_utf8(s, -1, NULL, NULL, NULL);
-  if (r) {
-    lua_pushstring(L, r);
-    g_free(r);
-    return 1;
-  }
-  return 0;
-}
-
 static int ipeui_currentDateTime(lua_State *L)
 {
   time_t t = time(NULL);
@@ -944,12 +915,8 @@ static const struct luaL_Reg ipeui_functions[] = {
   { "getColor", ipeui_getColor },
   { "fileDialog", ipeui_fileDialog },
   { "messageBox", ipeui_messageBox },
-  { "WaitDialog", ipeui_wait },
-  { "closeAllWindows", ipeui_closeAllWindows },
+  { "waitDialog", ipeui_wait },
   { "currentDateTime", ipeui_currentDateTime },
-  { "setClipboard", ipeui_setClipboard },
-  { "clipboard", ipeui_clipboard },
-  { "local8bit", ipeui_local8bit },
   { 0, 0},
 };
 
