@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -32,7 +32,6 @@
 
 #include <QApplication>
 #include <QCheckBox>
-#include <QClipboard>
 #include <QCloseEvent>
 #include <QColorDialog>
 #include <QComboBox>
@@ -53,10 +52,6 @@
 #include <QThread>
 #include <QTimer>
 
-#if defined(WIN32)
-#include <windows.h>
-#endif
-
 // --------------------------------------------------------------------
 
 inline void push_string(lua_State *L, const QString &str)
@@ -325,6 +320,8 @@ bool PDialog::buildAndRun(int w, int h)
 	{
 	  QLineEdit *e = new QLineEdit(this);
 	  e->setText(QString::fromUtf8(m.text.z()));
+	  if (m.flags & ESelectAll)
+	    e->selectAll();
 	  w = e;
 	}
 	break;
@@ -343,6 +340,8 @@ bool PDialog::buildAndRun(int w, int h)
 	    markupLog(t, text);
 	  } else
 	    t->setPlainText(text);
+	  if (m.flags & ESelectAll)
+	    t->selectAll();
 	  w = t;
 	}
 	break;
@@ -368,8 +367,11 @@ bool PDialog::buildAndRun(int w, int h)
       }
       iWidgets.push_back(w);
       gridlayout()->addWidget(w, m.row, m.col, m.rowspan, m.colspan);
+      if (m.flags & EFocused)
+	w->setFocus(Qt::OtherFocusReason);
     }
   }
+  setMinimumSize(w, h);
   int result = exec();
   retrieveValues(); // for future reference
   return (result == QDialog::Accepted);
@@ -411,6 +413,12 @@ void PDialog::enableItem(int idx, bool value)
   iWidgets[idx]->setEnabled(value);
 }
 
+void PDialog::acceptDialog(lua_State *L)
+{
+  int accept = lua_toboolean(L, 2);
+  QDialog::done(accept);
+}
+
 // --------------------------------------------------------------------
 
 static int dialog_constructor(lua_State *L)
@@ -453,8 +461,8 @@ PMenu::~PMenu()
 
 int PMenu::execute(lua_State *L)
 {
-  int vx = luaL_checkint(L, 2);
-  int vy = luaL_checkint(L, 3);
+  int vx = (int)luaL_checkinteger(L, 2);
+  int vy = (int)luaL_checkinteger(L, 3);
   QAction *a = iMenu->exec(QPoint(vx, vy));
   MenuAction *ma = qobject_cast<MenuAction *>(a);
   if (ma) {
@@ -608,7 +616,7 @@ void EditorDialog::closeEvent(QCloseEvent *ev)
 
 static int ipeui_wait(lua_State *L)
 {
-  QString cmd = checkstring(L, 1);
+  QString cmd = checkstring(L, 2);
 
   EditorDialog *dialog = new EditorDialog();
 
@@ -645,7 +653,7 @@ void PTimer::elapsed()
 
 int PTimer::setInterval(lua_State *L)
 {
-  int t = luaL_checkint(L, 2);
+  int t = (int)luaL_checkinteger(L, 2);
   iTimer->setInterval(t);
   return 0;
 }
@@ -725,18 +733,30 @@ static int ipeui_fileDialog(lua_State *L)
   QWidget *parent = check_winid(L, 1);
   int type = luaL_checkoption(L, 2, 0, typenames);
   QString caption = checkstring(L, 3);
-  QString filter = checkstring(L, 4);
+  if (!lua_istable(L, 4))
+    luaL_argerror(L, 4, "table expected for filters");
+  QStringList filters;
+  int nFilters = lua_rawlen(L, 4);
+  for (int i = 1; i <= nFilters; i += 2) { // skip Windows versions
+    lua_rawgeti(L, 4, i);
+    luaL_argcheck(L, lua_isstring(L, -1), 4, "filter entry is not a string");
+    filters += checkstring(L, -1);
+    lua_pop(L, 1); // element i
+  }
 
   QString dir;
   if (!lua_isnoneornil(L, 5))
     dir = checkstring(L, 5);
-  QString selected;
+  QString name;
   if (!lua_isnoneornil(L, 6))
-    selected = checkstring(L, 6);
+    name = checkstring(L, 6);
+  int selected = 0;
+  if (!lua_isnoneornil(L, 7))
+    selected = luaL_checkinteger(L, 7);
 
   QFileDialog dialog(parent);
   dialog.setWindowTitle(caption);
-  dialog.setNameFilter(filter);
+  dialog.setNameFilters(filters);
   dialog.setConfirmOverwrite(false);
 
   if (type == 0) {
@@ -747,16 +767,23 @@ static int ipeui_fileDialog(lua_State *L)
     dialog.setAcceptMode(QFileDialog::AcceptSave);
   }
 
-  if (!selected.isNull())
-    dialog.selectFilter(selected);
+  if (selected)
+    dialog.selectNameFilter(filters[selected-1]);
   if (!dir.isNull())
     dialog.setDirectory(dir);
+  if (!name.isNull())
+    dialog.selectFile(name);
 
   if (dialog.exec() == QDialog::Accepted) {
     QStringList fns = dialog.selectedFiles();
     if (!fns.isEmpty()) {
       push_string(L, fns[0]);
-      push_string(L, dialog.selectedNameFilter());
+      QString f = dialog.selectedNameFilter();
+      int sf = 0;
+      while (filters[sf] != f && sf < nFilters/2)
+	++sf;
+      if (sf < nFilters/2)
+	lua_pushinteger(L, sf + 1);
       return 2;
     }
   }
@@ -781,7 +808,7 @@ static int ipeui_messageBox(lua_State *L)
     details = luaL_checkstring(L, 4);
   int buttons = 0;
   if (lua_isnumber(L, 5))
-    buttons = luaL_checkint(L, 5);
+    buttons = (int)luaL_checkinteger(L, 5);
   else if (!lua_isnoneornil(L, 5))
     buttons = luaL_checkoption(L, 5, 0, buttontype);
 
@@ -852,36 +879,6 @@ static int ipeui_messageBox(lua_State *L)
 
 // --------------------------------------------------------------------
 
-static int ipeui_closeAllWindows(lua_State *L)
-{
-  QApplication::closeAllWindows();
-  return 0;
-}
-
-static int ipeui_setClipboard(lua_State *L)
-{
-  QString data = checkstring(L, 1);
-  QClipboard *cb = QApplication::clipboard();
-  cb->setText(data);
-  return 0;
-}
-
-static int ipeui_clipboard(lua_State *L)
-{
-  QClipboard *cb = QApplication::clipboard();
-  QString data = cb->text();
-  push_string(L, data);
-  return 1;
-}
-
-static int ipeui_local8bit(lua_State *L)
-{
-  QString data = checkstring(L, 1);
-  QByteArray l8 = data.toLocal8Bit();
-  lua_pushstring(L, l8.data());
-  return 1;
-}
-
 static int ipeui_currentDateTime(lua_State *L)
 {
   QDateTime dt = QDateTime::currentDateTime();
@@ -893,15 +890,6 @@ static int ipeui_currentDateTime(lua_State *L)
   return 1;
 }
 
-#if defined(WIN32)
-static int ipeui_startBrowser(lua_State *L)
-{
-  const char *url = luaL_checkstring(L, 1);
-  ShellExecuteA(0, "open", url, 0, 0, 0);
-  return 0;
-}
-#endif
-
 // --------------------------------------------------------------------
 
 static const struct luaL_Reg ipeui_functions[] = {
@@ -911,15 +899,8 @@ static const struct luaL_Reg ipeui_functions[] = {
   { "getColor", ipeui_getColor },
   { "fileDialog", ipeui_fileDialog },
   { "messageBox", ipeui_messageBox },
-  { "WaitDialog", ipeui_wait },
-  { "closeAllWindows", ipeui_closeAllWindows },
+  { "waitDialog", ipeui_wait },
   { "currentDateTime", ipeui_currentDateTime },
-  { "setClipboard", ipeui_setClipboard },
-  { "clipboard", ipeui_clipboard },
-  { "local8bit", ipeui_local8bit },
-#ifdef WIN32
-  { "startBrowser", ipeui_startBrowser },
-#endif
   { 0, 0},
 };
 
