@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -29,15 +29,98 @@
 */
 
 #include "ipeui_common.h"
+#include "ipeui_wstring.h"
 
 #include <windowsx.h>
 
 // --------------------------------------------------------------------
 
 #define IDBASE 9000
-#define PAD 4
-#define BORDER 8
-#define BUTTONHEIGHT 16
+#define PAD 3
+#define BORDER 6
+#define BUTTONHEIGHT 14
+
+// --------------------------------------------------------------------
+
+void WString::init(const char *s)
+{
+  int rw = MultiByteToWideChar(CP_UTF8, 0, s, -1, NULL, 0);
+  iData.resize(rw);
+  MultiByteToWideChar(CP_UTF8, 0, s, -1, &iData[0], rw);
+}
+
+void WString::appendTo(std::vector<wchar_t> &out)
+{
+  for (int i = 0; i < int(iData.size()); ++i)
+    out.push_back(iData[i]);
+}
+
+BOOL setWindowText(HWND h, const char *s)
+{
+  WString ws(s);
+  wchar_t *p = ws.data();
+  std::vector<wchar_t> w;
+  w.reserve(ws.size());
+  while (*p) {
+    wchar_t ch = *p++;
+    if (ch != '\r') {
+      if (ch == '\n')
+	w.push_back('\r');
+      w.push_back(ch);
+    }
+  }
+  w.push_back(0);
+  return SetWindowTextW(h, &w[0]);
+}
+
+void sendMessage(HWND h, UINT code, const char *t, WPARAM wParam)
+{
+  WString w(t);
+  SendMessageW(h, code, wParam, (LPARAM) w.data());
+}
+
+static String wideToUtf8(const wchar_t *wbuf)
+{
+  int rm = WideCharToMultiByte(CP_UTF8, 0, wbuf, -1, NULL, 0, NULL, NULL);
+  char multi[rm];
+  WideCharToMultiByte(CP_UTF8, 0, wbuf, -1, multi, rm, NULL, NULL);
+  return String(multi);
+}
+
+static void buildFlags(std::vector<short> &t, DWORD flags)
+{
+  union {
+    DWORD dw;
+    short s[2];
+  } a;
+  a.dw = flags;
+  t.push_back(a.s[0]);
+  t.push_back(a.s[1]);
+  t.push_back(0);
+  t.push_back(0);
+}
+
+static void buildString(std::vector<short> &t, const char *s)
+{
+  WString w(s);
+  const wchar_t *p = w;
+  while (*p)
+    t.push_back(*p++);
+  t.push_back(0);
+}
+
+static void buildControl(std::vector<short> &t, short what, const char *s=0)
+{
+  t.push_back(0xffff);
+  t.push_back(what);
+  if (s)
+    buildString(t, s);
+  else
+    t.push_back(0);    // text
+  t.push_back(0);      // creation data
+}
+
+// --------------------------------------------------------------------
 
 class PDialog : public Dialog {
 public:
@@ -47,17 +130,16 @@ public:
 protected:
   virtual void setMapped(lua_State *L, int idx);
   virtual bool buildAndRun(int w, int h);
+  void buildElements(std::vector<short> &t);
   virtual void retrieveValues();
   virtual void enableItem(int idx, bool value);
 
   void computeDimensions(int &w, int &h);
-
-  void buildFlags(std::vector<short> &t, DWORD flags);
-  void buildString(std::vector<short> &t, const char *s);
   void buildDimensions(std::vector<short> &t, SElement &m, int id);
-  void buildControl(std::vector<short> &t, short what, const char *s = 0);
 
-  void initDialog();
+  virtual void acceptDialog(lua_State *L);
+
+  BOOL initDialog();
   BOOL dlgCommand(WPARAM wParam, LPARAM lParam);
   static BOOL CALLBACK dialogProc(HWND hwndDlg, UINT message,
 				  WPARAM wParam, LPARAM lParam);
@@ -74,7 +156,6 @@ PDialog::PDialog(lua_State *L0, WINID parent, const char *caption)
   LONG base = GetDialogBaseUnits();
   iBaseX = LOWORD(base);
   iBaseY = HIWORD(base);
-  // TODO: handle "Ctrl+Return"
 }
 
 PDialog::~PDialog()
@@ -90,13 +171,13 @@ void PDialog::setMapped(lua_State *L, int idx)
   case ETextEdit:
   case EInput:
   case ELabel:
-    SetWindowText(h, m.text.z());
+    setWindowText(h, m.text);
     break;
   case EList:
     if (!lua_isnumber(L, 3)) {
       ListBox_ResetContent(h);
       for (int j = 0; j < int(m.items.size()); ++j)
-	ListBox_AddString(h, m.items[j].z());
+	sendMessage(h, LB_ADDSTRING, m.items[j]);
     }
     ListBox_SetCurSel(h, m.value);
     break;
@@ -104,7 +185,7 @@ void PDialog::setMapped(lua_State *L, int idx)
     if (!lua_isnumber(L, 3)) {
       ComboBox_ResetContent(h);
       for (int j = 0; j < int(m.items.size()); ++j)
-	ComboBox_AddString(h, m.items[j].z());
+	sendMessage(h, CB_ADDSTRING, m.items[j]);
     }
     ComboBox_SetCurSel(h, m.value);
     break;
@@ -117,8 +198,35 @@ void PDialog::setMapped(lua_State *L, int idx)
   }
 }
 
-void PDialog::initDialog()
+static void markupLog(HWND h, String text)
 {
+  const char *p = text.z();
+  if (*p) ++p;
+  int pos = 1;
+  while (*p) {
+    if (p[-1] == '\n' && p[0] == '!') {
+      // found it
+      const char *b = p;
+      while (*p && *p != '\n')
+	++p;
+      SendMessage(h, EM_SETSEL, (WPARAM) pos, (LPARAM) pos + (p - b));
+      int line = SendMessage(h, EM_LINEFROMCHAR, (WPARAM) pos, 0);
+      SendMessage(h, EM_LINESCROLL, 0, (LPARAM) line - 1);
+      return;
+    }
+    // take into account conversion done by setText
+    if (*p == '\n')
+      pos += 2;
+    else if (*p != '\r')
+      ++pos;
+    ++p;
+  }
+  // not found, nothing to be done
+}
+
+BOOL PDialog::initDialog()
+{
+  BOOL result = TRUE;
   for (int i = 0; i < int(iElements.size()); ++i) {
     SElement &m = iElements[i];
     HWND h = GetDlgItem(hDialog, i+IDBASE);
@@ -127,16 +235,22 @@ void PDialog::initDialog()
     switch (m.type) {
     case EInput:
     case ETextEdit:
-      SetWindowText(h, m.text.z());
+      setWindowText(h, m.text);
+      if (m.flags & EFocused) {
+	SetFocus(h);
+	result = FALSE; // we set the focus ourselves
+      }
+      if (m.flags & ELogFile)
+	markupLog(h, m.text);
       break;
     case EList:
       for (int j = 0; j < int(m.items.size()); ++j)
-	ListBox_AddString(h, m.items[j].z());
+	sendMessage(h, LB_ADDSTRING, m.items[j]);
       ListBox_SetCurSel(h, m.value);
       break;
     case ECombo:
       for (int j = 0; j < int(m.items.size()); ++j)
-	ComboBox_AddString(h, m.items[j].z());
+	sendMessage(h, CB_ADDSTRING, m.items[j]);
       ComboBox_SetCurSel(h, m.value);
       break;
     case ECheckBox:
@@ -147,16 +261,25 @@ void PDialog::initDialog()
       break;
     }
   }
+  return result;
 }
 
 static String getEditText(HWND h)
 {
-  int n = Edit_GetTextLength(h);
+  int n = GetWindowTextLengthW(h);
   if (n == 0)
     return String("");
-  char buf[n+1];
-  Edit_GetText(h, buf, n+1);
-  return String(buf);
+  WCHAR wbuf[n+1];
+  GetWindowTextW(h, wbuf, n+1);
+  std::vector<wchar_t> w;
+  wchar_t *p = wbuf;
+  while (*p) {
+    wchar_t ch = *p++;
+    if (ch != '\r')
+      w.push_back(ch);
+  }
+  w.push_back(0);
+  return wideToUtf8(&w[0]);
 }
 
 void PDialog::retrieveValues()
@@ -189,19 +312,6 @@ void PDialog::retrieveValues()
   }
 }
 
-void PDialog::buildFlags(std::vector<short> &t, DWORD flags)
-{
-  union {
-    DWORD dw;
-    short s[2];
-  } a;
-  a.dw = flags;
-  t.push_back(a.s[0]);
-  t.push_back(a.s[1]);
-  t.push_back(0);
-  t.push_back(0);
-}
-
 void PDialog::buildDimensions(std::vector<short> &t, SElement &m, int id)
 {
   int x = BORDER;
@@ -214,8 +324,8 @@ void PDialog::buildDimensions(std::vector<short> &t, SElement &m, int id)
       y += iRowHeight[i] + PAD;
     w = m.minWidth;
     h = BUTTONHEIGHT;
-    x = iButtonX - w;
-    iButtonX -= w + PAD;
+    x = iButtonX;
+    iButtonX += w + PAD;
   } else {
     for (int i = 0; i < m.col; ++i)
       x += iColWidth[i] + PAD;
@@ -235,27 +345,6 @@ void PDialog::buildDimensions(std::vector<short> &t, SElement &m, int id)
   t.push_back(id); // control id
 }
 
-void PDialog::buildString(std::vector<short> &t, const char *s)
-{
-  int rw = MultiByteToWideChar(CP_UTF8, 0, s, -1, NULL, 0);
-  wchar_t wide[rw];
-  MultiByteToWideChar(CP_UTF8, 0, s, -1, wide, rw);
-  for (int i = 0; i < rw; ++i)
-    t.push_back(wide[i]);
-}
-
-void PDialog::buildControl(std::vector<short> &t, short what,
-			  const char *s)
-{
-  t.push_back(0xffff);
-  t.push_back(what);
-  if (s)
-    buildString(t, s);
-  else
-    t.push_back(0);      // text
-  t.push_back(0);      // creation data
-}
-
 BOOL PDialog::dlgCommand(WPARAM wParam, LPARAM lParam)
 {
   int id = LOWORD(wParam);
@@ -285,12 +374,19 @@ BOOL PDialog::dlgCommand(WPARAM wParam, LPARAM lParam)
   return FALSE;
 }
 
+void PDialog::acceptDialog(lua_State *L)
+{
+  int accept = lua_toboolean(L, 2);
+  retrieveValues();
+  EndDialog(hDialog, accept);
+}
+
 static WNDPROC wpOrigProc;
 static LRESULT subclassProc(HWND hwnd, UINT message,
 			    WPARAM wParam, LPARAM lParam)
 {
   if (message == WM_KEYDOWN) {
-    if (wParam == VK_RETURN && GetKeyState(VK_CONTROL)) {
+    if (wParam == VK_RETURN && (GetKeyState(VK_CONTROL) & 0x8000)) {
       SendMessage(GetParent(hwnd), WM_COMMAND, IDOK, 0);
       return 0;
     }
@@ -309,15 +405,15 @@ BOOL CALLBACK PDialog::dialogProc(HWND hwnd, UINT message,
     d = (PDialog *) lParam;
     SetWindowLongPtr(hwnd, GWLP_USERDATA, (LONG_PTR) d);
     d->hDialog = hwnd;
-    d->initDialog();
     // subclass all Edit controls
+    // TODO: Use modern SetSubclass method
     for (int i = 0; i < int(d->iElements.size()); ++i) {
       if (d->iElements[i].type == ETextEdit)
 	wpOrigProc =
 	  (WNDPROC) SetWindowLong(GetDlgItem(hwnd, i + IDBASE),
 				  GWL_WNDPROC, (LONG) subclassProc);
     }
-    return TRUE;
+    return d->initDialog();
 
   case WM_COMMAND:
     if (d)
@@ -337,31 +433,8 @@ BOOL CALLBACK PDialog::dialogProc(HWND hwnd, UINT message,
   }
 }
 
-bool PDialog::buildAndRun(int w, int h)
+void PDialog::buildElements(std::vector<short> &t)
 {
-  computeDimensions(w, h);
-
-  RECT rect;
-  GetWindowRect(iParent, &rect);
-  int pw = (rect.right - rect.left) * 4 / iBaseX;
-  int ph = (rect.bottom - rect.top) * 8 / iBaseY;
-
-  std::vector<short> t;
-
-  // Dialog flags
-  buildFlags(t, WS_POPUP | WS_BORDER | WS_SYSMENU | DS_MODALFRAME | WS_CAPTION);
-  t.push_back(iElements.size());
-  t.push_back((pw - w)/2);  // offset of popup-window from parent window
-  t.push_back((ph - h)/2);
-  t.push_back(w);
-  t.push_back(h);
-  // menu
-  t.push_back(0);
-  // class
-  t.push_back(0);
-  // title
-  buildString(t, iCaption.z());
-  // add elements
   for (int i = 0; i < int(iElements.size()); ++i) {
     if (t.size() % 2 != 0)
       t.push_back(0);
@@ -370,7 +443,7 @@ bool PDialog::buildAndRun(int w, int h)
     DWORD flags = WS_CHILD|WS_VISIBLE;
     switch (m.type) {
     case EButton:
-      flags |= BS_TEXT|WS_TABSTOP;
+      flags |= BS_TEXT|WS_TABSTOP|BS_FLAT;
       if (m.flags & EAccept)
 	flags |= BS_DEFPUSHBUTTON;
       else
@@ -417,10 +490,41 @@ bool PDialog::buildAndRun(int w, int h)
       break;
     }
   }
+}
+
+bool PDialog::buildAndRun(int w, int h)
+{
+  computeDimensions(w, h);
+
+  RECT rect;
+  GetWindowRect(iParent, &rect);
+  int pw = (rect.right - rect.left) * 4 / iBaseX;
+  int ph = (rect.bottom - rect.top) * 8 / iBaseY;
+
+  std::vector<short> t;
+
+  // Dialog flags
+  buildFlags(t, WS_POPUP | WS_BORDER | DS_SHELLFONT |
+	     WS_SYSMENU | DS_MODALFRAME | WS_CAPTION);
+  t.push_back(iElements.size());
+  t.push_back((pw - w)/2);  // offset of popup-window from parent window
+  t.push_back((ph - h)/2);
+  t.push_back(w);
+  t.push_back(h);
+  // menu
+  t.push_back(0);
+  // class
+  t.push_back(0);
+  // title
+  buildString(t, iCaption.z());
+  // for DS_SHELLFONT
+  t.push_back(10);
+  buildString(t,"MS Shell Dlg");
+  buildElements(t);
   int res =
-    DialogBoxIndirectParam((HINSTANCE) GetWindowLong(iParent, GWL_HINSTANCE),
-			   (LPCDLGTEMPLATE) &t[0],
-			   iParent, dialogProc, (LPARAM) this);
+    DialogBoxIndirectParamW((HINSTANCE) GetWindowLong(iParent, GWL_HINSTANCE),
+			    (LPCDLGTEMPLATE) &t[0],
+			    iParent, dialogProc, (LPARAM) this);
   // retrieveValues() has been called before EndDialog!
   hDialog = NULL; // already destroyed by Windows
   return (res > 0);
@@ -484,7 +588,7 @@ void PDialog::computeDimensions(int &w, int &h)
   if (buttonWidth + BORDER + BORDER > w)
     w = buttonWidth + BORDER + BORDER;
 
-  iButtonX = w - (w - buttonWidth) / 2;
+  iButtonX = (w - buttonWidth) / 2;
 
   int spareW = w - totalW;
   int spareH = h - totalH;
@@ -514,6 +618,13 @@ void PDialog::computeDimensions(int &w, int &h)
       iRowHeight[i] = minHeight[i] + extra;
     }
   }
+  /*
+  fprintf(stderr, "iColWidth\n");
+  for (int i = 0; i < iNoCols; ++i)
+    fprintf(stderr, "%d ", iColWidth[i]);
+  fprintf(stderr, "\n");
+  fflush(stderr);
+  */
 }
 
 void PDialog::enableItem(int idx, bool value)
@@ -574,8 +685,8 @@ PMenu::~PMenu()
 
 int PMenu::execute(lua_State *L)
 {
-  int vx = luaL_checkint(L, 2);
-  int vy = luaL_checkint(L, 3);
+  int vx = (int)luaL_checkinteger(L, 2);
+  int vy = (int)luaL_checkinteger(L, 3);
   int result = TrackPopupMenu(hMenu,
 			      TPM_NONOTIFY | TPM_RETURNCMD | TPM_RIGHTBUTTON,
 			      vx, vy, 0, hwnd, NULL);
@@ -617,9 +728,9 @@ static HBITMAP colorIcon(double red, double green, double blue)
 int PMenu::add(lua_State *L)
 {
   const char *name = luaL_checkstring(L, 2);
-  const char *title = luaL_checkstring(L, 3);
+  WString title(luaL_checkstring(L, 3));
   if (lua_gettop(L) == 3) {
-    AppendMenu(hMenu, MF_STRING, items.size() + 1, title);
+    AppendMenuW(hMenu, MF_STRING, items.size() + 1, title);
     Item item;
     item.name = String(name);
     item.itemIndex = 0;
@@ -639,7 +750,7 @@ int PMenu::add(lua_State *L)
 		    "argument is not a function or string");
       current = luaL_checkstring(L, 6);
     }
-    int no = lua_objlen(L, 4);
+    int no = lua_rawlen(L, 4);
     HMENU sm = CreatePopupMenu();
     for (int i = 1; i <= no; ++i) {
       lua_rawgeti(L, 4, i);
@@ -659,8 +770,8 @@ int PMenu::add(lua_State *L)
       } else
 	lua_pushvalue(L, -1);
 
-      const char *text = lua_tostring(L, -1);
-      AppendMenu(sm, MF_STRING, id, text);
+      WString text(lua_tostring(L, -1));
+      AppendMenuW(sm, MF_STRING, id, text);
       Item mitem;
       mitem.name = String(name);
       mitem.itemName = String(item);
@@ -684,7 +795,7 @@ int PMenu::add(lua_State *L)
       }
       lua_pop(L, 2); // item, text
     }
-    AppendMenu(hMenu, MF_STRING | MF_POPUP, UINT(sm), title);
+    AppendMenuW(hMenu, MF_STRING | MF_POPUP, UINT(sm), title);
   }
   return 0;
 }
@@ -769,7 +880,7 @@ void PTimer::elapsed()
 
 int PTimer::setInterval(lua_State *L)
 {
-  int t = luaL_checkint(L, 2);
+  int t = (int)luaL_checkinteger(L, 2);
   iInterval = t;
   if (iTimer)
     SetTimer(NULL, iTimer, iInterval, TimerProc);
@@ -858,54 +969,73 @@ static int ipeui_getColor(lua_State *L)
 
 // --------------------------------------------------------------------
 
-
 static int ipeui_fileDialog(lua_State *L)
 {
   static const char * const typenames[] = { "open", "save", 0 };
   HWND hwnd = check_winid(L, 1);
   int type = luaL_checkoption(L, 2, 0, typenames);
-  const char *caption = luaL_checkstring(L, 3);
-  // const char *filter = luaL_checkstring(L, 4);
+  WString caption(luaL_checkstring(L, 3));
+  if (!lua_istable(L, 4))
+    luaL_argerror(L, 4, "table expected for filters");
+  std::vector<wchar_t> filters;
+  int nFilters = lua_rawlen(L, 4);
+  for (int i = 1; i <= nFilters; ++i) {
+    lua_rawgeti(L, 4, i);
+    luaL_argcheck(L, lua_isstring(L, -1), 4, "filter entry is not a string");
+    WString el(lua_tostring(L, -1));
+    el.appendTo(filters);
+    lua_pop(L, 1); // element i
+  }
+  filters.push_back(0); // terminate list
+
   const char *dir = 0;
   if (!lua_isnoneornil(L, 5))
     dir = luaL_checkstring(L, 5);
-  // const char *selected = 0;
-  // if (!lua_isnoneornil(L, 6))
-  //   selected = luaL_checkstring(L, 6);
+  const char *name = 0;
+  if (!lua_isnoneornil(L, 6))
+    name = luaL_checkstring(L, 6);
+  int selected = 0;
+  if (!lua_isnoneornil(L, 7))
+    selected = luaL_checkinteger(L, 7);
 
-  OPENFILENAME ofn;
-  char szFileName[MAX_PATH] = "";
+  OPENFILENAMEW ofn;
+  wchar_t szFileName[MAX_PATH] = L"";
 
   ZeroMemory(&ofn, sizeof(ofn));
   ofn.lStructSize = sizeof(ofn);
   ofn.hwndOwner = hwnd;
 
-  // TODO!
-  ofn.lpstrFilter = "Ipe Files\0*.ipe;*.pdf;*.eps\0All Files\0*.*\0";
-  ofn.nFilterIndex = 1;
+  ofn.lpstrFilter = &filters[0];
+  ofn.nFilterIndex = selected;
 
   ofn.lpstrFile = szFileName;
   ofn.nMaxFile = MAX_PATH;
-  ofn.lpstrDefExt = "ipe";
-  ofn.lpstrInitialDir = dir;
+  ofn.lpstrDefExt = L"ipe";
+  if (name) {
+    WString wname(name);
+    wcsncpy(szFileName, wname.data(), MAX_PATH);
+  }
+  if (dir) {
+    WString wdir(dir);
+    ofn.lpstrInitialDir = wdir;
+  }
   ofn.lpstrTitle = caption;
 
+  BOOL result;
   if (type == 0) {
     ofn.Flags = OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY;
-    if (GetOpenFileName(&ofn)) {
-      lua_pushstring(L, ofn.lpstrFile);
-      lua_pushstring(L, "ipe");
-      return 2;
-    }
+    result = GetOpenFileNameW(&ofn);
   } else {
     ofn.Flags = OFN_EXPLORER | OFN_PATHMUSTEXIST | OFN_HIDEREADONLY;
-    if (GetSaveFileName(&ofn)) {
-      lua_pushstring(L, ofn.lpstrFile);
-      lua_pushstring(L, "ipe");
-      return 2;
-    }
+    result = GetSaveFileNameW(&ofn);
   }
-  return 0;
+  if (!result)
+    return 0;
+
+  String s = wideToUtf8(ofn.lpstrFile);
+  lua_pushstring(L, s.z());
+  lua_pushinteger(L, ofn.nFilterIndex);
+  return 2;
 }
 
 // --------------------------------------------------------------------
@@ -926,7 +1056,7 @@ static int ipeui_messageBox(lua_State *L)
     details = luaL_checkstring(L, 4);
   int buttons = 0;
   if (lua_isnumber(L, 5))
-    buttons = luaL_checkint(L, 5);
+    buttons = (int)luaL_checkinteger(L, 5);
   else if (!lua_isnoneornil(L, 5))
     buttons = luaL_checkoption(L, 5, 0, buttontype);
 
@@ -961,12 +1091,12 @@ static int ipeui_messageBox(lua_State *L)
     uType |= MB_YESNOCANCEL;
     break;
   case 3:
+    // should be Discard Cancel
     uType |= MB_OKCANCEL;
-    // TODO should be Discard Cancel
     break;
   case 4:
+    // should be Save Discard Cancel
     uType |= MB_YESNOCANCEL;
-    // TODO should be Save Discard Cancel
     break;
   }
 
@@ -974,9 +1104,12 @@ static int ipeui_messageBox(lua_State *L)
   if (details) {
     char buf[strlen(text) + strlen(details) + 8];
     sprintf(buf, "%s\n\n%s", text, details);
-    ret = MessageBox(hwnd, buf, "Ipe", uType);
-  } else
-    ret = MessageBox(hwnd, text, "Ipe", uType);
+    WString wbuf(buf);
+    ret = MessageBoxW(hwnd, wbuf, L"Ipe", uType);
+  } else {
+    WString wbuf(text);
+    ret = MessageBoxW(hwnd, wbuf, L"Ipe", uType);
+  }
 
   switch (ret) {
   case IDOK:
@@ -997,62 +1130,101 @@ static int ipeui_messageBox(lua_State *L)
 
 // --------------------------------------------------------------------
 
-static int ipeui_wait(lua_State *L)
+BOOL CALLBACK waitDialogProc(HWND hwnd, UINT message,
+			     WPARAM wParam, LPARAM lParam)
 {
-  luaL_error(L, "'WaitDialog' is not implemented on Windows.");
-  return 0;
+  switch (message) {
+  case WM_INITDIALOG: {
+    HWND *p = (HWND *) lParam;
+    *p = hwnd;
+    return TRUE; }
+  default:
+    return FALSE;
+  }
 }
 
-static int ipeui_closeAllWindows(lua_State *L)
+VOID CALLBACK waitCallback(PVOID lpParameter, BOOLEAN timerOrWaitFired)
 {
-  luaL_error(L, "'closeAllWindows' is not implemented on Windows.");
+  HWND *hDialog = (HWND *) lpParameter;
+  if (*hDialog)
+    EndDialog(*hDialog, 1);
+}
+
+static int ipeui_wait(lua_State *L)
+{
+  Dialog **dlg = check_dialog(L, 1);
+  HWND parent = (*dlg)->winId();
+
+  std::vector<short> t;
+
+  // Dialog flags
+  buildFlags(t, WS_POPUP | WS_BORDER | DS_SHELLFONT |
+	     WS_SYSMENU | DS_MODALFRAME | WS_CAPTION);
+  t.push_back(1);
+  t.push_back(0); // offset of popup-window from parent window
+  t.push_back(0);
+  t.push_back(240);
+  t.push_back(60);
+  // menu
+  t.push_back(0);
+  // class
+  t.push_back(0);
+  // title
+  buildString(t, "Ipe: waiting");
+  // for DS_SHELLFONT
+  t.push_back(10);
+  buildString(t,"MS Shell Dlg");
+  if (t.size() % 2 != 0)
+    t.push_back(0);
+  buildFlags(t, WS_CHILD|WS_VISIBLE|SS_LEFT);
+  t.push_back(40);
+  t.push_back(20);
+  t.push_back(120);
+  t.push_back(20);
+  t.push_back(IDBASE);
+  buildControl(t, 0x0082, "Waiting for external editor");
+
+  // Declare and initialize process blocks
+  PROCESS_INFORMATION processInformation;
+  STARTUPINFOW startupInfo;
+
+  memset(&processInformation, 0, sizeof(processInformation));
+  memset(&startupInfo, 0, sizeof(startupInfo));
+  startupInfo.cb = sizeof(startupInfo);
+
+  // Call the executable program
+  const char *cmd = luaL_checkstring(L, 2);
+  WString wcmd(cmd);
+
+  int result = CreateProcessW(NULL, wcmd, NULL, NULL, FALSE,
+			      NORMAL_PRIORITY_CLASS|CREATE_NO_WINDOW,
+			      NULL, NULL, &startupInfo, &processInformation);
+  if (result == 0)
+    return 0;
+
+  HWND dialogHandle = NULL;
+  HANDLE waitHandle;
+
+  RegisterWaitForSingleObject(&waitHandle,
+			      processInformation.hProcess,
+			      waitCallback,
+			      &dialogHandle,
+			      INFINITE,
+			      WT_EXECUTEINWAITTHREAD|WT_EXECUTEONLYONCE);
+
+  DialogBoxIndirectParamW((HINSTANCE) GetWindowLong(parent, GWL_HINSTANCE),
+			  (LPCDLGTEMPLATE) &t[0],
+			  parent, waitDialogProc,
+			  (LPARAM) &dialogHandle);
+
+  UnregisterWait(waitHandle);
+  CloseHandle(processInformation.hProcess);
+  CloseHandle(processInformation.hThread);
   return 0;
 }
 
 // --------------------------------------------------------------------
 
-static int ipeui_setClipboard(lua_State *L)
-{
-  const char *data = luaL_checkstring(L, 1);
-  int l = strlen(data);
-  GLOBALHANDLE hGlobal = GlobalAlloc(GHND | GMEM_SHARE, l + 1);
-  char *p = (char *) GlobalLock(hGlobal);
-  for (int i = 0; i < l; ++i)
-    *p++ = *data++;
-  GlobalUnlock(hGlobal);
-  OpenClipboard(NULL);  // hwnd!
-  EmptyClipboard();
-  SetClipboardData(CF_TEXT, hGlobal);
-  CloseClipboard();
-  return 0;
-}
-
-static int ipeui_clipboard(lua_State *L)
-{
-  OpenClipboard(NULL);  // hwnd!
-  GLOBALHANDLE hGlobal = GetClipboardData(CF_TEXT);
-  if (hGlobal == NULL)
-    return 0;
-  char *q = (char *) GlobalLock(hGlobal);
-  lua_pushstring(L, q);  // this makes a copy!
-  GlobalUnlock(hGlobal);
-  CloseClipboard();
-  return 1;
-}
-
-static int ipeui_local8bit(lua_State *L)
-{
-  const char *in = luaL_checkstring(L, 1);
-  int rw = MultiByteToWideChar(CP_UTF8, 0, in, -1, NULL, 0);
-  wchar_t wide[rw];
-  MultiByteToWideChar(CP_UTF8, 0, in, -1, wide, rw);
-  int rm = WideCharToMultiByte(CP_ACP, 0, wide, -1, NULL, 0, NULL, NULL);
-  char multi[rm];
-  WideCharToMultiByte(CP_ACP, 0, wide, -1, multi, rm, NULL, NULL);
-  lua_pushstring(L, multi);
-  return 1;
-}
-
 static int ipeui_currentDateTime(lua_State *L)
 {
   SYSTEMTIME st;
@@ -1068,8 +1240,9 @@ static int ipeui_currentDateTime(lua_State *L)
 static int ipeui_startBrowser(lua_State *L)
 {
   const char *url = luaL_checkstring(L, 1);
-  ShellExecuteA(0, "open", url, 0, 0, 0);
-  return 0;
+  int res = int(ShellExecuteA(0, "open", url, 0, 0, SW_SHOWNORMAL));
+  lua_pushboolean(L, (res >= 32));
+  return 1;
 }
 
 // --------------------------------------------------------------------
@@ -1081,12 +1254,8 @@ static const struct luaL_Reg ipeui_functions[] = {
   { "getColor", ipeui_getColor },
   { "fileDialog", ipeui_fileDialog },
   { "messageBox", ipeui_messageBox },
-  { "WaitDialog", ipeui_wait },
-  { "closeAllWindows", ipeui_closeAllWindows },
+  { "waitDialog", ipeui_wait },
   { "currentDateTime", ipeui_currentDateTime },
-  { "setClipboard", ipeui_setClipboard },
-  { "clipboard", ipeui_clipboard },
-  { "local8bit", ipeui_local8bit },
   { "startBrowser", ipeui_startBrowser },
   { 0, 0},
 };
@@ -1095,7 +1264,8 @@ static const struct luaL_Reg ipeui_functions[] = {
 
 int luaopen_ipeui(lua_State *L)
 {
-  luaL_register(L, "ipeui", ipeui_functions);
+  luaL_newlib(L, ipeui_functions);
+  lua_setglobal(L, "ipeui");
   luaopen_ipeui_common(L);
   return 0;
 }
