@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2013  Otfried Cheong
+    Copyright (C) 1993-2015  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -48,6 +48,7 @@ public:
 
   void show();
 
+  void load();
   bool load(const char* fn);
 
   void zoom(int delta);
@@ -122,7 +123,7 @@ void AppUi::addItem(int m, int action, const char *label,
 AppUi::AppUi()
 {
   iWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-  gtk_widget_set_size_request (GTK_WIDGET(iWindow), 600, 400);
+  gtk_widget_set_size_request(GTK_WIDGET(iWindow), 600, 400);
   gtk_window_set_title(GTK_WINDOW(iWindow), "Ipeview");
   g_signal_connect(iWindow, "delete-event", G_CALLBACK(gtk_main_quit), NULL);
 
@@ -146,13 +147,21 @@ AppUi::AppUi()
   iRootMenu[1] = gtk_menu_item_new_with_mnemonic("_View");
   iRootMenu[2] = gtk_menu_item_new_with_mnemonic("_Move");
 
+#if GTK_MAJOR_VERSION < 3
   vbox = gtk_vbox_new(FALSE, 0);
+#else
+  vbox = gtk_grid_new();
+#endif
   gtk_container_add(GTK_CONTAINER(iWindow), vbox);
   gtk_widget_show(vbox);
 
   GtkWidget *menu_bar = gtk_menu_bar_new();
 
-  gtk_box_pack_start(GTK_BOX (vbox), menu_bar, FALSE, FALSE, 2);
+#if GTK_MAJOR_VERSION < 3
+  gtk_box_pack_start(GTK_BOX(vbox), menu_bar, FALSE, FALSE, 2);
+#else
+  gtk_grid_attach(GTK_GRID(vbox), menu_bar, 0, 0, 1, 1);
+#endif
   gtk_widget_show(menu_bar);
 
   for (int i = 0; i < 3; ++i) {
@@ -162,7 +171,12 @@ AppUi::AppUi()
   }
 
   Canvas *canvas = new Canvas(iWindow);
+#if GTK_MAJOR_VERSION < 3
   gtk_box_pack_end(GTK_BOX(vbox), canvas->window(), TRUE, TRUE, 2);
+#else
+  gtk_grid_attach_next_to(GTK_GRID(vbox), canvas->window(),
+			  menu_bar, GTK_POS_BOTTOM, 1, 1);
+#endif
   gtk_widget_show(canvas->window());
 
   iDoc = 0;
@@ -196,12 +210,32 @@ void AppUi::show()
 
 // --------------------------------------------------------------------
 
+void AppUi::load()
+{
+  GtkWidget *dlg =
+    gtk_file_chooser_dialog_new("Open file...",
+				GTK_WINDOW(iWindow),
+				GTK_FILE_CHOOSER_ACTION_OPEN,
+				"_Cancel", GTK_RESPONSE_REJECT,
+				"_Ok", GTK_RESPONSE_ACCEPT,
+				NULL);
+
+  int result = gtk_dialog_run(GTK_DIALOG(dlg));
+
+  if (result == GTK_RESPONSE_ACCEPT) {
+    char *s = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dlg));
+    load(s);
+    g_free(s);
+  }
+  gtk_widget_destroy(dlg);
+}
+
 void AppUi::cmd(int cmd)
 {
   ipeDebug("Command %d", cmd);
   switch (cmd) {
   case EOpen:
-    // load(szFileName);
+    load();
     break;
   case EQuit:
     gtk_main_quit();
@@ -266,21 +300,6 @@ bool AppUi::load(const char *fname)
   iPageNo = 0;
   iViewNo = 0;
 
-  /*
-  if (iDoc->countPages() > 1) {
-    int p = PageSelector::selectPageOrView(iDoc);
-    if (p >= 0)
-      iPageNo = p;
-    if (iDoc->page(iPageNo)->countViews() > 1) {
-      int v = PageSelector::selectPageOrView(iDoc, iPageNo);
-      if (v >= 0)
-	iViewNo = v;
-    }
-  }
-  */
-
-  iDoc->page(iPageNo)->setSelect(0, EPrimarySelected);
-
   iCanvas->setFontPool(iDoc->fontPool());
   iCanvas->setPage(iDoc->page(iPageNo), iPageNo, iViewNo, iDoc->cascade());
   iCanvas->setPan(Vector(300, 400));
@@ -349,14 +368,23 @@ void AppUi::nextView(int delta)
 
 // --------------------------------------------------------------------
 
+static void usage()
+{
+  fprintf(stderr, "Usage: ipeview <filename>\n");
+  exit(1);
+}
+
 int main(int argc, char *argv[])
 {
   Platform::initLib(IPELIB_VERSION);
 
   gtk_init(&argc, &argv);
 
+  if (argc != 2)
+    usage();
+
   AppUi *ui = new AppUi();
-  if (argc > 1 && !ui->load(argv[1]))
+  if (!ui->load(argv[1]))
     exit(2);
 
   ui->show();
